@@ -129,7 +129,7 @@ class IntegrationSigningTests(IntegrationTestHelpersMixin, TestCase):
             "--workspace",
             "debusine/System",
             "signer",
-            "Owners",
+            "Admins",
         )
         self.assertEqual(result.returncode, 0)
 
