@@ -16,6 +16,8 @@ sed "s,api-url = .*,api-url = https://$(hostname -f)/api," \
 # Generate a private key to be used to encrypt other private keys
 sudo -u debusine-signing debusine-signing generate_service_key "$debusine_signing_config_directory/0.key"
 
+systemctl reset-failed debusine-signing-migrate
+systemctl restart debusine-signing-migrate
 systemctl restart debusine-signing
 
 debusine_signing_token_file="$debusine_signing_config_directory/token"
