@@ -102,7 +102,7 @@ class ArtifactPlayground:
         if source:
             architectures.add("source")
             dsc = destdir / f"{src_name}_{version}.dsc"
-            src_files: list[Path] = [dsc]
+            src_files: list[Path] = []
             v_parts = version.split("-", 1)
             if len(v_parts) == 2:
                 src_files.append(
@@ -123,7 +123,7 @@ class ArtifactPlayground:
                 binaries=binaries,
                 version=version,
             )
-            files += src_files
+            files += [dsc] + src_files
         if binary:
             architectures.add(bin_architecture)
             for bin_name in binaries:
