@@ -58,37 +58,6 @@ class DebusineHistoricalTaskRunTests(TestCase):
                 task_type="does-not-exist", **kwargs
             )
 
-    def test_context_valid(self) -> None:
-        """Context fields without colons are allowed."""
-        for context in ("sid/amd64", None):
-            with self.subTest(context=context):
-                data_models.DebusineHistoricalTaskRun(
-                    task_type=TaskTypes.WORKER,
-                    task_name="noop",
-                    subject="subject",
-                    context=context,
-                    timestamp=0,
-                    work_request_id=1,
-                    result="success",
-                    runtime_statistics=data_models.RuntimeStatistics(
-                        duration=1
-                    ),
-                )
-
-    def test_context_invalid(self) -> None:
-        """Context fields with colons are forbidden."""
-        with self.assertRaisesRegex(ValueError, r"may not contain ':'"):
-            data_models.DebusineHistoricalTaskRun(
-                task_type=TaskTypes.WORKER,
-                task_name="noop",
-                subject="subject",
-                context="sid:amd64",
-                timestamp=0,
-                work_request_id=1,
-                result="success",
-                runtime_statistics=data_models.RuntimeStatistics(duration=1),
-            )
-
 
 class DebusineTaskConfigurationTests(TestCase):
     """Test the DebusineTaskConfiguration model."""
