@@ -14,42 +14,40 @@ import http
 import logging
 import math
 import os
-import re
 import shutil
 import signal
 import subprocess
 import sys
 import tempfile
 import types
-from collections.abc import Callable, Generator
+from collections.abc import Generator
 from contextlib import contextmanager
 from datetime import datetime
-from functools import partial
 from itertools import islice
 from pathlib import Path
 from typing import Any, NoReturn
 
 import yaml
 from dateutil.parser import isoparse
-from debian.deb822 import Changes, Dsc
 from requests.exceptions import RequestException
 from yaml import YAMLError
 
-from debusine.artifacts import (
-    BinaryPackage,
-    BinaryPackages,
-    LocalArtifact,
-    SourcePackage,
-    Upload,
-)
+from debusine.artifacts import LocalArtifact, Upload
 from debusine.assets import AssetCategory, asset_data_model
 from debusine.client import exceptions
-from debusine.client.client_utils import get_debian_package
+from debusine.client.client_utils import (
+    copy_file,
+    get_debian_package,
+    prepare_changes_for_upload,
+    prepare_deb_for_upload,
+    prepare_dsc_for_upload,
+)
 from debusine.client.config import ConfigHandler
 from debusine.client.debusine import Debusine
 from debusine.client.exceptions import DebusineError
 from debusine.client.models import (
     CreateWorkflowRequest,
+    FileResponse,
     RelationType,
     WorkRequestExternalDebsignRequest,
     WorkRequestRequest,
@@ -82,15 +80,15 @@ class Cli:
         """Parse argv and store results in self.args."""
         parser = argparse.ArgumentParser(
             prog='debusine',
-            description='Interacts with a debusine server.',
+            description='Interacts with a Debusine server.',
             formatter_class=argparse.ArgumentDefaultsHelpFormatter,
         )
 
         parser.add_argument(
             '--server',
             help=(
-                'Set server to be used (use configuration file default '
-                'if not specified)'
+                'Set server to be used, either by section name or as '
+                'FQDN/scope (use configuration file default if not specified)'
             ),
         )
 
@@ -192,6 +190,15 @@ class Cli:
             type=int,
             help="Work request id that needs a signature",
         )
+        provide_signature.add_argument(
+            "--local-file",
+            "-l",
+            type=Path,
+            help=(
+                "Path to the .changes file to sign, locally. "
+                "If not specified, it will be downloaded from the server."
+            ),
+        )
         provide_signature.add_argument(
             "extra_args",
             nargs="*",
@@ -408,16 +415,20 @@ class Cli:
             help="Path to the command to execute",
         )
 
+        subparsers.add_parser("setup", help="Guided setup of debusine-client")
+
         self.args = parser.parse_args(self._argv)
 
     def _build_debusine_object(self) -> Debusine:
         """Return the debusine object matching the command line parameters."""
         configuration = ConfigHandler(
-            server_name=self.args.server,
-            config_file_path=self.args.config_file,
+            server_name=self.args.server, config_file_path=self.args.config_file
         )
 
-        server_configuration = configuration.server_configuration()
+        try:
+            server_configuration = configuration.server_configuration()
+        except ValueError as exc:
+            self._fail(exc)
 
         logging_level = logging.WARNING if self.args.silent else logging.INFO
 
@@ -473,6 +484,20 @@ class Cli:
         signal.signal(signal.SIGTERM, self._exit)
 
         self._parse_args()
+
+        # Early dispatch for subcommands that do not need the Debusine object
+        # to operate
+        match getattr(self.args, 'sub-command'):
+            case "setup":
+                from debusine.client.setup import setup_server
+
+                setup_server(
+                    config_file_path=self.args.config_file,
+                    server=self.args.server,
+                    scope=self.args.scope,
+                )
+                return
+
         debusine = self._build_debusine_object()
         self._setup_http_logging()
 
@@ -496,7 +521,10 @@ class Cli:
                 )
             case "provide-signature":
                 self._provide_signature(
-                    debusine, self.args.work_request_id, self.args.extra_args
+                    debusine,
+                    self.args.work_request_id,
+                    self.args.local_file,
+                    self.args.extra_args,
                 )
             case "create-artifact":
                 if self.args.data is not None:
@@ -694,13 +722,39 @@ class Cli:
         with self._api_call_or_fail():
             debusine.work_request_retry(work_request_id)
 
+    def _fetch_local_file(
+        self, src: Path, dest: Path, artifact_file: FileResponse
+    ) -> None:
+        """Copy src to dest and verify that its hash matches artifact_file."""
+        assert src.exists()
+        hashes = copy_file(src, dest, artifact_file.checksums.keys())
+        if hashes["size"] != artifact_file.size:
+            self._fail(
+                f'"{src}" size mismatch (expected {artifact_file.size} bytes)'
+            )
+
+        for hash_name, expected_value in artifact_file.checksums.items():
+            if hashes[hash_name] != expected_value:
+                self._fail(
+                    f'"{src}" hash mismatch (expected {hash_name} '
+                    f'= {expected_value})'
+                )
+
     def _provide_signature_debsign(
         self,
         debusine: Debusine,
         work_request: WorkRequestResponse,
+        local_file: Path | None,
         debsign_args: list[str],
     ) -> None:
         """Provide a work request with an external signature using `debsign`."""
+        if local_file is not None:
+            if local_file.suffix != ".changes":
+                self._fail(
+                    f"--local-file {str(local_file)!r} is not a .changes file."
+                )
+            if not local_file.exists():
+                self._fail(f"--local-file {str(local_file)!r} does not exist.")
         # Get a version of the work request with its dynamic task data
         # resolved.
         with self._api_call_or_fail():
@@ -727,8 +781,15 @@ class Cli:
                     or name.endswith(".dsc")
                     or name.endswith(".buildinfo")
                 ):
-                    with self._api_call_or_fail():
-                        debusine.download_artifact_file(unsigned, name, path)
+                    if local_file:
+                        self._fetch_local_file(
+                            local_file.parent / name, path, file_response
+                        )
+                    else:
+                        with self._api_call_or_fail():
+                            debusine.download_artifact_file(
+                                unsigned, name, path
+                            )
             # Upload artifacts are guaranteed to have exactly one .changes
             # file.
             [changes_path] = [
@@ -737,7 +798,7 @@ class Cli:
 
             # Call debsign.
             subprocess.run(
-                ["debsign", changes_path.name, *debsign_args],
+                ["debsign", "--re-sign", changes_path.name, *debsign_args],
                 cwd=tmp,
                 check=True,
             )
@@ -761,7 +822,11 @@ class Cli:
                 )
 
     def _provide_signature(
-        self, debusine: Debusine, work_request_id: int, extra_args: list[str]
+        self,
+        debusine: Debusine,
+        work_request_id: int,
+        local_file: Path | None,
+        extra_args: list[str],
     ) -> None:
         """Provide a work request with an external signature."""
         # Find out what kind of work request we're dealing with.
@@ -770,7 +835,7 @@ class Cli:
         match (work_request.task_type, work_request.task_name):
             case "Wait", "externaldebsign":
                 self._provide_signature_debsign(
-                    debusine, work_request, extra_args
+                    debusine, work_request, local_file, extra_args
                 )
             case _:
                 self._fail(
@@ -985,92 +1050,6 @@ class Cli:
             with self._api_call_or_fail():
                 debusine.upload_files(artifact_created.id, files_to_upload)
 
-    def _prepare_changes_for_upload(
-        self, changes: Path
-    ) -> list[LocalArtifact[Any]]:
-        with changes.open("rb") as f:
-            parsed = Changes(f)
-        architectures = parsed["Architecture"].split()
-        artifact_factories: list[Callable[[], LocalArtifact[Any]]] = []
-        for architecture in architectures:
-            if architecture == "source":
-                dscs = [
-                    file["name"]
-                    for file in parsed["Files"]
-                    if file["name"].endswith(".dsc")
-                ]
-                if len(dscs) != 1:
-                    self._fail(
-                        f"Expecting exactly 1 .dsc in source package, "
-                        f"found {len(dscs)}"
-                    )
-                artifact_factories.append(
-                    partial(
-                        self._prepare_dsc_for_upload, changes.parent / dscs[0]
-                    )
-                )
-            else:
-                debs = [
-                    changes.parent / file["name"]
-                    for file in parsed["Files"]
-                    if file["name"].endswith(
-                        (
-                            f"_{architecture}.deb",
-                            f"_{architecture}.udeb",
-                        )
-                    )
-                ]
-                if len(debs) < 1:
-                    self._fail(
-                        "Expecting at least one .deb per arch in "
-                        "binary packages"
-                    )
-
-                srcpkg_name = parsed["Source"]
-                srcpkg_version = parsed["Version"]
-                m = re.match(
-                    r"^(?P<name>\S+) \((?P<version>\S+)\)$", srcpkg_name
-                )
-                if m:
-                    srcpkg_name = m.group("name")
-                    srcpkg_version = m.group("version")
-
-                # Create artifacts for each individual .deb.
-                for deb in debs:
-                    artifact_factories.append(
-                        partial(BinaryPackage.create, file=deb)
-                    )
-                # For convenience, also create a grouped artifact for all
-                # the .debs.
-                artifact_factories.append(
-                    partial(
-                        BinaryPackages.create,
-                        srcpkg_name=srcpkg_name,
-                        srcpkg_version=srcpkg_version,
-                        version=parsed["Version"],
-                        architecture=architecture,
-                        files=debs,
-                    )
-                )
-        artifacts: list[LocalArtifact[Any]] = [
-            Upload.create(changes_file=changes)
-        ] + [factory() for factory in artifact_factories]
-        return artifacts
-
-    def _prepare_dsc_for_upload(self, dsc: Path) -> SourcePackage:
-        files = [dsc]
-        directory = dsc.parent
-        with dsc.open() as f:
-            parsed = Dsc(f)
-            name = parsed["source"]
-            version = parsed["version"]
-            for file in parsed["files"]:
-                files.append(directory / file["name"])
-        return SourcePackage.create(name=name, version=version, files=files)
-
-    def _prepare_deb_for_upload(self, deb: Path) -> BinaryPackage:
-        return BinaryPackage.create(file=deb)
-
     def _import_debian_artifact(
         self,
         debusine: Debusine,
@@ -1084,11 +1063,11 @@ class Cli:
             with get_debian_package(upload) as path:
                 artifacts: list[LocalArtifact[Any]]
                 if path.suffix == ".changes":
-                    artifacts = self._prepare_changes_for_upload(path)
+                    artifacts = prepare_changes_for_upload(path)
                 elif path.suffix == ".dsc":
-                    artifacts = [self._prepare_dsc_for_upload(path)]
+                    artifacts = [prepare_dsc_for_upload(path)]
                 elif path.suffix in (".deb", ".ddeb", ".udeb"):
-                    artifacts = [self._prepare_deb_for_upload(path)]
+                    artifacts = [prepare_deb_for_upload(path)]
                 else:
                     raise ValueError(
                         "Only source packages (.dsc), binary packages (.deb), "
