@@ -311,10 +311,7 @@ class Debusine:
         """Return URL to upload a file in the artifact."""
         quoted_path = urllib.parse.quote(path_in_artifact)
 
-        return (
-            f"{self.base_api_url}/{self.API_VERSION}/artifact/"
-            f"{artifact_id}/files/{quoted_path}/"
-        )
+        return f"{self.api_url}/artifact/{artifact_id}/files/{quoted_path}/"
 
     def upload_files(
         self,
@@ -335,7 +332,7 @@ class Debusine:
         # harmlessly; this just helps with type-checking.
         assert self.token is not None
 
-        file_uploader = FileUploader(self.token)
+        file_uploader = FileUploader(self.token, logger=self._logger)
 
         for artifact_path, local_path in upload_files.items():
             if not local_path.is_absolute():
@@ -486,6 +483,9 @@ class Debusine:
         if not tarball and not self._should_download_as_tarball(artifact):
             self._logger.info("Downloading artifact files into %s", destination)
             for name, file in artifact.files.items():
+                self._logger.debug(
+                    "Downloading artifact file %s from %s", name, file.url
+                )
                 path = destination / name
                 if path.parent != destination:
                     if not path.is_relative_to(destination):
