@@ -78,7 +78,10 @@ def create_workflow(
         workspace=workspace,
         task_data=task_data,
     )
-    workflow_created = debusine.workflow_create(workflow)
+    try:
+        workflow_created = debusine.workflow_create(workflow)
+    except Exception as e:
+        raise HookException(str(e)) from e
     logger.info(
         "Created workflow: %s",
         # TODO: Ideally we wouldn't have to hardcode the URL structure here.
