@@ -617,6 +617,19 @@ class DebusineTests(TestCase):
 
         for url in urls:
             self.assertTrue(responses.assert_call_count(url, 1))
+        self.assertEqual(
+            self.logger.info.mock_calls,
+            [
+                call(
+                    "Uploading %s to %s",
+                    path,
+                    self.debusine._url_for_file_in_artifact(
+                        artifact_id, path.name
+                    ),
+                )
+                for path in file_paths
+            ],
+        )
 
     def test_upload_files_invalid_base_directory(self) -> None:
         """Base directory is None but there is a relative file path."""
