@@ -10,6 +10,7 @@
 """Unit tests for the FileUploader class."""
 
 import io
+import logging
 import mmap
 from collections.abc import Callable
 from typing import Any
@@ -111,7 +112,9 @@ class FileUploaderTests(TestCase):
         self.url = "https://localhost:8011/1.0/artifact/2/README.md"
         self.token = "21efb7587bd10b8eb2c56b4280b4c4a92a9"
 
-        self.file_uploader = FileUploader(self.token)
+        self.file_uploader = FileUploader(
+            self.token, logger=logging.getLogger(__name__)
+        )
 
         self.file_content = b"test-content"
         self.file_path = self.create_temporary_file(contents=self.file_content)
@@ -412,6 +415,17 @@ class FileUploaderTests(TestCase):
 
         responses.assert_call_count(self.url, url_called_times)
 
+    @responses.activate
+    def test_file_upload_logs(self) -> None:
+        """The uploader logs a message."""
+        responses.add(responses.PUT, self.url)
+        with self.assertLogsContains(
+            f"Uploading {self.file_path} to {self.url}",
+            logger=__name__,
+            level=logging.INFO,
+        ):
+            self.file_uploader.upload(self.file_path, self.url)
+
     def test_upload_file_put_fails(self) -> None:
         """FileUpload._upload_file() requests.put() raise exception."""
         request = mock.create_autospec(
