@@ -1,7 +1,9 @@
 import hashlib
 
 from django.db import migrations
+from django.db.backends.base.schema import BaseDatabaseSchemaEditor
 from django.db.migrations.exceptions import IrreversibleError
+from django.db.migrations.state import StateApps
 
 
 def _generate_hash(secret: str) -> str:
@@ -9,7 +11,9 @@ def _generate_hash(secret: str) -> str:
     return hashlib.sha256(secret.encode()).hexdigest()
 
 
-def update_hash(apps, schema_editor):
+def update_hash(
+    apps: StateApps, schema_editor: BaseDatabaseSchemaEditor
+) -> None:
     Token = apps.get_model("db", "Token")
 
     for token in Token.objects.all():
@@ -17,7 +21,9 @@ def update_hash(apps, schema_editor):
         token.save()
 
 
-def revert_hash(apps, schema_editor):
+def revert_hash(
+    apps: StateApps, schema_editor: BaseDatabaseSchemaEditor
+) -> None:
     Token = apps.get_model("db", "Token")
 
     if Token.objects.all():
