@@ -16,15 +16,14 @@ from django.db import migrations
 from django.db.backends.base.schema import BaseDatabaseSchemaEditor
 from django.db.migrations.state import StateApps
 
-from debusine.db.models import DEFAULT_WORKSPACE_NAME
-
 
 def set_default_workspace_public(
     apps: StateApps, schema_editor: BaseDatabaseSchemaEditor, public: bool
 ) -> None:
     """Set the public field on the default workspace."""
     default_workspace = apps.get_model("db", "Workspace").objects.get(
-        scope__name=settings.DEBUSINE_DEFAULT_SCOPE, name=DEFAULT_WORKSPACE_NAME
+        scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+        name=settings.DEBUSINE_DEFAULT_WORKSPACE,
     )
     default_workspace.public = public
     default_workspace.save()
