@@ -15,7 +15,6 @@ from django.db.backends.base.schema import BaseDatabaseSchemaEditor
 from django.db.migrations.state import StateApps
 
 from debusine.artifacts.models import CollectionCategory
-from debusine.db.models import DEFAULT_WORKSPACE_NAME
 
 # We only create the collections that were defined as singletons at the time
 # of this migration.  If we add new singleton categories and want to create
@@ -33,7 +32,8 @@ def create_singleton_collections(
     Workspace = apps.get_model("db", "Workspace")
     Collection = apps.get_model("db", "Collection")
     default_workspace = Workspace.objects.get(
-        scope__name=settings.DEBUSINE_DEFAULT_SCOPE, name=DEFAULT_WORKSPACE_NAME
+        scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+        name=settings.DEBUSINE_DEFAULT_WORKSPACE,
     )
     for category in singleton_collection_categories:
         Collection.objects.create(
@@ -48,7 +48,8 @@ def remove_singleton_collections(
     Workspace = apps.get_model("db", "Workspace")
     Collection = apps.get_model("db", "Collection")
     default_workspace = Workspace.objects.get(
-        scope__name=settings.DEBUSINE_DEFAULT_SCOPE, name=DEFAULT_WORKSPACE_NAME
+        scope__name=settings.DEBUSINE_DEFAULT_SCOPE,
+        name=settings.DEBUSINE_DEFAULT_WORKSPACE,
     )
     for category in singleton_collection_categories:
         Collection.objects.filter(
