@@ -44,7 +44,12 @@ from debusine.db.models.work_requests import (
     WorkRequest,
     WorkflowTemplate,
 )
-from debusine.db.models.worker_pools import ScopeWorkerPool, WorkerPool
+from debusine.db.models.worker_pools import (
+    ScopeWorkerPool,
+    WorkerPool,
+    WorkerPoolStatistics,
+    WorkerPoolTaskExecutionStatistics,
+)
 from debusine.db.models.workers import Worker
 from debusine.db.models.workspaces import Workspace, default_workspace
 
@@ -74,6 +79,8 @@ __all__ = [
     "User",
     "Worker",
     "WorkerPool",
+    "WorkerPoolStatistics",
+    "WorkerPoolTaskExecutionStatistics",
     "WorkflowTemplate",
     "WorkRequest",
     "Workspace",
