@@ -9,11 +9,12 @@
 
 """Data models for db scopes."""
 
+import enum
 import re
-from collections.abc import Collection
 from typing import (
     Any,
     Generic,
+    Self,
     TYPE_CHECKING,
     TypeAlias,
     TypeVar,
@@ -23,7 +24,7 @@ from typing import (
 )
 
 import pgtrigger
-from django.core.exceptions import ValidationError
+from django.core.exceptions import ImproperlyConfigured, ValidationError
 from django.db import models
 from django.db.models import CheckConstraint, F, Q, QuerySet, UniqueConstraint
 
@@ -31,9 +32,9 @@ from debusine.db.context import context
 from debusine.db.models import permissions
 from debusine.db.models.files import File, FileStore
 from debusine.db.models.permissions import (
+    AllowWorkers,
     PartialCheckResult,
     PermissionUser,
-    ROLES,
     permission_check,
     permission_filter,
 )
@@ -88,24 +89,56 @@ def validate_scope_name(value: str) -> None:
 A = TypeVar("A")
 
 
+class ScopeRoleBase(permissions.RoleBase):
+    """Scope role implementation."""
+
+    implied_by_scope_roles: frozenset["ScopeRoles"]
+
+    def _setup(self) -> None:
+        """Set up implications for a newly constructed role."""
+        for i in self.implied_by:
+            raise ImproperlyConfigured(
+                f"Scope roles do not support implications by {type(i)}"
+            )
+        self.implied_by_scope_roles = frozenset((cast(ScopeRoles, self),))
+
+    def q(self, user: "User") -> Q:
+        """Return a Q expression to select scopes with this role."""
+        return Q(
+            roles__group__users=user,
+            roles__role__in=self.implied_by_scope_roles,
+        )
+
+
+class ScopeRoles(permissions.Roles, ScopeRoleBase, enum.ReprEnum):
+    """Available roles for a Scope."""
+
+    OWNER = "owner"
+
+
+ScopeRoles.setup()
+
+
 class ScopeQuerySet(QuerySet["Scope", A], Generic[A]):
     """Custom QuerySet for Scope."""
 
-    @permission_filter
-    def can_display(
-        self, user: PermissionUser  # noqa: U100
-    ) -> "ScopeQuerySet[A]":
+    def with_role(self, user: "User", role: ScopeRoles) -> Self:
+        """Keep only resources where the user has the given role."""
+        return self.filter(role.q(user))
+
+    @permission_filter(workers=AllowWorkers.ALWAYS)
+    def can_display(self, user: PermissionUser) -> Self:  # noqa: U100
         """Keep only Scopes that can be displayed."""
         assert user is not None  # Enforced by decorator
         return self
 
-    @permission_filter
-    def can_create_workspace(self, user: PermissionUser) -> "ScopeQuerySet[A]":
+    @permission_filter(workers=AllowWorkers.NEVER)
+    def can_create_workspace(self, user: PermissionUser) -> Self:
         """Keep only Scopes where the user can create workspaces."""
         assert user is not None  # Enforced by decorator
         if not user.is_authenticated:
             return self.none()
-        return self.filter(ROLES(user, Scope.Roles.OWNER))
+        return self.with_role(user, Scope.Roles.OWNER)
 
 
 class ScopeManager(models.Manager["Scope"]):
@@ -120,12 +153,6 @@ class ScopeManager(models.Manager["Scope"]):
         return ScopeQuerySet(self.model, using=self._db)
 
 
-class ScopeRoles(permissions.Roles):
-    """Available roles for a Scope."""
-
-    OWNER = "owner", "Owner"
-
-
 class Scope(models.Model):
     """
     Scope model.
@@ -165,13 +192,19 @@ class Scope(models.Model):
         """Return basic information of Scope."""
         return self.name
 
-    @permission_check("{user} cannot display scope {resource}")
+    @permission_check(
+        "{user} cannot display scope {resource}",
+        workers=AllowWorkers.ALWAYS,
+    )
     def can_display(self, user: PermissionUser) -> bool:  # noqa: U100
         """Check if the scope can be displayed."""
         assert user is not None  # enforced by decorator
         return True
 
-    @permission_check("{user} cannot create workspaces in {resource}")
+    @permission_check(
+        "{user} cannot create workspaces in {resource}",
+        workers=AllowWorkers.NEVER,
+    )
     def can_create_workspace(self, user: PermissionUser) -> bool:
         """Check if the user can create workspaces in this scope."""
         assert user is not None  # enforced by decorator
@@ -193,10 +226,10 @@ class Scope(models.Model):
         )
 
     def context_has_role(
-        self, user: "User", roles: ScopeRoles | Collection[ScopeRoles]
+        self, user: "User", role: ScopeRoles
     ) -> PartialCheckResult:
         """
-        Check user roles in the current context.
+        Check the context to see if the user has a role on this Scope.
 
         :returns:
           * ALLOW if the context has enough information to determine that the
@@ -205,16 +238,12 @@ class Scope(models.Model):
             user does not have any of the given roles
           * PASS if the context does not have enough information to decide
         """
-        if not roles:
-            raise ValueError("context_has_role needs at least one role")
         if context.user != user or context.scope != self:
             return PartialCheckResult.PASS
-        if isinstance(roles, ScopeRoles):
-            roles = (roles,)
-        for role in roles:
-            if role in context.scope_roles:
-                return PartialCheckResult.ALLOW
-        return PartialCheckResult.DENY
+        if role.implied_by_scope_roles.isdisjoint(context.scope_roles):
+            return PartialCheckResult.DENY
+        else:
+            return PartialCheckResult.ALLOW
 
     # See https://github.com/typeddjango/django-stubs/issues/1047 for the typing
     def get_roles(
