@@ -183,6 +183,26 @@ class CollectionTests(TestCase):
             ),
         )
 
+    def test_get_absolute_url_search(self) -> None:
+        """Test get_absolute_url_search results."""
+        name = "Environments"
+        category = CollectionCategory.ENVIRONMENTS
+        collection = Collection.objects.create(
+            name=name, category=category, workspace=self.scenario.workspace
+        )
+
+        self.assertEqual(
+            collection.get_absolute_url_search(),
+            reverse(
+                "workspaces:collections:search",
+                kwargs={
+                    "wname": self.scenario.workspace.name,
+                    "ccat": category,
+                    "cname": name,
+                },
+            ),
+        )
+
     def test_constraint_name_category_workspace(self) -> None:
         """Raise integrity error: name, category, workspace must be unique."""
         name = "Environments"
