@@ -9,17 +9,26 @@
 
 """Unit tests for the worker pool models."""
 
+import logging
+from datetime import timedelta
 from unittest.mock import PropertyMock, patch
 
 from django.core.exceptions import ValidationError
 from django.db import IntegrityError
+from django.utils import timezone
 
 from debusine.assets import (
     HetznerProviderAccountConfiguration,
     HetznerProviderAccountCredentials,
     HetznerProviderAccountData,
 )
-from debusine.db.models import ScopeWorkerPool, Worker, WorkerPool
+from debusine.db.models import (
+    ScopeWorkerPool,
+    WorkRequest,
+    Worker,
+    WorkerPool,
+    WorkerPoolStatistics,
+)
 from debusine.server.worker_pools import (
     AWSEC2WorkerPoolSpecification,
     DummyWorkerPool,
@@ -54,6 +63,14 @@ class WorkerPoolTests(TestCase):
             "Id: 42 Name: foo",
         )
 
+    def test_get_absolute_url(self) -> None:
+        """Test the get_absolute_url method."""
+        worker_pool = self.playground.create_worker_pool()
+        self.assertEqual(
+            worker_pool.get_absolute_url(),
+            f"/-/status/worker-pools/{worker_pool.name}/",
+        )
+
     def test_limits_model(self) -> None:
         limits = WorkerPoolLimits(max_active_instances=10)
         worker_pool = WorkerPool(limits=limits.dict())
@@ -243,6 +260,99 @@ class WorkerPoolTests(TestCase):
         # The provider should have done this on termination:
         self.assertIsNone(worker.instance_created_at)
 
+    def test_terminate_worker_records_statistics(self) -> None:
+        """Terminating a worker records running time statistics."""
+        worker_pool = self.playground.create_worker_pool()
+        worker = self.playground.create_worker(worker_pool=worker_pool)
+        worker.instance_created_at = timezone.now() - timedelta(minutes=1)
+        worker.save()
+
+        worker_pool.terminate_worker(worker)
+
+        self.assertQuerySetEqual(
+            WorkerPoolStatistics.objects.filter(
+                worker_pool=worker_pool
+            ).values_list("worker_pool", "worker", "runtime"),
+            [(worker_pool.id, worker.id, 60)],
+        )
+        worker.refresh_from_db()
+        self.assertIsNone(worker.instance_created_at)
+
+    def test_terminate_worker_no_instance_created_at_no_statistics(
+        self,
+    ) -> None:
+        """If instance_created_at is null, terminate_worker skips statistics."""
+        worker_pool = self.playground.create_worker_pool()
+        worker = self.playground.create_worker(worker_pool=worker_pool)
+        worker.instance_created_at = None
+        worker.save()
+
+        worker_pool.terminate_worker(worker)
+
+        self.assertFalse(
+            WorkerPoolStatistics.objects.filter(
+                worker_pool=worker_pool
+            ).exists()
+        )
+
+    def test_terminate_worker_retries_running_work_request(self) -> None:
+        """Terminate a worker retries a running work request."""
+        worker_pool = self.playground.create_worker_pool()
+        worker = self.playground.create_worker(worker_pool=worker_pool)
+        work_request_pending = self.playground.create_work_request()
+        work_request_running = self.playground.create_work_request()
+        work_request_running.assign_worker(worker)
+        work_request_running.mark_running()
+
+        with self.assertNoLogs(
+            logger="debusine.db.models.worker_pools", level=logging.DEBUG
+        ):
+            worker_pool.terminate_worker(worker)
+
+        # The running work request was retried, though not yet assigned to a
+        # worker.
+        work_request_running.refresh_from_db()
+        self.assertEqual(
+            work_request_running.status, WorkRequest.Statuses.ABORTED
+        )
+        self.assertTrue(hasattr(work_request_running, "superseded"))
+        self.assertIsNone(work_request_running.superseded.worker)
+        self.assertEqual(
+            work_request_running.superseded.workflow_data.retry_count, 1
+        )
+
+        # The pending work request was left alone.
+        work_request_pending.refresh_from_db()
+        self.assertEqual(
+            work_request_pending.status, WorkRequest.Statuses.PENDING
+        )
+        self.assertFalse(hasattr(work_request_pending, "superseded"))
+
+    def test_terminate_worker_cannot_retry_running_work_request(self) -> None:
+        """If a running work request cannot be retried, it is logged."""
+        worker_pool = self.playground.create_worker_pool()
+        worker = self.playground.create_worker(worker_pool=worker_pool)
+        work_request_running = self.playground.create_work_request()
+        work_request_running.assign_worker(worker)
+        work_request_running.task_data = {"invalid": True}
+        work_request_running.save()
+        work_request_running.mark_running()
+
+        with self.assertLogsContains(
+            "Cannot retry previously-running work request: "
+            "Task dependencies cannot be satisfied",
+            logger="debusine.db.models.worker_pools",
+            level=logging.DEBUG,
+        ):
+            worker_pool.terminate_worker(worker)
+
+        # The running work request was aborted but could not be retried.
+        work_request_running.refresh_from_db()
+        self.assertEqual(
+            work_request_running.status, WorkRequest.Statuses.ABORTED
+        )
+        self.assertFalse(hasattr(work_request_running, "superseded"))
+
     def test_clean_valid(self) -> None:
         worker_pool = self.playground.create_worker_pool()
         worker_pool.clean()
