@@ -9,20 +9,23 @@
 
 """Unit tests for the workspace models."""
 import datetime
-from functools import partial
 from typing import ClassVar
 from unittest import mock
 
 from django.conf import settings
 from django.contrib.auth.models import AnonymousUser
-from django.core.exceptions import PermissionDenied, ValidationError
+from django.core.exceptions import (
+    ImproperlyConfigured,
+    PermissionDenied,
+    ValidationError,
+)
 from django.db import IntegrityError, transaction
+from django.db.models import Q
 from django.utils import timezone
 
 from debusine.artifacts.models import CollectionCategory
 from debusine.db.context import ContextConsistencyError, context
 from debusine.db.models import (
-    Artifact,
     Collection,
     FileInArtifact,
     FileInStore,
@@ -37,9 +40,10 @@ from debusine.db.models.auth import User
 from debusine.db.models.permissions import PartialCheckResult
 from debusine.db.models.workspaces import (
     DeleteWorkspaces,
-    WORKSPACE_ROLES,
     WorkspaceChain,
     WorkspaceRole,
+    WorkspaceRoleBase,
+    WorkspaceRoles,
     is_valid_workspace_name,
 )
 from debusine.db.playground import scenarios
@@ -51,6 +55,76 @@ from debusine.test.django import (
 )
 
 
+class WorkspaceRolesTests(TestCase):
+    """Tests for the WorkspaceRoles class."""
+
+    scenario = scenarios.DefaultContext()
+
+    def test_init(self) -> None:
+        r = WorkspaceRoleBase("test")
+        r._setup()
+        self.assertEqual(r.implied_by_scope_roles, frozenset())
+        self.assertEqual(r.implied_by_workspace_roles, frozenset([r]))
+
+    def test_init_invalid_implication(self) -> None:
+        r = WorkspaceRoleBase("test", implied_by=[Q()])
+        with self.assertRaisesRegex(
+            ImproperlyConfigured,
+            r"Workspace roles do not support implications"
+            r" by <class 'django.db.models.query_utils.Q'>",
+        ):
+            r._setup()
+
+    def test_q_no_scope_implications(self) -> None:
+        """Test generating a query with no scope implications."""
+        user = self.scenario.user
+        self.playground.create_group_role(
+            self.scenario.scope, Scope.Roles.OWNER, users=[user]
+        )
+
+        r = WorkspaceRoleBase("test")
+        r._setup()
+        self.assertQuerySetEqual(Workspace.objects.filter(r.q(user)), [])
+
+        # Assign r as a role
+        with context.disable_permission_checks():
+            group = Group.objects.create(scope=self.scenario.scope, name="test")
+            group.users.add(user)
+            Workspace.objects.get_roles_model().objects.create(
+                group=group, resource=self.scenario.workspace, role=r
+            )
+        self.assertQuerySetEqual(
+            Workspace.objects.filter(r.q(user)), [self.scenario.workspace]
+        )
+
+    def test_choices(self) -> None:
+        self.assertEqual(
+            WorkspaceRoles.choices,
+            [("owner", "Owner"), ("contributor", "Contributor")],
+        )
+
+    def test_implications(self) -> None:
+        self.assertEqual(Workspace.Roles.OWNER.label, "Owner")
+        self.assertEqual(
+            Workspace.Roles.OWNER.implied_by_scope_roles,
+            frozenset([Scope.Roles.OWNER]),
+        )
+        self.assertEqual(
+            Workspace.Roles.OWNER.implied_by_workspace_roles,
+            frozenset([Workspace.Roles.OWNER]),
+        )
+
+        self.assertEqual(Workspace.Roles.CONTRIBUTOR.label, "Contributor")
+        self.assertEqual(
+            Workspace.Roles.CONTRIBUTOR.implied_by_scope_roles,
+            frozenset([Scope.Roles.OWNER]),
+        )
+        self.assertEqual(
+            Workspace.Roles.CONTRIBUTOR.implied_by_workspace_roles,
+            frozenset([Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR]),
+        )
+
+
 class WorkspaceManagerTests(TestCase):
     """Tests for the WorkspaceManager class."""
 
@@ -254,14 +328,6 @@ class WorkspaceTests(TestCase):
                 workspace, name=name, category=category, user=user
             )
 
-    def assertAllow(self, value: PartialCheckResult) -> None:
-        """Check that the value is ALLOW."""
-        self.assertEqual(value, PartialCheckResult.ALLOW)
-
-    def assertDeny(self, value: PartialCheckResult) -> None:
-        """Check that the value is DENY."""
-        self.assertEqual(value, PartialCheckResult.DENY)
-
     def assertPass(self, value: PartialCheckResult) -> None:
         """Check that the value is PASS."""
         self.assertEqual(value, PartialCheckResult.PASS)
@@ -358,98 +424,6 @@ class WorkspaceTests(TestCase):
         ):
             Workspace.objects.create(name="test", scope=scope1)
 
-    def test_workspace_roles_empty(self) -> None:
-        """Test the WORKSPACE_ROLES query filter with no args."""
-        with self.assertRaisesRegex(
-            ValueError, r"at least one of scope or workspace needs to be set"
-        ):
-            WORKSPACE_ROLES(self.scenario.user)
-
-    def test_workspace_roles(self) -> None:
-        """Test the WORKSPACE_ROLES query filter."""
-        scope_owner = self.playground.create_user("scope_owner")
-        self.playground.create_group_role(
-            self.scenario.scope, Scope.Roles.OWNER, users=[scope_owner]
-        )
-        workspace_owner = self.playground.create_user("workspace_owner")
-        self.playground.create_group_role(
-            self.scenario.workspace,
-            Workspace.Roles.OWNER,
-            users=[scope_owner, workspace_owner],
-        )
-
-        owns_scope = dict(scope=[Scope.Roles.OWNER])
-        owns_workspace = dict(workspace=[Workspace.Roles.OWNER])
-        owns_both = dict(
-            scope=[Scope.Roles.OWNER], workspace=[Workspace.Roles.OWNER]
-        )
-        for user, kwargs, expected in (
-            (self.scenario.user, owns_scope, []),
-            (self.scenario.user, owns_workspace, []),
-            (self.scenario.user, owns_both, []),
-            (
-                scope_owner,
-                owns_scope,
-                [self.scenario.workspace, self.a, self.b, self.c, self.d],
-            ),
-            (scope_owner, owns_workspace, [self.scenario.workspace]),
-            (
-                scope_owner,
-                owns_both,
-                [self.scenario.workspace, self.a, self.b, self.c, self.d],
-            ),
-            (workspace_owner, owns_scope, []),
-            (workspace_owner, owns_workspace, [self.scenario.workspace]),
-            (workspace_owner, owns_both, [self.scenario.workspace]),
-        ):
-            with self.subTest(user=user, kwargs=kwargs):
-                self.assertQuerySetEqual(
-                    Workspace.objects.filter(
-                        WORKSPACE_ROLES(user, **kwargs)
-                    ).distinct(),
-                    expected,
-                    ordered=False,
-                )
-
-    def test_workspace_roles_member(self) -> None:
-        """Test the WORKSPACE_ROLES query filter member argument."""
-        scope_owner = self.playground.create_user("scope_owner")
-        self.playground.create_group_role(
-            self.scenario.scope, Scope.Roles.OWNER, users=[scope_owner]
-        )
-        workspace_owner = self.playground.create_user("workspace_owner")
-        self.playground.create_group_role(
-            self.scenario.workspace,
-            Workspace.Roles.OWNER,
-            users=[scope_owner, workspace_owner],
-        )
-        artifact, _ = self.playground.create_artifact()
-
-        owns_scope = dict(scope=[Scope.Roles.OWNER])
-        owns_workspace = dict(workspace=[Workspace.Roles.OWNER])
-        owns_both = dict(
-            scope=[Scope.Roles.OWNER], workspace=[Workspace.Roles.OWNER]
-        )
-
-        for user, kwargs, expected in (
-            (self.scenario.user, owns_scope, []),
-            (self.scenario.user, owns_workspace, []),
-            (self.scenario.user, owns_both, []),
-            (scope_owner, owns_scope, [artifact]),
-            (scope_owner, owns_workspace, [artifact]),
-            (scope_owner, owns_both, [artifact]),
-            (workspace_owner, owns_scope, []),
-            (workspace_owner, owns_workspace, [artifact]),
-            (workspace_owner, owns_both, [artifact]),
-        ):
-            with self.subTest(user=user, kwargs=kwargs):
-                self.assertQuerySetEqual(
-                    Artifact.objects.filter(
-                        WORKSPACE_ROLES(user, member="workspace", **kwargs)
-                    ).distinct(),
-                    expected,
-                )
-
     def test_context_has_role_no_context(self) -> None:
         """Test context_has_role with no context set."""
         self.assertPass(
@@ -496,82 +470,67 @@ class WorkspaceTests(TestCase):
             )
         )
 
-        # Enough information to decide about scope roles
-        self.assertAllow(
-            workspace.context_has_role(
-                self.scenario.user, scope_roles=Scope.Roles.OWNER
-            )
-        )
-
-    def test_context_has_role_no_roles(self) -> None:
-        """Test context_has_role with no roles."""
-        with self.assertRaisesRegex(
-            ValueError,
-            r"context_has_role needs at least one workspace or scope role",
-        ):
-            self.scenario.workspace.context_has_role(self.scenario.user, ())
-
     def test_context_has_role(self) -> None:
         """Test context_has_role."""
-        user = self.scenario.user
-        workspace = self.scenario.workspace
-
-        # Shortcut for more concise tests
-        f = partial(workspace.context_has_role, user)
-
-        with context.local():
-            self.scenario.set_current()
-            self.assertDeny(f(Workspace.Roles.OWNER))
-            self.assertDeny(f([Workspace.Roles.OWNER]))
-            self.assertDeny(f(scope_roles=Scope.Roles.OWNER))
-            self.assertDeny(f(scope_roles=[Scope.Roles.OWNER]))
-
-        # User becomes scope owner
-        scope_group = self.playground.create_group_role(
-            self.scenario.scope, Scope.Roles.OWNER, users=[self.scenario.user]
-        )
-        with context.local():
-            self.scenario.set_current()
-            self.assertDeny(f(Workspace.Roles.OWNER))
-            self.assertDeny(f([Workspace.Roles.OWNER]))
-            self.assertAllow(f(scope_roles=Scope.Roles.OWNER))
-            self.assertAllow(f(scope_roles=[Scope.Roles.OWNER]))
-            self.assertAllow(
-                f(Workspace.Roles.OWNER, scope_roles=Scope.Roles.OWNER)
-            )
-            self.assertAllow(
-                f([Workspace.Roles.OWNER], scope_roles=[Scope.Roles.OWNER])
-            )
-
-        # User becomes workspace owner
-        self.playground.create_group_role(
-            workspace, Workspace.Roles.OWNER, users=[self.scenario.user]
-        )
-        with context.local():
-            self.scenario.set_current()
-            self.assertAllow(f(Workspace.Roles.OWNER))
-            self.assertAllow(f([Workspace.Roles.OWNER]))
-            self.assertAllow(
-                f(Workspace.Roles.OWNER, scope_roles=Scope.Roles.OWNER)
-            )
-            self.assertAllow(
-                f([Workspace.Roles.OWNER], scope_roles=[Scope.Roles.OWNER])
-            )
-
-        # Use is not scope owner anymore, but remains workspace owner
-        scope_group.delete()
-        with context.local():
-            self.scenario.set_current()
-            self.assertAllow(f(Workspace.Roles.OWNER))
-            self.assertAllow(f([Workspace.Roles.OWNER]))
-            self.assertDeny(f(scope_roles=Scope.Roles.OWNER))
-            self.assertDeny(f(scope_roles=[Scope.Roles.OWNER]))
-            self.assertAllow(
-                f(Workspace.Roles.OWNER, scope_roles=Scope.Roles.OWNER)
-            )
-            self.assertAllow(
-                f([Workspace.Roles.OWNER], scope_roles=[Scope.Roles.OWNER])
-            )
+        for scope_roles, workspace_roles, roles, expected in (
+            (
+                [],
+                [],
+                [Workspace.Roles.CONTRIBUTOR, Workspace.Roles.OWNER],
+                PartialCheckResult.DENY,
+            ),
+            (
+                [Scope.Roles.OWNER],
+                [],
+                [Workspace.Roles.CONTRIBUTOR, Workspace.Roles.OWNER],
+                PartialCheckResult.ALLOW,
+            ),
+            (
+                [],
+                [Workspace.Roles.OWNER],
+                [Workspace.Roles.CONTRIBUTOR, Workspace.Roles.OWNER],
+                PartialCheckResult.ALLOW,
+            ),
+            (
+                [],
+                [Workspace.Roles.CONTRIBUTOR],
+                [Workspace.Roles.CONTRIBUTOR],
+                PartialCheckResult.ALLOW,
+            ),
+            (
+                [],
+                [Workspace.Roles.CONTRIBUTOR],
+                [Workspace.Roles.OWNER],
+                PartialCheckResult.DENY,
+            ),
+        ):
+            for role in roles:
+                with (
+                    self.subTest(
+                        scope_roles=scope_roles,
+                        workspace_roles=workspace_roles,
+                        role=role,
+                    ),
+                    context.local(),
+                ):
+                    Group.objects.all().delete()
+                    for sr in scope_roles:
+                        self.playground.create_group_role(
+                            self.scenario.scope, sr, users=[self.scenario.user]
+                        )
+                    for wr in workspace_roles:
+                        self.playground.create_group_role(
+                            self.scenario.workspace,
+                            wr,
+                            users=[self.scenario.user],
+                        )
+                    self.scenario.set_current()
+                    self.assertEqual(
+                        self.scenario.workspace.context_has_role(
+                            self.scenario.user, role
+                        ),
+                        expected,
+                    )
 
     @context.disable_permission_checks()
     def test_file_needs_upload_initial_file_store(self) -> None:
@@ -1539,13 +1498,12 @@ class WorkspaceDeleteTests(TestCase):
         self.assertQuerySetEqual(
             operation.workflow_templates, [self.scenario.template_sbuild]
         )
-        self.assertEqual(operation.artifacts.count(), 13)
-        self.assertEqual(operation.work_requests.count(), 11)
+        self.assertEqual(operation.artifacts.count(), 17)
+        self.assertEqual(operation.work_requests.count(), 12)
         self.assertEqual(operation.collection_items.count(), 7)
         self.assertEqual(operation.collections.count(), 6)
-        self.assertEqual(operation.file_in_artifacts.count(), 16)
-        self.assertEqual(operation.artifact_relations.count(), 12)
-        self.assertEqual(operation.artifacts.count(), 13)
+        self.assertEqual(operation.file_in_artifacts.count(), 18)
+        self.assertEqual(operation.artifact_relations.count(), 14)
         self.assertEqual(operation.asset_usages.count(), 1)
         self.assertEqual(operation.assets.count(), 1)
         self.assertEqual(operation.workspace_roles.count(), 1)
