@@ -64,15 +64,15 @@ from debusine.client.models import (
 from debusine.db.context import context
 from debusine.db.models import Collection, CollectionItem
 from debusine.db.models.permissions import (
+    AllowWorkers,
     PartialCheckResult,
     PermissionUser,
     enforce,
     permission_check,
     permission_filter,
 )
-from debusine.db.models.scopes import Scope
 from debusine.db.models.worker_pools import WorkerPoolTaskExecutionStatistics
-from debusine.db.models.workspaces import WORKSPACE_ROLES, Workspace
+from debusine.db.models.workspaces import Workspace
 from debusine.server import notifications
 from debusine.tasks import BaseTask, TaskConfigError
 from debusine.tasks.models import (
@@ -151,7 +151,7 @@ class WorkflowTemplateQuerySet(QuerySet["WorkflowTemplate", A], Generic[A]):
         """Filter to work requests in the current workspace."""
         return self.filter(workspace=context.require_workspace())
 
-    @permission_filter
+    @permission_filter(workers=AllowWorkers.ALWAYS)
     def can_display(
         self, user: PermissionUser
     ) -> "WorkflowTemplateQuerySet[A]":
@@ -159,19 +159,16 @@ class WorkflowTemplateQuerySet(QuerySet["WorkflowTemplate", A], Generic[A]):
         # Delegate to workspace can_display check
         return self.filter(workspace__in=Workspace.objects.can_display(user))
 
-    @permission_filter
+    @permission_filter(workers=AllowWorkers.NEVER)
     def can_run(self, user: PermissionUser) -> "WorkflowTemplateQuerySet[A]":
         """Keep only WorkflowTemplates that can be started."""
         assert user is not None  # Enforced by decorator
-        # Workers and anonymous users are not allowed
-        if context.worker_token or not user.is_authenticated:
+        # Anonymous users are not allowed
+        if not user.is_authenticated:
             return self.none()
         return self.filter(
-            WORKSPACE_ROLES(
-                user,
-                scope=[Scope.Roles.OWNER],
-                workspace=[Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR],
-                member="workspace",
+            workspace__in=Workspace.objects.with_role(
+                user, Workspace.Roles.CONTRIBUTOR
             )
         ).distinct()
 
@@ -239,24 +236,26 @@ class WorkflowTemplate(models.Model):
         ) as exc:
             raise ValidationError({"task_data": str(exc)})
 
-    @permission_check("{user} cannot display {resource}")
+    @permission_check(
+        "{user} cannot display {resource}", workers=AllowWorkers.ALWAYS
+    )
     def can_display(self, user: PermissionUser) -> bool:
         """Check if the workflow template can be displayed."""
         # Delegate to workspace can_display check
         return self.workspace.can_display(user)
 
-    @permission_check("{user} cannot run {resource}")
+    @permission_check(
+        "{user} cannot run {resource}", workers=AllowWorkers.NEVER
+    )
     def can_run(self, user: PermissionUser) -> bool:
         """Check if the workflow template can be run."""
         assert user is not None  # enforced by decorator
-        # Workers and anonymous users are not allowed
-        if context.worker_token or not user.is_authenticated:
+        # anonymous users are not allowed
+        if not user.is_authenticated:
             return False
         # Shortcut to avoid hitting the database for common cases
         match self.workspace.context_has_role(
-            user,
-            (Workspace.Roles.OWNER, Workspace.Roles.CONTRIBUTOR),
-            scope_roles=Scope.Roles.OWNER,
+            user, Workspace.Roles.CONTRIBUTOR
         ):
             case PartialCheckResult.ALLOW:
                 return True
@@ -317,11 +316,11 @@ class _WorkRequestStatuses(models.TextChoices):
 class WorkRequestQuerySet(QuerySet["WorkRequest", A], Generic[A]):
     """Custom QuerySet for WorkRequest."""
 
-    def in_current_scope(self) -> "WorkRequestQuerySet[A]":
+    def in_current_scope(self) -> Self:
         """Filter to work requests in the current scope."""
         return self.filter(workspace__scope=context.require_scope())
 
-    def in_current_workspace(self) -> "WorkRequestQuerySet[A]":
+    def in_current_workspace(self) -> Self:
         """Filter to work requests in the current workspace."""
         return self.filter(workspace=context.require_workspace())
 
@@ -358,13 +357,13 @@ class WorkRequestQuerySet(QuerySet["WorkRequest", A], Generic[A]):
             )
         )
 
-    @permission_filter
+    @permission_filter(workers=AllowWorkers.ALWAYS)
     def can_display(self, user: PermissionUser) -> "WorkRequestQuerySet[A]":
         """Keep only WorkRequests that can be displayed."""
         # Delegate to workspace can_display check
         return self.filter(workspace__in=Workspace.objects.can_display(user))
 
-    @permission_filter
+    @permission_filter(workers=AllowWorkers.ALWAYS)
     def can_retry(self, user: PermissionUser) -> "WorkRequestQuerySet[A]":
         """Keep only WorkRequests that can be retried."""
         # Delegate to workspace can_display check
@@ -372,24 +371,104 @@ class WorkRequestQuerySet(QuerySet["WorkRequest", A], Generic[A]):
         # for a later implementation of a more accurate check
         return self.filter(workspace__in=Workspace.objects.can_display(user))
 
-    @permission_filter
+    @permission_filter(workers=AllowWorkers.NEVER)
     def can_unblock(self, user: PermissionUser) -> "WorkRequestQuerySet[A]":
         """Keep only WorkRequests that can be unblocked."""
         assert user is not None  # enforced by decorator
-        # Workers and anonymous users are not allowed
-        if context.worker_token or not user.is_authenticated:
+        # Anonymous users are not allowed
+        if not user.is_authenticated:
             return self.none()
 
         # Delegate to workspace/scope ownership
         return self.filter(
-            WORKSPACE_ROLES(
-                user,
-                scope=[Scope.Roles.OWNER],
-                workspace=[Workspace.Roles.OWNER],
-                member="workspace",
+            workspace__in=Workspace.objects.with_role(
+                user, Workspace.Roles.OWNER
             )
         ).distinct()
 
+    def pending(
+        self, exclude_assigned: bool = False, worker: Optional["Worker"] = None
+    ) -> "WorkRequestQuerySet[Any]":
+        """
+        Return a QuerySet of tasks in WorkRequest.Statuses.PENDING status.
+
+        QuerySet is ordered by descending effective priority, then by
+        created_at.
+
+        Filter out the assigned pending ones if exclude_assigned=True,
+        and include only the WorkRequest for worker.
+
+        PENDING is the default status of a task on creation.
+        """
+        if exclude_assigned and worker is not None:
+            raise ValueError("Cannot exclude_assigned and filter by worker")
+
+        qs = self.filter(status=WorkRequest.Statuses.PENDING)
+
+        if exclude_assigned:
+            qs = qs.exclude(worker__isnull=False)
+
+        if worker is not None:
+            qs = qs.filter(worker=worker)
+
+        qs = qs.order_by(
+            (F("priority_base") + F("priority_adjustment")).desc(), "created_at"
+        )
+
+        return qs
+
+    def running(
+        self, worker: Optional["Worker"] = None
+    ) -> "WorkRequestQuerySet[Any]":
+        """Return a QuerySet of tasks in running status."""
+        qs = self.filter(status=WorkRequest.Statuses.RUNNING)
+
+        if worker is not None:
+            qs = qs.filter(worker=worker)
+
+        return qs
+
+    def completed(self) -> "WorkRequestQuerySet[Any]":
+        """Return a QuerySet of tasks in completed status."""
+        return self.filter(status=WorkRequest.Statuses.COMPLETED)
+
+    def aborted(self) -> "WorkRequestQuerySet[Any]":
+        """Return a QuerySet of tasks in aborted status."""
+        return self.filter(status=WorkRequest.Statuses.ABORTED)
+
+    def aborted_or_failed(self) -> "WorkRequestQuerySet[Any]":
+        """Return a QuerySet of tasks that aborted or failed."""
+        return self.filter(
+            Q(status=WorkRequest.Statuses.ABORTED)
+            | Q(
+                status=WorkRequest.Statuses.COMPLETED,
+                result=WorkRequest.Results.FAILURE,
+            )
+        )
+
+    def expired(self, at: datetime) -> "WorkRequestQuerySet[Any]":
+        """
+        Return queryset with work requests that have expired.
+
+        :param at: datetime to check if the work requests are expired.
+        :return: work requests which expire before the given datetime.
+        """
+        return (
+            self.annotate(
+                effective_expiration_delay=Coalesce(
+                    "expiration_delay",
+                    "workspace__default_expiration_delay",
+                )
+            )
+            .exclude(effective_expiration_delay=timedelta(0))
+            .filter(
+                # https://github.com/typeddjango/django-stubs/issues/1548
+                created_at__lte=(
+                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
+                )
+            )
+        )
+
 
 class WorkRequestManager(models.Manager["WorkRequest"]):
     """Manager for WorkRequest model."""
@@ -522,90 +601,6 @@ class WorkRequestManager(models.Manager["WorkRequest"]):
             ).dict(exclude_unset=True),
         )
 
-    def pending(
-        self, exclude_assigned: bool = False, worker: Optional["Worker"] = None
-    ) -> QuerySet["WorkRequest"]:
-        """
-        Return a QuerySet of tasks in WorkRequest.Statuses.PENDING status.
-
-        QuerySet is ordered by descending effective priority, then by
-        created_at.
-
-        Filter out the assigned pending ones if exclude_assigned=True,
-        and include only the WorkRequest for worker.
-
-        PENDING is the default status of a task on creation.
-        """
-        if exclude_assigned and worker is not None:
-            raise ValueError("Cannot exclude_assigned and filter by worker")
-
-        qs = WorkRequest.objects.filter(status=WorkRequest.Statuses.PENDING)
-
-        if exclude_assigned:
-            qs = qs.exclude(worker__isnull=False)
-
-        if worker is not None:
-            qs = qs.filter(worker=worker)
-
-        qs = qs.order_by(
-            (F("priority_base") + F("priority_adjustment")).desc(), "created_at"
-        )
-
-        return qs
-
-    def running(
-        self, worker: Optional["Worker"] = None
-    ) -> QuerySet["WorkRequest"]:
-        """Return a QuerySet of tasks in running status."""
-        qs = WorkRequest.objects.filter(status=WorkRequest.Statuses.RUNNING)
-
-        if worker is not None:
-            qs = qs.filter(worker=worker)
-
-        return qs
-
-    def completed(self) -> QuerySet["WorkRequest"]:
-        """Return a QuerySet of tasks in completed status."""
-        return WorkRequest.objects.filter(status=WorkRequest.Statuses.COMPLETED)
-
-    def aborted(self) -> QuerySet["WorkRequest"]:
-        """Return a QuerySet of tasks in aborted status."""
-        return WorkRequest.objects.filter(status=WorkRequest.Statuses.ABORTED)
-
-    def aborted_or_failed(self) -> QuerySet["WorkRequest"]:
-        """Return a QuerySet of tasks that aborted or failed."""
-        return WorkRequest.objects.filter(
-            Q(status=WorkRequest.Statuses.ABORTED)
-            | Q(
-                status=WorkRequest.Statuses.COMPLETED,
-                result=WorkRequest.Results.FAILURE,
-            )
-        )
-
-    def expired(self, at: datetime) -> QuerySet["WorkRequest"]:
-        """
-        Return queryset with work requests that have expired.
-
-        :param at: datetime to check if the work requests are expired.
-        :return: work requests which expire before the given datetime.
-        """
-        return (
-            self.get_queryset()
-            .annotate(
-                effective_expiration_delay=Coalesce(
-                    "expiration_delay",
-                    "workspace__default_expiration_delay",
-                )
-            )
-            .exclude(effective_expiration_delay=timedelta(0))
-            .filter(
-                # https://github.com/typeddjango/django-stubs/issues/1548
-                created_at__lte=(
-                    at - F("effective_expiration_delay")  # type: ignore[operator] # noqa: E501
-                )
-            )
-        )
-
 
 class _RuntimeStatuses(models.TextChoices):
     NEEDS_INPUT = "needs_input", "Needs Input"
@@ -892,13 +887,17 @@ class WorkRequest(models.Model):
             self.process_event_reactions("on_creation")
         super().save(*args, **kwargs)
 
-    @permission_check("{user} cannot display {resource}")
+    @permission_check(
+        "{user} cannot display {resource}", workers=AllowWorkers.ALWAYS
+    )
     def can_display(self, user: PermissionUser) -> bool:
         """Check if the work request can be displayed."""
         # Delegate to workspace can_display check
         return self.workspace.can_display(user)
 
-    @permission_check("{user} cannot retry {resource}")
+    @permission_check(
+        "{user} cannot retry {resource}", workers=AllowWorkers.ALWAYS
+    )
     def can_retry(self, user: PermissionUser) -> bool:
         """Check if the work request can be retried."""
         # Delegate to workspace can_display check
@@ -906,19 +905,17 @@ class WorkRequest(models.Model):
         # for a later implementation of a more accurate check
         return self.workspace.can_display(user)
 
-    @permission_check("{user} cannot unblock {resource}")
+    @permission_check(
+        "{user} cannot unblock {resource}", workers=AllowWorkers.NEVER
+    )
     def can_unblock(self, user: PermissionUser) -> bool:
         """Check if the work request can be unblocked."""
         assert user is not None  # enforced by decorator
-        # Workers and anonymous users are not allowed
-        if context.worker_token or not user.is_authenticated:
+        # Anonymous users are not allowed
+        if not user.is_authenticated:
             return False
         # Shortcut to avoid hitting the database for common cases
-        match self.workspace.context_has_role(
-            user,
-            Workspace.Roles.OWNER,
-            scope_roles=Scope.Roles.OWNER,
-        ):
+        match self.workspace.context_has_role(user, Workspace.Roles.OWNER):
             case PartialCheckResult.ALLOW:
                 return True
             case PartialCheckResult.DENY:
