@@ -7,14 +7,20 @@
 # modified, propagated, or distributed except according to the terms
 # contained in the LICENSE file.
 """Inheritable test scenarios."""
-
+import textwrap
 from functools import cached_property
 from typing import TYPE_CHECKING
 
-from debusine.artifacts.models import ArtifactCategory, CollectionCategory
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    CollectionCategory,
+    DebianSourcePackage,
+    DebianUpload,
+)
 from debusine.db.context import context
 from debusine.db.models import (
     Artifact,
+    ArtifactRelation,
     Collection,
     CollectionItem,
     Group,
@@ -29,6 +35,7 @@ from debusine.db.models import (
 )
 from debusine.db.models.auth import Identity
 from debusine.server.collections.debian_suite import DebianSuiteManager
+from debusine.tasks import DebDiff
 from debusine.tasks.models import WorkerType
 
 if TYPE_CHECKING:
@@ -308,6 +315,97 @@ class UIPlayground(DefaultContext):
 
         self.simulate_sbuild_workflow(self.template_sbuild, self.source_udev)
 
+        # Create a binary DebDiff artifact and its required relation
+        debdiff_report = textwrap.dedent(
+            """
+            some debdiff text header"
+
+            Files in second .deb but not in first
+            -------------------------------------
+            -rw-r--r--  root/root   /etc/simplemonitor/monitor.ini
+            -rw-r--r--  root/root   DEBIAN/conffiles
+
+            Files in first .deb but not in second
+            ------------------------------------
+            -rw-r--r--  root/root   /usr/lib/python3/dist-packages/pyaarlo
+            -rw-r--r--  root/root   /usr/share/doc/python3-pyaarlo/READ.gz
+
+            Control files: lines which differ (wdiff format)
+            ------------------------------------------------
+            Description: [-one description-] {+another description+}
+            Homepage: [-https://github.com/twrecked/pyaarlo-]
+            {+https://simplemonitor.readthedocs.io+}"""
+        ).encode("utf-8")
+
+        artifact, _ = playground.create_artifact(
+            paths={DebDiff.CAPTURE_OUTPUT_FILENAME: debdiff_report},
+            category=ArtifactCategory.DEBDIFF,
+            create_files=True,
+            data={"original": "hello-1.deb", "new": "hello-2.deb"},
+        )
+
+        target, _ = playground.create_artifact(
+            category=ArtifactCategory.UPLOAD,
+            data=DebianUpload(
+                type="dpkg",
+                changes_fields={
+                    "Architecture": "amd64",
+                    "Files": [{"name": "hello_1.0_amd64.deb"}],
+                },
+            ),
+        )
+
+        ArtifactRelation.objects.create(
+            artifact=artifact,
+            target=target,
+            type=ArtifactRelation.Relations.RELATES_TO,
+        )
+
+        # Create a source DebDiff artifact and its required relation
+        debdiff_report = textwrap.dedent(
+            """\
+            diff -u -N original/added.txt new/added.txt
+            --- original/added.txt	1970-01-01 01:00:00.000000000 +0100
+            +++ new/added.txt	2025-03-26 12:42:27.672906377 +0000
+            @@ -0,0 +1 @@
+            +new file
+            diff -u -N original/changed.txt new/changed.txt
+            --- original/changed.txt	2025-03-26 12:41:44.191924503 +0000
+            +++ new/changed.txt	2025-03-26 12:42:21.945798421 +0000
+            @@ -1,3 +1,2 @@
+             1
+             2
+            -3
+            diff -u -N original/removed.txt new/removed.txt
+            --- original/removed.txt	2025-03-26 12:42:37.607381416 +0000
+            +++ new/removed.txt	1970-01-01 01:00:00.000000000 +0100
+            @@ -1 +0,0 @@
+            -removed file
+        """
+        ).encode("utf-8")
+
+        artifact, _ = self.playground.create_artifact(
+            paths={DebDiff.CAPTURE_OUTPUT_FILENAME: debdiff_report},
+            category=ArtifactCategory.DEBDIFF,
+            create_files=True,
+            data={"original": "hello-1.dsc", "new": "hello-2.dsc"},
+        )
+
+        target, _ = playground.create_artifact(
+            category=ArtifactCategory.SOURCE_PACKAGE,
+            data=DebianSourcePackage(
+                name="hello",
+                version="1.0",
+                type="dpkg",
+                dsc_fields={},
+            ),
+        )
+        ArtifactRelation.objects.create(
+            artifact=artifact,
+            target=target,
+            type=ArtifactRelation.Relations.RELATES_TO,
+        )
+
     def add_results_to_suite(
         self,
         work_request: WorkRequest,
