@@ -13,8 +13,8 @@ except (ModuleNotFoundError, PermissionError):
     )
 
 # Use paths from the package
+from debusine.project.project_utils import read_secret_key
 from debusine.project.settings.pkg_paths import *  # noqa: F401, F403, I202
-from debusine.project.utils import read_secret_key
 
 # Read the SECRET_KEY from the file
 SECRET_KEY = read_secret_key(f"{DEBUSINE_DATA_PATH}/key")  # noqa: F405
