@@ -16,6 +16,8 @@ lookups, etc.
 Each specific CollectionManager implement their own business logic.
 """
 
+from django.conf import settings
+
 from debusine.server.collections.base import (
     CollectionManagerInterface,
     ItemAdditionError,
@@ -47,3 +49,13 @@ __all__ = [
     "TaskHistoryManager",
     "WorkflowInternalManager",
 ]
+
+if getattr(settings, "TEST_MODE", False):  # pragma: no cover
+    # Register a manager to make debusine:test collections easily usable by
+    # any test.  This manager doesn't implement any specific semantics, but
+    # it at least allows basic name lookups to work.
+    from debusine.server.collections.tests.test_base import (  # noqa: F401
+        TestManager,
+    )
+
+    __all__.append("TestManager")
