@@ -20,8 +20,10 @@ from django.db.models import Q
 from debusine.artifacts.models import (
     ArtifactCategory,
     BareDataCategory,
+    BaseArtifactDataModel,
     CollectionCategory,
 )
+from debusine.client.models import model_to_json_serializable_dict
 from debusine.db.models import (
     Artifact,
     Collection,
@@ -112,7 +114,7 @@ class CollectionManagerInterface(ABC):
         *,
         user: User,
         workflow: WorkRequest | None = None,
-        data: dict[str, Any] | None = None,
+        data: BaseArtifactDataModel | dict[str, Any] | None = None,
         name: str | None = None,
         replace: bool = False,
     ) -> CollectionItem:
@@ -140,6 +142,8 @@ class CollectionManagerInterface(ABC):
                 f'Bare data category "{category}" '
                 f'not supported by the collection'
             )
+        if isinstance(data, BaseArtifactDataModel):
+            data = model_to_json_serializable_dict(data, exclude_unset=True)
 
         return self.do_add_bare_data(
             category,
