@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -9,7 +9,9 @@
 
 """Unit tests for DebianPackageBuildLogsManager."""
 
-from django.db.models import Q
+from django.db.models import IntegerField, Q
+from django.db.models.functions import Cast
+from django.db.models.lookups import In
 
 from debusine.artifacts.models import (
     ArtifactCategory,
@@ -198,7 +200,7 @@ class DebianPackageBuildLogsManagerTests(TestCase):
     @context.disable_permission_checks()
     def test_do_add_artifact_no_variables(self) -> None:
         """`do_add_artifact` requires variables."""
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG, data={}
         )
 
@@ -214,11 +216,11 @@ class DebianPackageBuildLogsManagerTests(TestCase):
     def test_do_add_artifact_raise_item_addition_error(self) -> None:
         """`do_add_artifact` raises an error for duplicate names."""
         work_request = self.playground.create_work_request()
-        artifact_1, _ = self.create_artifact(
+        artifact_1, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             data={"source": "hello", "version": "1.0-1"},
         )
-        artifact_2, _ = self.create_artifact(
+        artifact_2, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             data={"source": "hello", "version": "1.0-1"},
         )
@@ -251,7 +253,7 @@ class DebianPackageBuildLogsManagerTests(TestCase):
         be more like `do_add_bare_data`.
         """
         work_request = self.playground.create_work_request()
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             data={"source": "hello", "version": "1.0-1"},
         )
@@ -277,12 +279,12 @@ class DebianPackageBuildLogsManagerTests(TestCase):
     def test_do_add_artifact_different_work_request_ids(self) -> None:
         """Adding artifacts with different work request IDs is OK."""
         work_request_1 = self.playground.create_work_request()
-        artifact_1, _ = self.create_artifact(
+        artifact_1, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             data={"source": "hello", "version": "1.0-1"},
         )
         work_request_2 = self.playground.create_work_request()
-        artifact_2, _ = self.create_artifact(
+        artifact_2, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             data={"source": "hello", "version": "1.0-1"},
         )
@@ -331,7 +333,7 @@ class DebianPackageBuildLogsManagerTests(TestCase):
             workflow=self.workflow,
             data=data,
         )
-        artifact_new, _ = self.create_artifact(
+        artifact_new, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG,
             data={"source": "hello", "version": "1.0-1"},
         )
@@ -368,10 +370,10 @@ class DebianPackageBuildLogsManagerTests(TestCase):
         worker_1 = self.playground.create_worker()
         worker_2 = self.playground.create_worker()
         work_request = self.playground.create_work_request(worker=worker_1)
-        artifact_old, _ = self.create_artifact(
+        artifact_old, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG, data={}
         )
-        artifact_new, _ = self.create_artifact(
+        artifact_new, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG, data={}
         )
         data = {
@@ -417,7 +419,7 @@ class DebianPackageBuildLogsManagerTests(TestCase):
     def test_do_add_artifact_replace_nonexistent(self) -> None:
         """Replacing a nonexistent artifact is allowed."""
         work_request = self.playground.create_work_request()
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG, data={}
         )
         data = {
@@ -447,7 +449,7 @@ class DebianPackageBuildLogsManagerTests(TestCase):
     def test_do_remove_artifact(self) -> None:
         """`do_remove_artifact` removes the item."""
         work_request = self.playground.create_work_request()
-        artifact, _ = self.create_artifact(
+        artifact, _ = self.playground.create_artifact(
             category=ArtifactCategory.PACKAGE_BUILD_LOG, data={}
         )
         data = {
@@ -524,7 +526,13 @@ class DebianPackageBuildLogsManagerTests(TestCase):
             user=self.user,
         )
         self.assertEqual(
-            condition, Q(data__work_request_id__in={work_requests[0].id})
+            condition,
+            Q(
+                In(
+                    Cast("data__work_request_id", IntegerField()),
+                    {work_requests[0].id},
+                )
+            ),
         )
         self.assertQuerySetEqual(
             CollectionItem.active_objects.filter(
@@ -600,9 +608,10 @@ class DebianPackageBuildLogsManagerTests(TestCase):
         self.assertEqual(
             condition,
             Q(
-                data__work_request_id__in={
-                    work_request.id for work_request in work_requests[:2]
-                }
+                In(
+                    Cast("data__work_request_id", IntegerField()),
+                    {work_request.id for work_request in work_requests[:2]},
+                )
             ),
         )
         self.assertQuerySetEqual(
