@@ -58,8 +58,8 @@ class TaskHistoryManagerTests(TestCase):
         self,
         task_type: TaskTypes,
         task_name: str,
-        subject: str,
-        context: str,
+        subject: str | None,
+        context: str | None,
         work_request_id: int,
         result: WorkRequest.Results = WorkRequest.Results.SUCCESS,
         collection: Collection | None = None,
@@ -70,7 +70,13 @@ class TaskHistoryManagerTests(TestCase):
             BareDataCategory.HISTORICAL_TASK_RUN,
             parent_collection=collection or self.collection,
             name=":".join(
-                [task_type, task_name, subject, context, str(work_request_id)]
+                [
+                    task_type,
+                    task_name,
+                    subject or "",
+                    context or "",
+                    str(work_request_id),
+                ]
             ),
             data=DebusineHistoricalTaskRun(
                 task_type=task_type,
@@ -179,6 +185,48 @@ class TaskHistoryManagerTests(TestCase):
             ],
         )
 
+    def test_add_bare_data_unset_subject_and_context(self) -> None:
+        """`add_bare_data` handles subject=None and context=None."""
+        item = self.manager.add_bare_data(
+            BareDataCategory.HISTORICAL_TASK_RUN,
+            user=self.scenario.user,
+            workflow=self.workflow,
+            data=DebusineHistoricalTaskRun(
+                task_type=TaskTypes.WORKER,
+                task_name="sbuild",
+                subject=None,
+                context=None,
+                timestamp=int(timezone.now().timestamp()),
+                work_request_id=1,
+                result=WorkRequest.Results.SUCCESS,
+                runtime_statistics=RuntimeStatistics(duration=1),
+            ),
+        )
+
+        self.assertEqual(item.name, "Worker:sbuild:::1")
+
+    def test_add_bare_data_quotes_subject_and_context(self) -> None:
+        """`add_bare_data` URL-encodes subject and context."""
+        item = self.manager.add_bare_data(
+            BareDataCategory.HISTORICAL_TASK_RUN,
+            user=self.scenario.user,
+            workflow=self.workflow,
+            data=DebusineHistoricalTaskRun(
+                task_type=TaskTypes.WORKER,
+                task_name="sbuild",
+                subject="hello:world",
+                context="any:amd64:amd64",
+                timestamp=int(timezone.now().timestamp()),
+                work_request_id=1,
+                result=WorkRequest.Results.SUCCESS,
+                runtime_statistics=RuntimeStatistics(duration=1),
+            ),
+        )
+
+        self.assertEqual(
+            item.name, "Worker:sbuild:hello%3Aworld:any%3Aamd64%3Aamd64:1"
+        )
+
     def test_add_bare_data_replace(self) -> None:
         """`add_bare_data` can replace an existing bare data item."""
         data = DebusineHistoricalTaskRun(
@@ -432,30 +480,37 @@ class TaskHistoryManagerTests(TestCase):
         """`lookup` for `last-entry` returns the last matching entry."""
         items = [
             self.create_historical_task_run(
-                TaskTypes.WORKER,
-                "sbuild",
-                subject,
-                "sid/amd64",
-                work_request_id,
+                TaskTypes.WORKER, "sbuild", subject, context, work_request_id
             )
-            for subject, work_request_id in (
-                ("base-files", 1),
-                ("base-passwd", 2),
-                ("base-files", 3),
+            for subject, context, work_request_id in (
+                (None, None, 1),
+                ("base-files", "sid/amd64", 2),
+                ("base-passwd", "sid/amd64", 3),
+                ("base-files", "sid/amd64", 4),
+                ("hello:world", "any:amd64:amd64", 5),
             )
         ]
 
+        self.assertEqual(
+            self.manager.lookup("last-entry:Worker:sbuild::"), items[0]
+        )
         self.assertEqual(
             self.manager.lookup(
                 "last-entry:Worker:sbuild:base-files:sid/amd64"
             ),
-            items[2],
+            items[3],
         )
         self.assertEqual(
             self.manager.lookup(
                 "last-entry:Worker:sbuild:base-passwd:sid/amd64"
             ),
-            items[1],
+            items[2],
+        )
+        self.assertEqual(
+            self.manager.lookup(
+                "last-entry:Worker:sbuild:hello%3Aworld:any%3Aamd64%3Aamd64"
+            ),
+            items[4],
         )
 
     def test_lookup_last_success(self) -> None:
@@ -465,22 +520,38 @@ class TaskHistoryManagerTests(TestCase):
                 TaskTypes.WORKER,
                 "sbuild",
                 subject,
-                "sid/amd64",
+                context,
                 work_request_id,
                 result=result,
             )
-            for subject, work_request_id, result in (
-                ("base-files", 1, WorkRequest.Results.SUCCESS),
-                ("base-passwd", 2, WorkRequest.Results.SUCCESS),
-                ("base-files", 3, WorkRequest.Results.FAILURE),
+            for subject, context, work_request_id, result in (
+                (None, None, 1, WorkRequest.Results.SUCCESS),
+                ("base-files", "sid/amd64", 2, WorkRequest.Results.SUCCESS),
+                ("base-passwd", "sid/amd64", 3, WorkRequest.Results.SUCCESS),
+                ("base-files", "sid/amd64", 4, WorkRequest.Results.FAILURE),
+                (
+                    "hello:world",
+                    "any:amd64:amd64",
+                    5,
+                    WorkRequest.Results.SUCCESS,
+                ),
             )
         ]
 
+        self.assertEqual(
+            self.manager.lookup("last-success:Worker:sbuild::"), items[0]
+        )
         self.assertEqual(
             self.manager.lookup(
                 "last-success:Worker:sbuild:base-files:sid/amd64"
             ),
-            items[0],
+            items[1],
+        )
+        self.assertEqual(
+            self.manager.lookup(
+                "last-success:Worker:sbuild:hello%3Aworld:any%3Aamd64%3Aamd64"
+            ),
+            items[4],
         )
 
     def test_lookup_last_failure(self) -> None:
@@ -490,22 +561,38 @@ class TaskHistoryManagerTests(TestCase):
                 TaskTypes.WORKER,
                 "sbuild",
                 subject,
-                "sid/amd64",
+                context,
                 work_request_id,
                 result=result,
             )
-            for subject, work_request_id, result in (
-                ("base-files", 1, WorkRequest.Results.FAILURE),
-                ("base-passwd", 2, WorkRequest.Results.FAILURE),
-                ("base-files", 3, WorkRequest.Results.SUCCESS),
+            for subject, context, work_request_id, result in (
+                (None, None, 1, WorkRequest.Results.FAILURE),
+                ("base-files", "sid/amd64", 2, WorkRequest.Results.FAILURE),
+                ("base-passwd", "sid/amd64", 3, WorkRequest.Results.FAILURE),
+                ("base-files", "sid/amd64", 4, WorkRequest.Results.SUCCESS),
+                (
+                    "hello:world",
+                    "any:amd64:amd64",
+                    5,
+                    WorkRequest.Results.FAILURE,
+                ),
             )
         ]
 
+        self.assertEqual(
+            self.manager.lookup("last-failure:Worker:sbuild::"), items[0]
+        )
         self.assertEqual(
             self.manager.lookup(
                 "last-failure:Worker:sbuild:base-files:sid/amd64"
             ),
-            items[0],
+            items[1],
+        )
+        self.assertEqual(
+            self.manager.lookup(
+                "last-failure:Worker:sbuild:hello%3Aworld:any%3Aamd64%3Aamd64"
+            ),
+            items[4],
         )
 
     def test_lookup_filter_unexpected_format(self) -> None:
