@@ -16,7 +16,7 @@ from typing import Any, NoReturn, TYPE_CHECKING
 
 from rest_framework import status
 from rest_framework.exceptions import APIException
-from rest_framework.views import exception_handler
+from rest_framework.views import exception_handler, set_rollback
 
 from debusine.db.context import context
 from debusine.db.models import Workspace
@@ -36,6 +36,7 @@ class DebusineAPIException(APIException):
         detail: str | None = None,
         validation_errors: Iterable[Any] | None = None,
         status_code: int = status.HTTP_400_BAD_REQUEST,
+        rollback_transaction: bool = True,
     ) -> None:
         """Initialize with ProblemResponse's arguments."""
         super().__init__(code=str(status_code))
@@ -43,6 +44,7 @@ class DebusineAPIException(APIException):
         self.debusine_detail = detail
         self.debusine_validation_errors = validation_errors
         self.debusine_status_code = status_code
+        self.rollback_transaction = rollback_transaction
 
 
 def raise_workspace_not_found(workspace: str | Workspace) -> NoReturn:
@@ -70,6 +72,8 @@ def debusine_exception_handler(
 
     match exc:
         case DebusineAPIException():
+            if exc.rollback_transaction:
+                set_rollback()
             return ProblemResponse(
                 title=exc.debusine_title,
                 detail=exc.debusine_detail,
