@@ -10,6 +10,7 @@
 """Tests for the file backend interface."""
 
 from collections.abc import Callable, Generator
+from contextlib import contextmanager
 from datetime import datetime
 from pathlib import Path
 from typing import ClassVar
@@ -37,6 +38,11 @@ class FakeFileBackendEntry(FileBackendEntryInterface["FakeFileBackend", bytes]):
         """Return local path of entry."""
         return None
 
+    @contextmanager
+    def get_temporary_local_path(self) -> Generator[Path]:
+        """Yield the path to a possibly-temporary local copy of the file."""
+        raise NotImplementedError()
+
     def get_url(self, content_type: str | None = None) -> None:  # noqa: U100
         """Return URL of entry."""
         return None
