@@ -15,7 +15,7 @@ from django.core.management import CommandParser
 
 from debusine.db.models import Token
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import Tokens
+from debusine.server.management.management_utils import Tokens
 
 
 class Command(DebusineBaseCommand):
