@@ -15,7 +15,7 @@ from django.core.management import CommandParser
 
 from debusine.db.models import Workspace
 from debusine.django.management.debusine_base_command import DebusineBaseCommand
-from debusine.server.management.utils import Workspaces
+from debusine.server.management.management_utils import Workspaces
 
 # TODO: this is deprecated in favor of the 'workspace' command
 
