@@ -22,7 +22,7 @@ from debusine.assets.models import (
     CloudProvidersType,
     KeyPurpose,
 )
-from debusine.db.models import Asset, DEFAULT_WORKSPACE_NAME
+from debusine.db.models import Asset
 from debusine.db.models.assets import AssetRoles, AssetUsageRoles
 from debusine.django.management.tests import call_command
 from debusine.server.management.commands.tests.utils import TabularOutputTests
@@ -156,7 +156,8 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
             "asset",
             "create",
             "--workspace",
-            f"{settings.DEBUSINE_DEFAULT_SCOPE}/{DEFAULT_WORKSPACE_NAME}",
+            f"{settings.DEBUSINE_DEFAULT_SCOPE}/"
+            f"{settings.DEBUSINE_DEFAULT_WORKSPACE}",
             AssetCategory.SIGNING_KEY,
             stdin=io.StringIO(yaml.safe_dump(data)),
         )
@@ -331,7 +332,6 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
         self.assertEqual(
             output.col(2), [expected["workspace"] for expected in expected_data]
         )
-        self.maxDiff = None
         self.assertEqual(
             output.col(3), [str(expected["data"]) for expected in expected_data]
         )
@@ -517,8 +517,7 @@ class AssetCommandsTests(TabularOutputTests, TestCase):
         asset = self.playground.create_signing_key_asset()
         group1 = self.playground.create_group(name="group1")
         with self.assertRaisesRegex(
-            CommandError,
-            r"Workspace 'unknown' does not exist within <Scope: debusine>",
+            CommandError, r"Workspace 'unknown' not found in scope 'debusine'"
         ):
             call_command(
                 "asset",
