@@ -10,7 +10,7 @@
 """Tests for the management command create_workflow."""
 
 import io
-from typing import ClassVar, assert_never
+from typing import ClassVar
 
 import yaml
 from django.core.management import CommandError
@@ -20,12 +20,10 @@ from debusine.db.models import (
     Artifact,
     WorkRequest,
     WorkflowTemplate,
-    Workspace,
     default_workspace,
     system_user,
 )
 from debusine.django.management.tests import call_command
-from debusine.server.management.commands.create_workflow import Command
 from debusine.tasks.models import TaskTypes
 from debusine.test.django import TestCase
 
@@ -57,36 +55,6 @@ class CreateWorkflowCommandTests(TestCase):
             },
         )
 
-    def test_get_workspace(self) -> None:
-        """Test get_workspace."""
-        with context.disable_permission_checks():
-            command = Command()
-            default_workspace = self.playground.get_default_workspace()
-            scope = self.playground.get_or_create_scope("scope")
-            workspace = self.playground.create_workspace(
-                name="workspace", scope=scope
-            )
-        for name, expected in (
-            ("System", default_workspace),
-            ("workspace", "Workspace 'workspace' not found"),
-            ("debusine/System", default_workspace),
-            ("scope/workspace", workspace),
-            ("does-not-exist/workspace", "Scope 'does-not-exist' not found"),
-            ("debusine/workspace", "Workspace 'debusine/workspace' not found"),
-        ):
-            with self.subTest(name=name):
-                match expected:
-                    case Workspace():
-                        self.assertEqual(command.get_workspace(name), expected)
-                    case str():
-                        with self.assertRaisesRegex(
-                            CommandError, expected
-                        ) as exc:
-                            command.get_workspace(name)
-                        self.assertEqual(exc.exception.returncode, 3)
-                    case _ as unreachable:
-                        assert_never(unreachable)
-
     def test_data_from_file(self) -> None:
         """`create_workflow` accepts data from a file."""
         data = {"target_distribution": "debian:bookworm"}
