@@ -14,7 +14,7 @@ from rich.pretty import pretty_repr
 from debusine.db.models import NotificationChannel
 from debusine.django.management.tests import call_command
 from debusine.server.management.commands.tests.utils import TabularOutputTests
-from debusine.server.management.utils import sort_keys
+from debusine.server.management.management_utils import sort_keys
 
 
 class ListNotificationChannelsTests(TabularOutputTests, TestCase):
