@@ -10,6 +10,7 @@
 """Schedule WorkRequests to Workers."""
 
 import logging
+from operator import attrgetter
 from typing import Any
 from uuid import uuid4
 
@@ -24,8 +25,7 @@ from django.db.models.signals import post_save
 from django.dispatch import receiver
 from django.utils import timezone
 
-from debusine.client.models import model_to_json_serializable_dict
-from debusine.db.models import TaskDatabase, WorkRequest, Worker
+from debusine.db.models import WorkRequest, Worker
 from debusine.server.celery import run_server_task
 from debusine.server.workflows.celery import run_workflow_task
 from debusine.tasks import TaskConfigError
@@ -100,32 +100,19 @@ def _assign_work_request_to_worker(
         # initial qs but has one now - nothing to do
         return None
 
-    try:
-        dynamic_task_data = task.compute_dynamic_data(
-            TaskDatabase(work_request)
-        )
-    except Exception as exc:
-        logger.warning(  # noqa: G200
-            "WorkRequest %s failed to pre-process task data, aborting it. "
-            "Task data: %s Error: %s",
-            work_request.id,
-            work_request.task_data,
-            exc,
-        )
-        work_request.mark_completed(WorkRequest.Results.ERROR)
-        return None
-
     with work_request.scheduling_disabled():
-        work_request.dynamic_task_data = (
-            None
-            if dynamic_task_data is None
-            else model_to_json_serializable_dict(
-                dynamic_task_data, exclude_unset=True
+        try:
+            work_request.assign_worker(worker)
+        except Exception as exc:
+            logger.warning(  # noqa: G200
+                "WorkRequest %s failed to pre-process task data, aborting it. "
+                "Task data: %s Error: %s",
+                work_request.id,
+                work_request.task_data,
+                exc,
             )
-        )
-        work_request.save()
-
-        work_request.assign_worker(worker)
+            work_request.mark_completed(WorkRequest.Results.ERROR)
+            return None
 
     if task.TASK_TYPE is TaskTypes.SERVER:
         # Use a slightly more informative task ID than the default.
@@ -210,6 +197,7 @@ def schedule_internal() -> list[WorkRequest]:
         | Q(task_type__in={TaskTypes.WORKFLOW, TaskTypes.WAIT})
     )
 
+    workflow_roots: set[WorkRequest] = set()
     result: list[WorkRequest] = []
     for work_request in qs:
         match (work_request.task_type, work_request.task_name):
@@ -229,15 +217,15 @@ def schedule_internal() -> list[WorkRequest]:
                 result.append(work_request)
 
             case (TaskTypes.WORKFLOW, _):
-                # Workflows are populated by the corresponding workflow
-                # orchestrator.  This may take a while, so we delegate it to
-                # a Celery task.
-                # Populate only root workflows. Sub-workflows are not populated
-                # directly from the scheduler.
-                if work_request.is_workflow_root:
-                    work_request.mark_running()
-                    run_workflow_task_via_celery(work_request)
-                    result.append(work_request)
+                # Workflow population always happens via the orchestrator
+                # for the root workflow.  Sub-workflow orchestrators are not
+                # called directly from the scheduler, although if a
+                # sub-workflow is pending then its root workflow may deal
+                # with populating it.
+                workflow_root = work_request.get_workflow_root()
+                # Workflows always have a root.
+                assert workflow_root is not None
+                workflow_roots.add(workflow_root)
 
             case (TaskTypes.WAIT, _):
                 # Wait tasks are marked running immediately, but otherwise
@@ -251,6 +239,14 @@ def schedule_internal() -> list[WorkRequest]:
                     f"Unexpected internal task name: {unreachable}"
                 )
 
+    for workflow_root in sorted(workflow_roots, key=attrgetter("id")):
+        # Workflows are populated by the corresponding workflow
+        # orchestrator.  This may take a while, so we delegate it to a
+        # Celery task.
+        workflow_root.mark_running()
+        run_workflow_task_via_celery(workflow_root)
+        result.append(workflow_root)
+
     return result
 
 
