@@ -9,7 +9,6 @@
 
 """Models used by debusine server-side tasks."""
 
-import datetime
 import re
 from typing import Any
 
@@ -134,7 +133,7 @@ class CreateExperimentWorkspaceData(BaseTaskData):
     public: bool = True
     owner_group: str | None = None
     workflow_template_names: list[str] = []
-    expiration_delay: datetime.timedelta | None = datetime.timedelta(days=60)
+    expiration_delay: int | None = 60
 
     @pydantic.validator("experiment_name")
     @classmethod
