@@ -12,6 +12,7 @@
 from debusine.server.tasks import BaseServerTask
 from debusine.server.tasks.models import ServerNoopData
 from debusine.tasks.models import BaseDynamicTaskData
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class ServerNoop(BaseServerTask[ServerNoopData, BaseDynamicTaskData]):
@@ -28,3 +29,10 @@ class ServerNoop(BaseServerTask[ServerNoopData, BaseDynamicTaskData]):
     def get_label(self) -> str:
         """Return the task label."""
         return "noop"
+
+    def build_dynamic_data(
+        self,
+        task_database: TaskDatabaseInterface,  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Resolve artifact lookups for this task."""
+        return BaseDynamicTaskData()
