@@ -46,14 +46,15 @@ class PackageUpload(
 
     TASK_VERSION = 1
 
-    def compute_dynamic_data(
-        self, task_database: TaskDatabaseInterface
+    def build_dynamic_data(
+        self,
+        task_database: TaskDatabaseInterface,
     ) -> PackageUploadDynamicData:
         """Resolve artifact lookups for this task."""
         return PackageUploadDynamicData(
             input_upload_id=task_database.lookup_single_artifact(
                 self.data.input.upload
-            ).id
+            ).id,
         )
 
     def fetch_upload(self, destination: Path) -> list[Path] | None:
