@@ -23,9 +23,12 @@ from typing import Any
 from unittest.mock import MagicMock, patch
 
 from debian.deb822 import Packages, Sources
+from django.db import connection, connections
+from django_pglocks import advisory_lock
 
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.db.context import context
+from debusine.db.locks import LockError, LockType
 from debusine.db.models import Artifact, ArtifactRelation, Collection, Workspace
 from debusine.db.tests.utils import _calculate_hash_from_data
 from debusine.server.collections.debian_suite import (
@@ -1433,8 +1436,25 @@ class APTMirrorTests(TestCase):
                 args=args, returncode=0, stdout=stdout, stderr=""
             )
 
-        with patch("subprocess.run", side_effect=fake_run):
-            self.assertTrue(task.execute())
+        # Pretend that a lock for another collection is held, to make sure
+        # that that doesn't interfere.
+        non_conflicting_alias = "non-conflicting-lock"
+        non_conflicting_connection = connection.copy(non_conflicting_alias)
+        connections[non_conflicting_alias] = non_conflicting_connection
+        try:
+            with (
+                advisory_lock(
+                    (LockType.APT_MIRROR, (collection.id + 1) & (2**31 - 1)),
+                    wait=False,
+                    using=non_conflicting_alias,
+                ) as acquired,
+                patch("subprocess.run", side_effect=fake_run),
+            ):
+                assert acquired
+                self.assertTrue(task.execute())
+        finally:
+            del connections[non_conflicting_alias]
+            non_conflicting_connection.close()
 
         self.assertQuerySetEqual(
             collection.child_items.values_list("name", flat=True),
@@ -1442,6 +1462,34 @@ class APTMirrorTests(TestCase):
             ordered=False,
         )
 
+    def test_execute_lock_error(self) -> None:
+        """The task fails if its lock is already held."""
+        collection = self.create_suite_collection("bookworm")
+        work_request = self.playground.create_work_request()
+        task = self.create_apt_mirror_task("bookworm")
+        task.set_work_request(work_request)
+
+        conflicting_alias = "conflicting-lock"
+        conflicting_connection = connection.copy(conflicting_alias)
+        connections[conflicting_alias] = conflicting_connection
+        try:
+            with (
+                advisory_lock(
+                    (LockType.APT_MIRROR, collection.id & (2**31 - 1)),
+                    wait=False,
+                    using=conflicting_alias,
+                ) as acquired,
+                self.assertRaisesRegex(
+                    LockError,
+                    "Another APTMirror task for bookworm is already running",
+                ),
+            ):
+                assert acquired
+                task.execute()
+        finally:
+            del connections[conflicting_alias]
+            conflicting_connection.close()
+
     def test_label(self) -> None:
         """Test get_label."""
         task = self.create_apt_mirror_task(
