@@ -18,18 +18,15 @@ from debusine.artifacts.local_artifact import WorkRequestDebugLogs
 from debusine.db.context import context
 from debusine.db.models import Artifact, ArtifactRelation, WorkRequest
 from debusine.db.models.permissions import format_permission_check_error
-from debusine.server.tasks.base import (
-    BaseServerTask,
-    ServerTaskPermissionDenied,
-)
+from debusine.server.tasks.base import ServerTaskPermissionDenied
+from debusine.server.tasks.tests.helpers import TestBaseServerTask
 from debusine.tasks.models import BaseDynamicTaskData, BaseTaskData, WorkerType
-from debusine.tasks.tests.helper_mixin import TestTaskMixin
 from debusine.test.django import TestCase
 from debusine.worker.system_information import host_architecture
 
 
-class TestBaseServerTask(
-    TestTaskMixin, BaseServerTask[BaseTaskData, BaseDynamicTaskData]
+class TestBaseServerTask1(
+    TestBaseServerTask[BaseTaskData, BaseDynamicTaskData]
 ):
     """Sample class to test BaseServerTask class."""
 
@@ -47,7 +44,7 @@ class TestBaseServerTask2Data(BaseTaskData):
 
 
 class TestBaseServerTask2(
-    TestTaskMixin, BaseServerTask[TestBaseServerTask2Data, BaseDynamicTaskData]
+    TestBaseServerTask[TestBaseServerTask2Data, BaseDynamicTaskData]
 ):
     """Test BaseServerTask class with jsonschema validation."""
 
@@ -61,7 +58,7 @@ class TestBaseServerTask2(
 class BaseServerTaskTests(TestCase):
     """Unit tests for :class:`BaseServerTask`."""
 
-    task: ClassVar[TestBaseServerTask]
+    task: ClassVar[TestBaseServerTask1]
     task2: ClassVar[TestBaseServerTask2]
     worker_metadata: ClassVar[dict[str, WorkerType]]
 
@@ -69,7 +66,7 @@ class BaseServerTaskTests(TestCase):
     def setUpTestData(cls) -> None:
         """Create the shared attributes."""
         super().setUpTestData()
-        cls.task = TestBaseServerTask({})
+        cls.task = TestBaseServerTask1({})
         cls.task2 = TestBaseServerTask2({"foo": "bar"})
         cls.worker_metadata = {"system:worker_type": WorkerType.CELERY}
 
@@ -83,7 +80,7 @@ class BaseServerTaskTests(TestCase):
         """Ensure can_run_on returns False if versions differ."""
         self.assertIsNone(self.task.TASK_VERSION)
         metadata = {**self.worker_metadata, **self.task.analyze_worker()}
-        metadata["server:testbaseservertask:version"] = 1
+        metadata["server:testbaseservertask1:version"] = 1
         self.assertEqual(self.task.can_run_on(metadata), False)
 
     def test_set_work_request(self) -> None:
@@ -112,7 +109,7 @@ class BaseServerTaskTests(TestCase):
             return True
 
         with mock.patch.object(
-            TestBaseServerTask, "_execute", side_effect=fake_execute
+            TestBaseServerTask1, "_execute", side_effect=fake_execute
         ):
             self.task.execute()
 
