@@ -532,9 +532,7 @@ class CopyCollectionItemsTests(TestCase):
         template = self.playground.create_workflow_template(
             name="test", task_name="noop", workspace=self.source_workspace
         )
-        workflow = WorkRequest.objects.create_workflow(
-            template=template, data={}, created_by=self.user
-        )
+        workflow = self.playground.create_workflow(template, task_data={})
         assert workflow.internal_collection is not None
         source_package = self.playground.create_source_artifact(
             workspace=self.source_workspace
