@@ -26,7 +26,7 @@ from debusine.tasks.models import EventReactions, LintianData
 
 
 class UpdateSuiteLintianCollection(
-    UpdateDerivedCollection[UpdateSuiteLintianCollectionData]
+    UpdateDerivedCollection[UpdateSuiteLintianCollectionData],
 ):
     """Task that updates a debian:suite-lintian collection from debian:suite."""
 
