@@ -9,8 +9,10 @@
 
 """Unit tests for the ExternalDebsign task."""
 
+from debusine.artifacts.models import ArtifactCategory, DebianUpload
 from debusine.server.tasks.wait import ExternalDebsign
 from debusine.server.tasks.wait.models import ExternalDebsignDynamicData
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import FakeTaskDatabase
 from debusine.test.django import TestCase
 
@@ -24,7 +26,17 @@ class ExternalDebsignTaskTests(TestCase):
         task_db = FakeTaskDatabase(
             single_lookups={
                 # unsigned
-                (unsigned_lookup, None): 1,
+                (unsigned_lookup, None): ArtifactInfo(
+                    1,
+                    ArtifactCategory.UPLOAD,
+                    DebianUpload(
+                        type="dpkg",
+                        changes_fields={
+                            "Architecture": "source",
+                            "Files": [{"name": "hello_1.0.dsc"}],
+                        },
+                    ),
+                ),
             }
         )
 
