@@ -21,6 +21,7 @@ from debusine.server.celery import (
     ServerTaskRunError,
     WorkRequestNotPending,
     run_server_task,
+    update_workflows,
 )
 from debusine.server.tasks.models import ServerNoopData
 from debusine.server.tasks.tests.helpers import TestBaseServerTask
@@ -347,3 +348,30 @@ class RunServerTaskTransactionTests(TransactionTestCase):
         result = run_server_task.apply(args=(work_request.id,))
         self.assertFalse(result.failed())
         self.assertTrue(result.result)
+
+
+class UpdateWorkflowTransactionTests(TransactionTestCase):
+    """Unit tests for :py:func:`update_workflows`."""
+
+    def test_updates_workflows(self) -> None:
+        """The Celery task updates workflow properties."""
+        with mock.patch("django.db.transaction.on_commit"):
+            workflow = self.playground.create_workflow()
+            work_request = workflow.create_child("noop")
+            self.assertTrue(work_request.mark_aborted())
+            self.assertTrue(work_request.workflows_need_update)
+
+        result = update_workflows.apply()
+        self.assertFalse(result.failed())
+        self.assertIsNone(result.result)
+
+        workflow.refresh_from_db()
+        work_request.refresh_from_db()
+        self.assertEqual(
+            workflow.workflow_last_activity_at, work_request.completed_at
+        )
+        self.assertEqual(
+            workflow.workflow_runtime_status,
+            WorkRequest.RuntimeStatuses.ABORTED,
+        )
+        self.assertFalse(work_request.workflows_need_update)
