@@ -9,6 +9,8 @@
 
 """Tests for queue and worker pool summaries."""
 
+from itertools import product
+
 from django.test import override_settings
 
 from debusine.db.models import WorkRequest
@@ -137,24 +139,31 @@ class WorkerStatusTests(TestCase):
 
         For a WorkRequest with host_architecture specified in task_data.
         """
-        self.playground.create_work_request(
-            task_name="noop",
-            task_data={"host_architecture": "amd64"},
-            status=WorkRequest.Statuses.PENDING,
-        )
-        self.playground.create_work_request(
-            task_name="noop",
-            task_data={"host_architecture": "amd64"},
-            status=WorkRequest.Statuses.RUNNING,
-        )
+        for task_type, wr_status in product(
+            [TaskTypes.WORKER, TaskTypes.SIGNING],
+            [WorkRequest.Statuses.PENDING, WorkRequest.Statuses.RUNNING],
+        ):
+            self.playground.create_work_request(
+                task_type=task_type,
+                task_name="noop",
+                task_data={"host_architecture": "amd64"},
+                status=wr_status,
+            )
 
         status = WorkerStatus.get_status()
         self.assertWorkerTasksArch(status, "amd64", "pending", 1)
         self.assertWorkerTasksArch(status, "amd64", "running", 1)
         self.assertTasks(status, TaskTypes.WORKER, "pending", 1)
         self.assertTasks(status, TaskTypes.WORKER, "running", 1)
+        self.assertTasks(status, TaskTypes.SIGNING, "pending", 1)
+        self.assertTasks(status, TaskTypes.SIGNING, "running", 1)
         self.assertWorkerTasks(status, WorkerType.EXTERNAL, "pending", 1)
         self.assertWorkerTasks(status, WorkerType.EXTERNAL, "running", 1)
+        self.assertWorkerTasks(status, WorkerType.SIGNING, "pending", 1)
+        self.assertWorkerTasks(status, WorkerType.SIGNING, "running", 1)
+
+    def test_work_request_filters_by_task_type(self) -> None:
+        """Test that worker_tasks_arch only includes WORKER work requests."""
 
     def test_work_request_counts_host_architecture_method(self) -> None:
         """
@@ -517,6 +526,13 @@ class WorkerStatusTests(TestCase):
             worker=worker_arm64_1,
         )
 
+        # Create a running wait task
+        self.playground.create_work_request(
+            task_type=TaskTypes.WAIT,
+            task_name="externaldebsign",
+            status=WorkRequest.Statuses.RUNNING,
+        )
+
         actual = WorkerStatus.get_status()
 
         expected = QueueStatus(
@@ -526,7 +542,7 @@ class WorkerStatusTests(TestCase):
                 TaskTypes.INTERNAL: TaskQueueSummary(),
                 TaskTypes.WORKFLOW: TaskQueueSummary(),
                 TaskTypes.SIGNING: TaskQueueSummary(),
-                TaskTypes.WAIT: TaskQueueSummary(),
+                TaskTypes.WAIT: TaskQueueSummary(running=1),
             },
             workers={
                 WorkerType.CELERY: WorkerSummary(
