@@ -18,6 +18,7 @@ from debusine.server.views.artifacts import (
 )
 from debusine.server.views.assets import AssetPermissionCheckView, AssetView
 from debusine.server.views.base import Whoami
+from debusine.server.views.enroll import EnrollView
 from debusine.server.views.lookups import LookupMultipleView, LookupSingleView
 from debusine.server.views.service_status import ServiceStatusView
 from debusine.server.views.work_requests import (
@@ -42,6 +43,7 @@ urlpatterns = [
         'auth/',
         include('rest_framework.urls', namespace='rest_framework'),
     ),
+    path("enroll/", EnrollView.as_view(), name="enroll"),
     path(
         '1.0/whoami/',
         Whoami.as_view(),
