@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -24,7 +24,8 @@ from debusine.server.serializers import (
     LookupResponseSerializer,
     LookupSingleSerializer,
 )
-from debusine.server.views.base import BaseAPIView, IsWorkerAuthenticated
+from debusine.server.views.base import BaseAPIView
+from debusine.server.views.rest import IsWorkerAuthenticated
 from debusine.tasks.models import LookupMultiple
 
 logger = logging.getLogger(__name__)
