@@ -35,7 +35,7 @@ class AutopkgtestWorkflow(
 
     TASK_NAME = "autopkgtest"
 
-    def __init__(self, work_request: "WorkRequest") -> None:
+    def __init__(self, work_request: WorkRequest) -> None:
         """Instantiate a Workflow with its database instance."""
         super().__init__(work_request)
         if self.data.backend == BackendType.AUTO:
@@ -90,6 +90,18 @@ class AutopkgtestWorkflow(
                 self, self.data.context_artifacts, (architecture, "all")
             )
         )
+
+        environment = f"{self.data.vendor}/match:codename={self.data.codename}"
+        extra_repositories = workflow_utils.configure_for_overlay_suite(
+            self,
+            extra_repositories=self.data.extra_repositories,
+            vendor=self.data.vendor,
+            codename=self.data.codename,
+            environment=environment,
+            backend=self.data.backend,
+            architecture=architecture,
+        )
+
         task_data = AutopkgtestData(
             input=AutopkgtestInput(
                 source_artifact=source_artifact,
@@ -97,13 +109,11 @@ class AutopkgtestWorkflow(
                 context_artifacts=filtered_context_artifacts,
             ),
             host_architecture=architecture,
-            environment=(
-                f"{self.data.vendor}/match:codename={self.data.codename}"
-            ),
+            environment=environment,
             backend=self.data.backend,
             include_tests=self.data.include_tests,
             exclude_tests=self.data.exclude_tests,
-            extra_repositories=self.data.extra_repositories,
+            extra_repositories=extra_repositories,
             debug_level=self.data.debug_level,
             extra_environment=self.data.extra_environment,
             needs_internet=self.data.needs_internet,
