@@ -15,8 +15,11 @@ from celery import shared_task
 from django.db import transaction
 
 from debusine.db.models import WorkRequest
-from debusine.server.workflows.base import orchestrate_workflow
-from debusine.tasks.models import TaskTypes
+from debusine.server.workflows.base import (
+    WorkflowRunError,
+    orchestrate_workflow,
+)
+from debusine.tasks.models import OutputData, OutputDataError, TaskTypes
 
 logger = logging.getLogger(__name__)
 
@@ -39,15 +42,29 @@ def run_workflow_task(work_request_id: int) -> bool:
     ):
         # This task must only be called to populate root workflows.
         # Sub-workflows are populated by their parent workflows instead.
+        message = "Must be populated by its parent workflow instead"
         logger.error(
-            "Error running work request %s/%s (%s): must be populated by its "
-            "parent workflow instead",
+            "Error running work request %s/%s (%s): %s",
             work_request.task_type,
             work_request.task_name,
             work_request.id,
+            message,
+        )
+        work_request.mark_completed(
+            WorkRequest.Results.ERROR,
+            output_data=OutputData(
+                errors=[
+                    OutputDataError(message=message, code="non-root-workflow")
+                ]
+            ),
         )
-        work_request.mark_completed(WorkRequest.Results.ERROR)
         return False
 
-    orchestrate_workflow(work_request)
-    return True
+    try:
+        orchestrate_workflow(work_request)
+    except WorkflowRunError:
+        # The details have been recorded in the work request's output data,
+        # so we don't need to fail the Celery task.
+        return False
+    else:
+        return True
