@@ -47,6 +47,11 @@ class LintianWorkflow(Workflow[LintianWorkflowData, BaseDynamicTaskData]):
         environment = f"{self.data.vendor}/match:codename={self.data.codename}"
 
         for arch in architectures:
+            source_artifact = (
+                workflow_utils.locate_debian_source_package_lookup(
+                    self, "source_artifact", self.data.source_artifact
+                )
+            )
             filtered_binary_artifacts = (
                 workflow_utils.filter_artifact_lookup_by_arch(
                     self, self.data.binary_artifacts, (arch, "all")
@@ -54,7 +59,7 @@ class LintianWorkflow(Workflow[LintianWorkflowData, BaseDynamicTaskData]):
             )
 
             self._populate_lintian(
-                source_artifact=self.data.source_artifact,
+                source_artifact=source_artifact,
                 binary_artifacts=filtered_binary_artifacts,
                 output=self.data.output,
                 environment=environment,
