@@ -9,11 +9,16 @@
 
 """Make signed source workflow."""
 
+from functools import cached_property
+
 from debusine.artifacts.models import ArtifactCategory
 from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.models import WorkRequest
-from debusine.server.collections.lookup import lookup_multiple
+from debusine.server.collections.lookup import (
+    lookup_multiple,
+    reconstruct_lookup,
+)
 from debusine.server.workflows import Workflow, workflow_utils
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
@@ -75,24 +80,42 @@ class MakeSignedSourceWorkflow(
         )
         return BaseDynamicTaskData(subject=" ".join(source_package_names))
 
-    def populate(self) -> None:
-        """Create work requests and sub-workflows."""
-        environment = f"{self.data.vendor}/match:codename={self.data.codename}"
-
-        architectures = workflow_utils.get_architectures(
-            self, self.data.signing_template_artifacts
-        ).intersection(
-            workflow_utils.get_architectures(self, self.data.binary_artifacts)
-        )
-
-        architectures = architectures.intersection(self.data.architectures)
-
-        for arch in architectures:
-            signing_templates_lookup = (
-                workflow_utils.filter_artifact_lookup_by_arch(
-                    self, self.data.signing_template_artifacts, (arch,)
+    @cached_property
+    def architectures(self) -> set[str]:
+        """Architectures to run on."""
+        return (
+            workflow_utils.get_architectures(
+                self, self.data.signing_template_artifacts
+            )
+            .intersection(
+                workflow_utils.get_architectures(
+                    self, self.data.binary_artifacts
                 )
             )
+            .intersection(self.data.architectures)
+        )
+
+    def populate(self) -> None:
+        """Create work requests and sub-workflows."""
+        workflow_root = self.work_request.get_workflow_root()
+        environment = f"{self.data.vendor}/match:codename={self.data.codename}"
+
+        signing_templates = lookup_multiple(
+            self.data.signing_template_artifacts,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=workflow_root,
+            expect_type=LookupChildType.ARTIFACT_OR_PROMISE,
+        )
+
+        for arch in self.architectures:
+            signing_template_lookups_by_name = {
+                workflow_utils.lookup_result_binary_package_name(
+                    result
+                ): reconstruct_lookup(result, workflow_root=workflow_root)
+                for result in signing_templates
+                if workflow_utils.lookup_result_architecture(result) == arch
+            }
 
             binary_artifacts_lookups = (
                 workflow_utils.filter_artifact_lookup_by_arch(
@@ -102,7 +125,9 @@ class MakeSignedSourceWorkflow(
                 )
             )
 
-            for signing_template_lookup in signing_templates_lookup:
+            for signing_template_name, signing_template_lookup in sorted(
+                signing_template_lookups_by_name.items()
+            ):
                 # filter_artifact_lookup_by_arch always returns a parsed list of
                 # single lookups.
                 assert not isinstance(signing_template_lookup, LookupDict)
@@ -113,6 +138,7 @@ class MakeSignedSourceWorkflow(
                         binary_artifacts=binary_artifacts_lookups,
                         environment=environment,
                         architecture=arch,
+                        template_name=signing_template_name,
                     )
                 )
 
@@ -122,6 +148,7 @@ class MakeSignedSourceWorkflow(
                     key=self.data.key,
                     extract_for_signing_wr=extract_for_signing_wr,
                     architecture=arch,
+                    template_name=signing_template_name,
                 )
 
                 assembled = self._populate_assemble_signed_source(
@@ -130,11 +157,12 @@ class MakeSignedSourceWorkflow(
                     signed=signed,
                     sign_wr=sign_wr,
                     architecture=arch,
+                    template_name=signing_template_name,
                 )
 
                 target_distribution = f"{self.data.vendor}:{self.data.codename}"
 
-                sbuild_architectures = sorted(architectures | {"all"})
+                sbuild_architectures = sorted(self.architectures | {"all"})
                 self._populate_sbuild_workflow(
                     assembled=assembled,
                     extra_binary_artifacts=binary_artifacts_lookups,
@@ -142,6 +170,7 @@ class MakeSignedSourceWorkflow(
                     backend=self.data.sbuild_backend,
                     architectures=sbuild_architectures,
                     architecture=arch,
+                    template_name=signing_template_name,
                 )
 
     def _populate_extract_for_signing(
@@ -151,6 +180,7 @@ class MakeSignedSourceWorkflow(
         binary_artifacts: LookupMultiple,
         environment: str,
         architecture: str,
+        template_name: str,
     ) -> tuple[WorkRequest, LookupMultiple]:
         """
         Create work request for ExtractForSigning.
@@ -168,8 +198,10 @@ class MakeSignedSourceWorkflow(
                 ),
             ),
             workflow_data=WorkRequestWorkflowData(
-                display_name=f"Extract for signing {architecture}",
-                step=f"extract-for-signing-{architecture}",
+                display_name=(
+                    f"Extract for signing ({template_name}/{architecture})"
+                ),
+                step=f"extract-for-signing-{architecture}-{template_name}",
             ),
         )
         self.requires_artifact(wr, template_artifact)
@@ -210,6 +242,7 @@ class MakeSignedSourceWorkflow(
         key: str,
         extract_for_signing_wr: WorkRequest,
         architecture: str,
+        template_name: str,
     ) -> tuple[WorkRequest, LookupMultiple]:
         """
         Create work request for Sign.
@@ -222,8 +255,8 @@ class MakeSignedSourceWorkflow(
             task_name="sign",
             task_data=SignData(purpose=purpose, unsigned=unsigned, key=key),
             workflow_data=WorkRequestWorkflowData(
-                display_name=f"Sign {architecture}",
-                step=f"sign-{architecture}",
+                display_name=f"Sign ({template_name}/{architecture})",
+                step=f"sign-{architecture}-{template_name}",
             ),
         )
 
@@ -233,12 +266,16 @@ class MakeSignedSourceWorkflow(
             "on_success",
             ActionUpdateCollectionWithArtifacts(
                 collection="internal@collections",
-                name_template="signed-{architecture}-{binary_package_name}",
+                name_template=(
+                    "signed-{architecture}-{template_name}-"
+                    "{binary_package_name}"
+                ),
                 variables={
                     # Extract the binary package name from the artifact
                     # data, since the workflow doesn't know it up-front.
                     "$binary_package_name": "binary_package_name",
                     "architecture": architecture,
+                    "template_name": template_name,
                 },
                 artifact_filters={"category": ArtifactCategory.SIGNING_OUTPUT},
             ),
@@ -248,7 +285,7 @@ class MakeSignedSourceWorkflow(
                 "collection": "internal@collections",
                 "child_type": LookupChildType.ARTIFACT,
                 "category": ArtifactCategory.SIGNING_OUTPUT,
-                "name__startswith": "signed-",
+                "name__startswith": f"signed-{architecture}-{template_name}-",
                 "data__architecture": architecture,
             }
         )
@@ -262,6 +299,7 @@ class MakeSignedSourceWorkflow(
         signed: LookupMultiple,
         sign_wr: WorkRequest,
         architecture: str,
+        template_name: str,
     ) -> LookupSingle:
         """
         Create work request for assembling the signed source.
@@ -274,17 +312,17 @@ class MakeSignedSourceWorkflow(
                 environment=environment, template=template, signed=signed
             ),
             workflow_data=WorkRequestWorkflowData(
-                display_name=f"Assemble signed source {architecture}",
-                step=f"assemble-signed-source-{architecture}",
+                display_name=(
+                    f"Assemble signed source ({template_name}/{architecture})"
+                ),
+                step=f"assemble-signed-source-{architecture}-{template_name}",
             ),
         )
 
         wr.add_dependency(sign_wr)
 
-        artifact_name = f"signed-source-{architecture}"
-        self.provides_artifact(
-            wr, ArtifactCategory.SOURCE_PACKAGE, artifact_name
-        )
+        artifact_name = f"signed-source-{architecture}-{template_name}"
+        self.provides_artifact(wr, ArtifactCategory.UPLOAD, artifact_name)
         return f"internal@collections/name:{artifact_name}"
 
     def _populate_sbuild_workflow(
@@ -295,13 +333,14 @@ class MakeSignedSourceWorkflow(
         backend: BackendType,
         architectures: list[str],
         architecture: str,
+        template_name: str,
     ) -> None:
         """Populate SbuildWorkflow."""
         wr = self.work_request_ensure_child(
             task_type=TaskTypes.WORKFLOW,
             task_name="sbuild",
             task_data=SbuildWorkflowData(
-                prefix="signed-source|",
+                prefix=f"signed-source-{architecture}-{template_name}|",
                 input=SbuildInput(
                     source_artifact=assembled,
                     extra_binary_artifacts=extra_binary_artifacts,
@@ -311,8 +350,8 @@ class MakeSignedSourceWorkflow(
                 architectures=architectures,
             ),
             workflow_data=WorkRequestWorkflowData(
-                display_name=f"Sbuild {architecture}",
-                step=f"sbuild-{architecture}",
+                display_name=f"Sbuild ({template_name}/{architecture})",
+                step=f"sbuild-{architecture}-{template_name}",
             ),
         )
 
