@@ -141,6 +141,7 @@ class QAWorkflow(Workflow[QAWorkflowData, BaseDynamicTaskData]):
                 codename=self.data.codename,
                 architectures=effective_architectures,
                 backend=self.data.piuparts_backend,
+                environment=self.data.piuparts_environment,
                 arch_all_host_architecture=self.data.arch_all_host_architecture,
             )
 
@@ -264,20 +265,24 @@ class QAWorkflow(Workflow[QAWorkflowData, BaseDynamicTaskData]):
         codename: str,
         architectures: list[str],
         backend: BackendType,
+        environment: LookupSingle | None,
         arch_all_host_architecture: str,
     ) -> None:
+        data = PiupartsWorkflowData(
+            binary_artifacts=binary_artifacts,
+            vendor=vendor,
+            codename=codename,
+            architectures=architectures,
+            backend=backend,
+            arch_all_host_architecture=arch_all_host_architecture,
+            extra_repositories=self.data.extra_repositories,
+        )
+        if environment is not None:
+            data.environment = environment
         wr = self.work_request_ensure_child(
             task_name="piuparts",
             task_type=TaskTypes.WORKFLOW,
-            task_data=PiupartsWorkflowData(
-                binary_artifacts=binary_artifacts,
-                vendor=vendor,
-                codename=codename,
-                architectures=architectures,
-                backend=backend,
-                arch_all_host_architecture=arch_all_host_architecture,
-                extra_repositories=self.data.extra_repositories,
-            ),
+            task_data=data,
             workflow_data=WorkRequestWorkflowData(
                 display_name="piuparts",
                 step="piuparts",
