@@ -13,7 +13,7 @@ import json
 import re
 from collections.abc import Iterable
 from dataclasses import dataclass
-from functools import partial
+from functools import cached_property, partial
 from itertools import chain
 from typing import Any, Self
 
@@ -38,10 +38,14 @@ from debusine.artifacts.models import (
     DebianUpload,
 )
 from debusine.client.models import LookupChildType
-from debusine.db.models import CollectionItem, WorkRequest
+from debusine.db.models import Collection, CollectionItem, WorkRequest
 from debusine.server.collections.lookup import lookup_multiple, lookup_single
 from debusine.server.workflows import workflow_utils
-from debusine.server.workflows.base import Workflow, orchestrate_workflow
+from debusine.server.workflows.base import (
+    Workflow,
+    WorkflowValidationError,
+    orchestrate_workflow,
+)
 from debusine.server.workflows.models import (
     AutopkgtestWorkflowData,
     ReverseDependenciesAutopkgtestWorkflowData,
@@ -169,6 +173,25 @@ class ReverseDependenciesAutopkgtestWorkflow(
         if self.data.backend == BackendType.AUTO:
             self.data.backend = BackendType.UNSHARE
 
+    @cached_property
+    def suite_collection(self) -> Collection:
+        """The collection to search for reverse-dependencies."""
+        return lookup_single(
+            self.data.suite_collection,
+            self.workspace,
+            user=self.work_request.created_by,
+            workflow_root=self.work_request.get_workflow_root(),
+            expect_type=LookupChildType.COLLECTION,
+        ).collection
+
+    def validate_input(self) -> None:
+        """Thorough validation of input data."""
+        # Validate that we can look up self.data.suite_collection.
+        try:
+            self.suite_collection
+        except LookupError as e:
+            raise WorkflowValidationError(str(e)) from e
+
     def get_binary_names(self) -> set[str]:
         """Return names of binary artifacts whose rdeps we want to test."""
         binary_names: set[str] = set()
@@ -222,16 +245,9 @@ class ReverseDependenciesAutopkgtestWorkflow(
         source_artifact_data = source_artifact.create_data()
         assert isinstance(source_artifact_data, DebianSourcePackage)
 
-        suite_collection = lookup_single(
-            self.data.suite_collection,
-            self.workspace,
-            user=self.work_request.created_by,
-            workflow_root=self.work_request.get_workflow_root(),
-            expect_type=LookupChildType.COLLECTION,
-        ).collection
         suite_artifacts = partial(
             CollectionItem.active_objects.all().artifacts_in_collection,
-            parent_collection=suite_collection,
+            parent_collection=self.suite_collection,
         )
 
         binary_names = self.get_binary_names()
@@ -394,7 +410,7 @@ class ReverseDependenciesAutopkgtestWorkflow(
             task_type=TaskTypes.WORKFLOW,
             task_name="autopkgtest",
             task_data=AutopkgtestWorkflowData(
-                prefix=f"{source.name}_{source.version}/",
+                prefix=f"{source.name}_{source.version}|",
                 source_artifact=source.to_lookup(),
                 binary_artifacts=LookupMultiple.parse_obj(
                     sorted([binary.to_lookup() for binary in binaries])
