@@ -578,6 +578,30 @@ class AutopkgtestWorkflowTests(TestCase):
             children = list(WorkRequest.objects.filter(parent=autopkgtest))
             self.assertEqual(len(children), len(architectures))
 
+    def test_populate_experimental(self) -> None:
+        """The workflow handles overlay distributions."""
+        source_artifact = self.playground.create_source_artifact()
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+        self.playground.create_debian_environment(codename="experimental")
+        wr = self.playground.create_workflow(
+            task_name="autopkgtest",
+            task_data=AutopkgtestWorkflowData(
+                source_artifact=source_artifact.id,
+                binary_artifacts=LookupMultiple.parse_obj([binary_artifact.id]),
+                vendor="debian",
+                codename="experimental",
+            ),
+        )
+        AutopkgtestWorkflow(wr).populate()
+        children = list(wr.children.all())
+        self.assertEqual(len(children), 1)
+        child = children[0]
+        self.assertEqual(len(child.task_data["extra_repositories"]), 1)
+        repo = child.task_data["extra_repositories"][0]
+        self.assertEqual(repo["suite"], "experimental")
+
     def test_compute_dynamic_data(self) -> None:
         source_artifact = self.playground.create_source_artifact(
             name="hello", version="1.0-1"
