@@ -10,12 +10,19 @@
 """Unit tests for the debian pipeline workflow."""
 from typing import Any, ClassVar
 
+from django.test import override_settings
+
 from debusine.artifacts.models import ArtifactCategory, CollectionCategory
 from debusine.assets import KeyPurpose
 from debusine.client.models import LookupChildType
 from debusine.db.context import context
 from debusine.db.models import Artifact, TaskDatabase, WorkRequest
-from debusine.server.workflows import DebianPipelineWorkflow
+from debusine.server.scheduler import schedule
+from debusine.server.workflows import (
+    DebianPipelineWorkflow,
+    WorkflowValidationError,
+    workflow_utils,
+)
 from debusine.server.workflows.base import (
     WorkflowRunError,
     orchestrate_workflow,
@@ -46,7 +53,10 @@ class DebianPipelineWorkflowTests(TestCase):
         )
 
     def create_debian_pipeline_workflow(
-        self, *, extra_task_data: dict[str, Any]
+        self,
+        *,
+        extra_task_data: dict[str, Any] | None = None,
+        validate: bool = True,
     ) -> DebianPipelineWorkflow:
         """Create a debian pipeline workflow."""
         task_data = {
@@ -54,9 +64,10 @@ class DebianPipelineWorkflowTests(TestCase):
             "vendor": "debian",
             "codename": "bookworm",
         }
-        task_data.update(extra_task_data)
+        if extra_task_data is not None:
+            task_data.update(extra_task_data)
         wr = self.playground.create_workflow(
-            task_name="debian_pipeline", task_data=task_data
+            task_name="debian_pipeline", task_data=task_data, validate=validate
         )
         return DebianPipelineWorkflow(wr)
 
@@ -82,6 +93,30 @@ class DebianPipelineWorkflowTests(TestCase):
 
         return wr
 
+    def test_validate_input(self) -> None:
+        """validate_input passes a valid case."""
+        workflow = self.create_debian_pipeline_workflow()
+
+        workflow.validate_input()
+
+    def test_validate_input_bad_suite_collection(self) -> None:
+        """validate_input raises errors in looking up a suite."""
+        workflow = self.create_debian_pipeline_workflow(
+            extra_task_data={
+                "enable_reverse_dependencies_autopkgtest": True,
+                "reverse_dependencies_autopkgtest_suite": (
+                    "nonexistent@debian:suite"
+                ),
+            },
+            validate=False,
+        )
+
+        with self.assertRaisesRegex(
+            WorkflowValidationError,
+            "'nonexistent@debian:suite' does not exist or is hidden",
+        ):
+            workflow.validate_input()
+
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data includes a parameter summary."""
         source_artifact = self.playground.create_source_artifact(
@@ -136,7 +171,9 @@ class DebianPipelineWorkflowTests(TestCase):
             ),
         )
         workflow = DebianPipelineWorkflow(wr)
-        self.assertEqual(workflow.source_package, upload_artifacts.source)
+        self.assertEqual(
+            workflow_utils.source_package(workflow), upload_artifacts.source
+        )
 
     def test_populate_use_available_architectures(self) -> None:
         """
@@ -452,6 +489,50 @@ class DebianPipelineWorkflowTests(TestCase):
             },
         )
 
+    def test_populate_qa_piuparts_configuration(self) -> None:
+        """Test populate create qa: piuparts non-default configuration."""
+        workflow = self.orchestrate(
+            extra_data={
+                "architectures": ["amd64"],
+                "source_artifact": self.source_artifact.id,
+                "enable_piuparts": True,
+                "piuparts_backend": BackendType.INCUS_LXC,
+                "piuparts_environment": "debian/match:codename=trixie",
+            }
+        )
+
+        self.assert_qa(
+            workflow,
+            {
+                "arch_all_host_architecture": "amd64",
+                "architectures": ["amd64"],
+                "autopkgtest_backend": BackendType.AUTO,
+                "binary_artifacts": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "build-",
+                        },
+                    }
+                ],
+                "codename": "bookworm",
+                "extra_repositories": None,
+                "enable_autopkgtest": False,
+                "enable_lintian": False,
+                "enable_piuparts": True,
+                "enable_reverse_dependencies_autopkgtest": False,
+                "lintian_backend": BackendType.AUTO,
+                "lintian_fail_on_severity": "none",
+                "piuparts_backend": BackendType.INCUS_LXC,
+                "piuparts_environment": "debian/match:codename=trixie",
+                "reverse_dependencies_autopkgtest_suite": None,
+                "source_artifact": self.source_artifact.id,
+                "vendor": "debian",
+            },
+        )
+
     def test_populate_architectures_allow_deny(self) -> None:
         """Populate uses architectures, architectures_allow/deny."""
         workflow = self.orchestrate(
@@ -485,7 +566,7 @@ class DebianPipelineWorkflowTests(TestCase):
     ) -> None:
         """Populate raise WorkflowRunError: missing required field."""
         msg = (
-            'orchestrator failed: "make_signed_source_purpose" must be '
+            'Orchestrator failed: "make_signed_source_purpose" must be '
             'set when signing the source'
         )
         with self.assertRaisesRegex(WorkflowRunError, msg):
@@ -503,7 +584,7 @@ class DebianPipelineWorkflowTests(TestCase):
     ) -> None:
         """Populate raise WorkflowRunError: missing required field."""
         msg = (
-            'orchestrator failed: "make_signed_source_key" must be set '
+            'Orchestrator failed: "make_signed_source_key" must be set '
             'when signing the source'
         )
         with self.assertRaisesRegex(WorkflowRunError, msg):
@@ -617,10 +698,6 @@ class DebianPipelineWorkflowTests(TestCase):
             extra_data={
                 "source_artifact": self.source_artifact.id,
                 "architectures": ["amd64", "i386"],
-                "enable_make_signed_source": True,
-                "make_signed_source_purpose": KeyPurpose.OPENPGP,
-                "signing_template_names": {"amd64": ["hello"]},
-                "make_signed_source_key": "ABC123",
                 "enable_upload": True,
                 "vendor": "debian",
                 "codename": "trixie",
@@ -722,16 +799,280 @@ class DebianPipelineWorkflowTests(TestCase):
             * 2,
         )
 
+    def test_populate_package_upload_signed_source(self) -> None:
+        """Populate creates package_upload workflows for signed source."""
+        workflow = self.orchestrate(
+            extra_data={
+                "source_artifact": self.source_artifact.id,
+                "architectures": ["amd64", "i386"],
+                "enable_make_signed_source": True,
+                "make_signed_source_purpose": KeyPurpose.OPENPGP,
+                "signing_template_names": {"amd64": ["hello"]},
+                "make_signed_source_key": "ABC123",
+                "enable_upload": True,
+                "vendor": "debian",
+                "codename": "trixie",
+                "upload_merge_uploads": False,
+                "upload_since_version": "1.1",
+                "upload_target": "ftp://user@example.org/pub/UploadQueue1/",
+                "upload_target_distribution": "debian:testing",
+            }
+        )
+
+        initial_sbuild_workflow = workflow.children.get(
+            task_name="sbuild", task_type=TaskTypes.WORKFLOW
+        )
+        initial_sbuild_amd64 = initial_sbuild_workflow.children.get(
+            task_name="sbuild",
+            task_type=TaskTypes.WORKER,
+            task_data__host_architecture="amd64",
+        )
+        make_signed_source = workflow.children.get(
+            task_name="make_signed_source", task_type=TaskTypes.WORKFLOW
+        )
+        extract_for_signing = make_signed_source.children.get(
+            task_name="extractforsigning", task_type=TaskTypes.WORKER
+        )
+        sign = make_signed_source.children.get(
+            task_name="sign", task_type=TaskTypes.SIGNING
+        )
+        assemble_signed_source = make_signed_source.children.get(
+            task_name="assemblesignedsource", task_type=TaskTypes.WORKER
+        )
+        package_upload_source, package_upload_signed_amd64 = (
+            workflow.children.filter(
+                task_name="package_upload", task_type=TaskTypes.WORKFLOW
+            ).order_by("id")
+        )
+
+        self.assertEqual(
+            package_upload_source.task_data,
+            {
+                "binary_artifacts": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "build-",
+                        },
+                    }
+                ],
+                "codename": "trixie",
+                "delayed_days": None,
+                "key": None,
+                "merge_uploads": False,
+                "require_signature": True,
+                "since_version": "1.1",
+                "source_artifact": self.source_artifact.id,
+                "target": "ftp://user@example.org/pub/UploadQueue1/",
+                "target_distribution": "debian:testing",
+                "vendor": "debian",
+            },
+        )
+
+        self.assertEqual(
+            package_upload_source.workflow_data_json,
+            {"display_name": "package upload", "step": "package_upload"},
+        )
+        self.assertQuerySetEqual(package_upload_source.dependencies.all(), [])
+
+        # PackageUpload.populate() was called and created its tasks.
+        # MakeSourcePackageUpload only requires the source artifact and so
+        # can start immediately.  Everything else is blocked.
+        self.assertQuerySetEqual(
+            package_upload_source.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
+            ),
+            # Work requests to construct, sign, and upload the source package.
+            [
+                (
+                    TaskTypes.WORKER,
+                    "makesourcepackageupload",
+                    WorkRequest.Statuses.PENDING,
+                    None,
+                    None,
+                ),
+                (
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "makesourcepackageupload",
+                ),
+                (
+                    TaskTypes.SERVER,
+                    "packageupload",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                ),
+            ]
+            # Work requests to sign and upload each of the two binary packages.
+            + [
+                (
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
+                ),
+                (
+                    TaskTypes.SERVER,
+                    "packageupload",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                ),
+            ]
+            * 2,
+        )
+
+        self.assertEqual(
+            package_upload_signed_amd64.task_data,
+            {
+                "binary_artifacts": [
+                    {
+                        "collection": "internal@collections",
+                        "child_type": LookupChildType.ARTIFACT_OR_PROMISE,
+                        "name_matcher": {
+                            "kind": CollectionItemMatcherKind.STARTSWITH,
+                            "value": "signed-source-amd64-hello|build-",
+                        },
+                    }
+                ],
+                "codename": "trixie",
+                "delayed_days": None,
+                "key": None,
+                "merge_uploads": False,
+                "require_signature": True,
+                "since_version": "1.1",
+                "source_artifact": (
+                    "internal@collections/name:signed-source-amd64-hello"
+                ),
+                "target": "ftp://user@example.org/pub/UploadQueue1/",
+                "target_distribution": "debian:testing",
+                "vendor": "debian",
+            },
+        )
+
+        self.assertEqual(
+            package_upload_signed_amd64.workflow_data_json,
+            {"display_name": "package upload", "step": "package_upload"},
+        )
+
+        # The workflow can't orchestrate uploads of the signed source
+        # package and its binaries until the signed source package has been
+        # assembled.
+        self.assertEqual(
+            package_upload_signed_amd64.status, WorkRequest.Statuses.BLOCKED
+        )
+        self.assertQuerySetEqual(
+            package_upload_signed_amd64.dependencies.all(),
+            [assemble_signed_source],
+        )
+        self.assertQuerySetEqual(package_upload_signed_amd64.children.all(), [])
+
+        # Assembling the signed source package causes the remaining uploads
+        # to be orchestrated.
+        self.assertTrue(
+            initial_sbuild_amd64.mark_completed(WorkRequest.Results.SUCCESS)
+        )
+        self.playground.create_signing_input_artifact(
+            binary_package_name="hello-signed",
+            workspace=extract_for_signing.workspace,
+            work_request=extract_for_signing,
+        )
+        extract_for_signing.refresh_from_db()
+        self.assertTrue(
+            extract_for_signing.mark_completed(WorkRequest.Results.SUCCESS)
+        )
+        self.playground.create_signing_output_artifact(
+            purpose=KeyPurpose.OPENPGP,
+            fingerprint="ABC123",
+            binary_package_name="hello-signed",
+            workspace=sign.workspace,
+            work_request=sign,
+        )
+        sign.refresh_from_db()
+        self.assertTrue(sign.mark_completed(WorkRequest.Results.SUCCESS))
+        self.playground.create_upload_artifacts(
+            src_name="hello-signed-template",
+            source=True,
+            binary=False,
+            workspace=assemble_signed_source.workspace,
+            work_request=assemble_signed_source,
+        )
+        assemble_signed_source.refresh_from_db()
+        self.assertTrue(
+            assemble_signed_source.mark_completed(WorkRequest.Results.SUCCESS)
+        )
+        package_upload_signed_amd64.refresh_from_db()
+        self.assertEqual(
+            package_upload_signed_amd64.status, WorkRequest.Statuses.PENDING
+        )
+        with self.captureOnCommitCallbacks() as on_commit:
+            self.assertIn(workflow, schedule())
+        with override_settings(CELERY_TASK_ALWAYS_EAGER=True):
+            for callback in on_commit:
+                callback()
+
+        package_upload_signed_amd64.refresh_from_db()
+        self.assertEqual(
+            package_upload_signed_amd64.status, WorkRequest.Statuses.RUNNING
+        )
+        self.assertQuerySetEqual(
+            package_upload_signed_amd64.children.order_by("id").values_list(
+                "task_type",
+                "task_name",
+                "status",
+                "dependencies__task_type",
+                "dependencies__task_name",
+            ),
+            [
+                # Work requests to sign and upload the source package.
+                (
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                    WorkRequest.Statuses.PENDING,
+                    None,
+                    None,
+                ),
+                (
+                    TaskTypes.SERVER,
+                    "packageupload",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                ),
+                # Work requests to sign and upload the binary package.
+                (
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WORKER,
+                    "sbuild",
+                ),
+                (
+                    TaskTypes.SERVER,
+                    "packageupload",
+                    WorkRequest.Statuses.BLOCKED,
+                    TaskTypes.WAIT,
+                    "externaldebsign",
+                ),
+            ],
+        )
+
     def test_populate_upload_package_no_include_source_binaries(self) -> None:
         """Populate create upload_workflow without source neither binaries."""
         workflow = self.orchestrate(
             extra_data={
                 "architectures": ["amd64"],
                 "source_artifact": self.source_artifact.id,
-                "enable_make_signed_source": True,
-                "make_signed_source_purpose": KeyPurpose.OPENPGP,
-                "signing_template_names": {"amd64": ["hello"]},
-                "make_signed_source_key": "ABC123",
                 "enable_upload": True,
                 "vendor": "debian",
                 "codename": "trixie",
@@ -877,5 +1218,5 @@ class DebianPipelineWorkflowTests(TestCase):
 
     def test_get_label(self) -> None:
         """Test get_label."""
-        w = self.create_debian_pipeline_workflow(extra_task_data={})
+        w = self.create_debian_pipeline_workflow()
         self.assertEqual(w.get_label(), "run Debian pipeline")
