@@ -21,7 +21,7 @@ from debusine.tasks.models import (
     SbuildInput,
     empty_lookup_multiple,
 )
-from debusine.test.base import TestCase
+from debusine.test.django import TestCase
 
 
 class SbuildWorkflowDataTests(TestCase):
