@@ -110,7 +110,7 @@ class PiupartsWorkflowTests(TestCase):
         self.assertEqual(
             piuparts_wr.task_data,
             {
-                "backend": "auto",
+                "backend": "unshare",
                 "input": {
                     "binary_artifacts": [
                         f"{binaries_all.id}@artifacts",
@@ -140,7 +140,7 @@ class PiupartsWorkflowTests(TestCase):
         self.assertEqual(
             piuparts_wr.task_data,
             {
-                "backend": "auto",
+                "backend": "unshare",
                 "input": {
                     "binary_artifacts": [
                         f"{binaries_all.id}@artifacts",
@@ -176,6 +176,8 @@ class PiupartsWorkflowTests(TestCase):
                 "vendor": "debian",
                 "codename": "bookworm",
                 "architectures": ["all"],
+                # Override the environment
+                "environment": "debian/match:codename=trixie",
             },
         )
         PiupartsWorkflow(o).populate()
@@ -196,7 +198,7 @@ class PiupartsWorkflowTests(TestCase):
         self.assertEqual(
             piuparts_wr.task_data,
             {
-                "backend": "auto",
+                "backend": "unshare",
                 "input": {
                     "binary_artifacts": [
                         f"{binaries_all.id}@artifacts",
@@ -204,11 +206,32 @@ class PiupartsWorkflowTests(TestCase):
                 },
                 "host_architecture": "amd64",
                 "base_tgz": "debian/match:codename=bookworm",
-                "environment": "debian/match:codename=bookworm",
+                "environment": "debian/match:codename=trixie",
                 "extra_repositories": None,
             },
         )
 
+    def test_populate_experimental(self) -> None:
+        """The workflow handles overlay distributions."""
+        self.playground.create_debian_environment(codename="experimental")
+        binary_artifact = (
+            self.playground.create_minimal_binary_package_artifact()
+        )
+        workflow = self.create_piuparts_workflow(
+            extra_task_data={
+                "binary_artifacts": [binary_artifact.id],
+                "vendor": "debian",
+                "codename": "experimental",
+            }
+        )
+        workflow.populate()
+        children = list(workflow.work_request.children.all())
+        self.assertEqual(len(children), 1)
+        child = children[0]
+        self.assertEqual(len(child.task_data["extra_repositories"]), 1)
+        repo = child.task_data["extra_repositories"][0]
+        self.assertEqual(repo["suite"], "experimental")
+
     def test_compute_dynamic_data_binary_package(self) -> None:
         binary_artifact = (
             self.playground.create_minimal_binary_package_artifact(
