@@ -10,7 +10,7 @@
 """Unit tests for the reverse_dependencies_autopkgtest workflow."""
 
 from collections.abc import Iterable, Mapping
-from typing import Any
+from typing import Any, ClassVar
 
 from debusine.artifacts.local_artifact import BinaryPackages, Upload
 from debusine.artifacts.models import (
@@ -25,7 +25,10 @@ from debusine.db.context import context
 from debusine.db.models import Artifact, Collection, TaskDatabase, WorkRequest
 from debusine.server.collections.debian_suite import DebianSuiteManager
 from debusine.server.collections.lookup import lookup_single
-from debusine.server.workflows import ReverseDependenciesAutopkgtestWorkflow
+from debusine.server.workflows import (
+    ReverseDependenciesAutopkgtestWorkflow,
+    WorkflowValidationError,
+)
 from debusine.server.workflows.base import orchestrate_workflow
 from debusine.server.workflows.models import (
     BaseWorkflowData,
@@ -57,10 +60,21 @@ from debusine.test.test_utils import preserve_task_registry
 class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
     """Unit tests for :py:class:`ReverseDependenciesAutopkgtestWorkflow`."""
 
+    sid: ClassVar[Collection]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
+        """Set up common data."""
+        super().setUpTestData()
+        cls.sid = cls.playground.create_collection(
+            name="sid", category=CollectionCategory.SUITE
+        )
+
     def create_rdep_autopkgtest_workflow(
         self,
         extra_task_data: dict[str, Any] | None = None,
         parent: WorkRequest | None = None,
+        validate: bool = True,
     ) -> ReverseDependenciesAutopkgtestWorkflow:
         """Create a reverse_dependencies_autopkgtest workflow."""
         task_data = {
@@ -76,6 +90,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
             task_name="reverse_dependencies_autopkgtest",
             task_data=task_data,
             parent=parent,
+            validate=validate,
         )
         return ReverseDependenciesAutopkgtestWorkflow(wr)
 
@@ -157,6 +172,9 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
 
     def test_create_orchestrator(self) -> None:
         """A ReverseDependenciesAutopkgtestWorkflow can be instantiated."""
+        self.playground.create_collection(
+            name="trixie", category=CollectionCategory.SUITE
+        )
         source_artifact = 2
         binary_artifacts = ["internal@collections/name:build-arm64"]
         suite_collection = "trixie@debian:suite"
@@ -190,6 +208,25 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
 
         self.assertEqual(workflow.data.backend, BackendType.INCUS_LXC)
 
+    def test_validate_input(self) -> None:
+        """validate_input passes a valid case."""
+        workflow = self.create_rdep_autopkgtest_workflow()
+
+        workflow.validate_input()
+
+    def test_validate_input_bad_suite_collection(self) -> None:
+        """validate_input raises errors in looking up suite_collection."""
+        workflow = self.create_rdep_autopkgtest_workflow(
+            extra_task_data={"suite_collection": "nonexistent@debian:suite"},
+            validate=False,
+        )
+
+        with self.assertRaisesRegex(
+            WorkflowValidationError,
+            "'nonexistent@debian:suite' does not exist or is hidden",
+        ):
+            workflow.validate_input()
+
     def test_get_binary_names_promise(self) -> None:
         """Get binary names from a promise."""
         workflow = self.create_rdep_autopkgtest_workflow()
@@ -355,22 +392,19 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
         trixie = self.playground.create_collection(
             name="trixie", category=CollectionCategory.SUITE
         )
-        sid = self.playground.create_collection(
-            name="sid", category=CollectionCategory.SUITE
-        )
 
         pre_depends = self.create_source_package(
             name="pre-depends",
             version="1.0",
             dsc_fields={"Testsuite": "autopkgtest"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
             source_package=pre_depends,
             name="python3-pre-depends",
             version="1:1.0",
             deb_fields={"Pre-Depends": "python3-urllib3:any"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
 
         depends = self.create_source_package(
@@ -379,35 +413,35 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
             dsc_fields={
                 "Testsuite": "autopkgtest-pkg-python, autopkgtest-pkg-pybuild"
             },
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
             source_package=depends,
             name="python3-depends",
             deb_fields={"Depends": "python3-urllib3 (>= 1.21.1), python3:any"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
             source_package=depends,
             name="depends-doc",
             deb_fields={"Depends": "libjs-sphinxdoc"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
 
         similar_depends = self.create_source_package(
             name="similar-depends",
             version="1",
             dsc_fields={"Testsuite": "autopkgtest-pkg-pybuild"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
             source_package=similar_depends,
             name="python3-similar-depends",
             deb_fields={"Depends": "python3-urllib3-plugin"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
 
-        for version, add_to_collection in (("1", trixie), ("2", sid)):
+        for version, add_to_collection in (("1", trixie), ("2", self.sid)):
             trigger = self.create_source_package(
                 name="trigger",
                 version=version,
@@ -428,7 +462,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                 "Testsuite": "autopkgtest",
                 "Testsuite-Triggers": "python3-urllib3",
             },
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
             source_package=previous_version, name="python3-urllib3"
@@ -438,19 +472,19 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
             name="unrelated",
             version="1",
             dsc_fields={"Testsuite": "autopkgtest"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
             source_package=unrelated, add_to_collection=add_to_collection
         )
 
         untested = self.create_source_package(
-            name="untested", version="1", add_to_collection=sid
+            name="untested", version="1", add_to_collection=self.sid
         )
         self.create_binary_package(
             source_package=untested,
             deb_fields={"Depends": "python3-urllib3"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
 
         unsuitable_testsuite = self.create_source_package(
@@ -460,10 +494,10 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                 "Testsuite": "autopkgtest-unsuitable",
                 "Testsuite-Triggers": "python3-urllib3",
             },
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
-            source_package=unsuitable_testsuite, add_to_collection=sid
+            source_package=unsuitable_testsuite, add_to_collection=self.sid
         )
 
         source_package = self.create_source_package(
@@ -476,7 +510,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
             extra_task_data={
                 "source_artifact": source_package.id,
                 "binary_artifacts": [binary_package.id],
-                "suite_collection": sid.id,
+                "suite_collection": self.sid.id,
             }
         )
 
@@ -530,7 +564,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
             extra_task_data={
                 "source_artifact": upload_artifacts.upload.id,
                 "binary_artifacts": [upload_artifacts.binaries[0].id],
-                "suite_collection": sid.id,
+                "suite_collection": self.sid.id,
             }
         )
         self.assert_reverse_dependencies(
@@ -551,19 +585,16 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
 
     def test_populate(self) -> None:
         """The workflow populates child work requests."""
-        sid = self.playground.create_collection(
-            name="sid", category=CollectionCategory.SUITE
-        )
         depends = self.create_source_package(
             name="depends",
             version="1.0",
             dsc_fields={"Testsuite": "autopkgtest"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
             source_package=depends,
             deb_fields={"Depends": "hello"},
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         trigger = self.create_source_package(
             name="trigger",
@@ -572,10 +603,10 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                 "Testsuite": "autopkgtest",
                 "Testsuite-Triggers": "hello",
             },
-            add_to_collection=sid,
+            add_to_collection=self.sid,
         )
         self.create_binary_package(
-            source_package=trigger, add_to_collection=sid
+            source_package=trigger, add_to_collection=self.sid
         )
         source_artifact = self.create_source_package(
             name="hello", version="1.0", dsc_fields={"Binary": "hello"}
@@ -591,6 +622,9 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
 
                 def populate(self) -> None:
                     """Populate the pipeline."""
+                    sid = Collection.objects.get(
+                        name="sid", category=CollectionCategory.SUITE
+                    )
                     sbuild = self.work_request.create_child(
                         task_type=TaskTypes.WORKFLOW,
                         task_name="sbuild",
@@ -676,12 +710,13 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                 self.assertEqual(
                     child.task_data,
                     {
-                        "prefix": f"{source}/",
+                        "prefix": f"{source}|",
                         "source_artifact": (
-                            f"{sid.id}@collections/source-version:{source}"
+                            f"{self.sid.id}@collections/source-version:{source}"
                         ),
                         "binary_artifacts": [
-                            f"{sid.id}@collections/binary-version:{source}_all"
+                            f"{self.sid.id}@collections/"
+                            f"binary-version:{source}_all"
                         ],
                         "context_artifacts": [
                             f"internal@collections/name:build-{architecture}"
@@ -714,10 +749,11 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                     {
                         "input": {
                             "source_artifact": (
-                                f"{sid.id}@collections/source-version:{source}"
+                                f"{self.sid.id}@collections/"
+                                f"source-version:{source}"
                             ),
                             "binary_artifacts": [
-                                f"{sid.id}@collections/name:{source}_all"
+                                f"{self.sid.id}@collections/name:{source}_all"
                             ],
                             "context_artifacts": [
                                 f"internal@collections/"
@@ -753,7 +789,7 @@ class ReverseDependenciesAutopkgtestWorkflowTests(TestCase):
                             {
                                 "action": "update-collection-with-artifacts",
                                 "collection": "internal@collections",
-                                "name_template": f"{source}/autopkgtest-amd64",
+                                "name_template": f"{source}|autopkgtest-amd64",
                                 "variables": None,
                                 "artifact_filters": {
                                     "category": ArtifactCategory.AUTOPKGTEST
