@@ -727,7 +727,10 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
                 },
                 "collection": "internal@collections",
                 "name_template": "signing-template-amd64-hello-unsigned",
-                "variables": {"architecture": "amd64"},
+                "variables": {
+                    "architecture": "amd64",
+                    "binary_package_name": "hello-unsigned",
+                },
             },
             child.event_reactions_json["on_success"],
         )
@@ -851,6 +854,28 @@ class SbuildWorkflowTests(CollectionTestMixin, TestCase):
             f"{upload_artifacts.source.id}@artifacts",
         )
 
+    def test_populate_experimental(self) -> None:
+        """The workflow handles overlay distributions."""
+        source_artifact = self.playground.create_source_artifact()
+        self.playground.create_debian_environment(codename="experimental")
+        wr = self.playground.create_workflow(
+            task_name="sbuild",
+            task_data=SbuildWorkflowData(
+                input=SbuildInput(
+                    source_artifact=source_artifact.id,
+                ),
+                target_distribution="debian:experimental",
+                architectures=["amd64"],
+            ),
+        )
+        SbuildWorkflow(wr).populate()
+        children = list(wr.children.all())
+        self.assertEqual(len(children), 1)
+        child = children[0]
+        self.assertEqual(len(child.task_data["extra_repositories"]), 1)
+        repo = child.task_data["extra_repositories"][0]
+        self.assertEqual(repo["suite"], "experimental")
+
     def test_label(self) -> None:
         """Test get_label."""
         wr = WorkRequest(
