@@ -120,7 +120,7 @@ class WorkflowTests(TestCase):
 
         self.assert_work_request_event_reactions(child, on_creation=[])
 
-        name = "testing"
+        name = "hello_1:1.0-1|testing"
         data = {"architecture": "amd64"}
         category = ArtifactCategory.TEST
         artifact_filters = {"xxx": "yyy"}
@@ -182,8 +182,34 @@ class WorkflowTests(TestCase):
         self.assert_work_request_event_reactions(child, on_success=on_success)
 
     @preserve_task_registry()
-    def test_provides_artifact_raise_value_error(self) -> None:
-        r"""Test requires_artifact() LookupError: key starts with promise\_."""
+    def test_provides_artifact_bad_name(self) -> None:
+        r"""Test requires_artifact() ValueError: name contains a slash."""
+
+        class ExampleWorkflow(
+            TestWorkflow[BaseWorkflowData, BaseDynamicTaskData]
+        ):
+            """Concrete workflow instance to use for tests."""
+
+            def populate(self) -> None:
+                """Unused abstract method from Workflow."""
+                raise NotImplementedError()
+
+        workflow_root = self.playground.create_workflow()
+
+        child = workflow_root.create_child("noop")
+
+        workflow = ExampleWorkflow(child)
+
+        with self.assertRaisesRegex(
+            ValueError, r'Collection item name may not contain "/"\.'
+        ):
+            workflow.provides_artifact(
+                child, ArtifactCategory.TEST, "prefix/testing"
+            )
+
+    @preserve_task_registry()
+    def test_provides_artifact_bad_data_key(self) -> None:
+        r"""Test requires_artifact() ValueError: key starts with promise\_."""
 
         class ExampleWorkflow(
             TestWorkflow[BaseWorkflowData, BaseDynamicTaskData]
