@@ -9,6 +9,7 @@
 
 """Unit tests for the signing database models."""
 
+import hashlib
 import io
 import os
 import random
@@ -322,7 +323,8 @@ class KeyTests(TestCase, DjangoTestCase):
             storage_private_key=storage_private_key, purpose=Key.Purpose.UEFI
         )
         temp_path = self.create_temporary_directory()
-        (data_path := temp_path / "data").write_bytes(b"data to sign")
+        raw_data = b"data to sign"
+        (data_path := temp_path / "data").write_bytes(raw_data)
         signature_path = temp_path / "signed"
         signature_bytes = random.randbytes(64)
         log_file = io.BytesIO()
@@ -345,7 +347,14 @@ class KeyTests(TestCase, DjangoTestCase):
             self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
         ):
             key.sign(
-                data_path, signature_path, SigningMode.ATTACHED, 123, log_file
+                data_path,
+                signature_path,
+                SigningMode.ATTACHED,
+                123,
+                username="testuser",
+                user_id=1234,
+                resource={"package": "linux"},
+                log_file=log_file,
             )
 
         mock_run.assert_called_once_with(
@@ -367,6 +376,23 @@ class KeyTests(TestCase, DjangoTestCase):
         self.assertEqual(certificate_bytes, key.public_key)
         self.assertEqual(signature_path.read_bytes(), signature_bytes)
 
+        audit_log = AuditLog.objects.latest("created_at")
+        self.assertEqual(audit_log.purpose, Key.Purpose.UEFI)
+        self.assertEqual(audit_log.fingerprint, key.fingerprint)
+        self.assertEqual(audit_log.event, AuditLog.Event.SIGN)
+        self.assertEqual(
+            audit_log.data,
+            {
+                "data_sha256": hashlib.sha256(raw_data).hexdigest(),
+                "username": "testuser",
+                "user_id": 1234,
+                "resource": {
+                    "package": "linux",
+                },
+            },
+        )
+        self.assertEqual(audit_log.created_by_work_request_id, 123)
+
     def test_sign_uefi_detached(self) -> None:
         """Sign data using a UEFI key, returning a detached signature."""
         # Signing data is slow, so we just test that the correct
@@ -399,7 +425,11 @@ class KeyTests(TestCase, DjangoTestCase):
             self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
         ):
             key.sign(
-                data_path, signature_path, SigningMode.DETACHED, 123, log_file
+                data_path,
+                signature_path,
+                SigningMode.DETACHED,
+                123,
+                log_file=log_file,
             )
 
         mock_run.assert_called_once_with(
@@ -442,7 +472,6 @@ class KeyTests(TestCase, DjangoTestCase):
         (data_path := temp_path / "data").write_bytes(data_bytes)
         signature_path = temp_path / "data.asc"
         signature_bytes = random.randbytes(64)
-        log_file = io.BytesIO()
 
         def sign(
             ctx: gpg.Context,
@@ -471,9 +500,7 @@ class KeyTests(TestCase, DjangoTestCase):
                 "gpg.Context.sign", autospec=True, side_effect=sign
             ) as mock_sign,
         ):
-            key.sign(
-                data_path, signature_path, SigningMode.CLEAR, 123, log_file
-            )
+            key.sign(data_path, signature_path, SigningMode.CLEAR, 123)
 
         mock_key_import.assert_called_once_with(secret_key)
         mock_keylist.assert_called_once_with(
@@ -498,15 +525,12 @@ class KeyTests(TestCase, DjangoTestCase):
         temp_path = self.create_temporary_directory()
         (data_path := temp_path / "data").write_bytes(b"data to sign")
         signature_path = temp_path / "data.asc"
-        log_file = io.BytesIO()
 
         with self.assertRaisesRegex(
             GnuPGError,
             "OpenPGP signing currently only supports software-encrypted keys",
         ):
-            key.sign(
-                data_path, signature_path, SigningMode.CLEAR, 123, log_file
-            )
+            key.sign(data_path, signature_path, SigningMode.CLEAR, 123)
 
     def test_sign_openpgp_key_import_failed(self) -> None:
         """Key signing error: failed to import OpenPGP secret key."""
@@ -524,16 +548,13 @@ class KeyTests(TestCase, DjangoTestCase):
         temp_path = self.create_temporary_directory()
         (data_path := temp_path / "data").write_bytes(b"data to sign")
         signature_path = temp_path / "data.asc"
-        log_file = io.BytesIO()
 
         with (
             self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
             mock.patch("gpg.Context.key_import", return_value=import_result),
             self.assertRaisesRegex(GnuPGError, "Failed to import secret key"),
         ):
-            key.sign(
-                data_path, signature_path, SigningMode.CLEAR, 123, log_file
-            )
+            key.sign(data_path, signature_path, SigningMode.CLEAR, 123)
 
     def test_sign_openpgp_debsign(self) -> None:
         """Sign an upload using an OpenPGP key and debsign."""
@@ -583,7 +604,7 @@ class KeyTests(TestCase, DjangoTestCase):
                 signed_changes_path,
                 SigningMode.DEBSIGN,
                 123,
-                log_file,
+                log_file=log_file,
             )
 
         mock_key_import.assert_called_once_with(secret_key)
@@ -621,7 +642,6 @@ class KeyTests(TestCase, DjangoTestCase):
         )
         signed_path = self.create_temporary_directory()
         signed_changes_path = signed_path / "foo.changes"
-        log_file = io.BytesIO()
 
         with self.assertRaisesRegex(
             GnuPGError,
@@ -632,7 +652,6 @@ class KeyTests(TestCase, DjangoTestCase):
                 signed_changes_path,
                 SigningMode.DEBSIGN,
                 123,
-                log_file,
             )
 
     def test_sign_openpgp_debsign_key_import_failed(self) -> None:
@@ -654,7 +673,6 @@ class KeyTests(TestCase, DjangoTestCase):
         )
         signed_path = self.create_temporary_directory()
         signed_changes_path = signed_path / "foo.changes"
-        log_file = io.BytesIO()
 
         with (
             self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
@@ -666,7 +684,6 @@ class KeyTests(TestCase, DjangoTestCase):
                 signed_changes_path,
                 SigningMode.DEBSIGN,
                 123,
-                log_file,
             )
 
     def test_sign_openpgp_debsign_signature_directory_not_empty(self) -> None:
@@ -690,7 +707,6 @@ class KeyTests(TestCase, DjangoTestCase):
         signed_path = self.create_temporary_directory()
         signed_changes_path = signed_path / "foo.changes"
         (signed_path / "foo.dsc").touch()
-        log_file = io.BytesIO()
 
         with (
             self.settings(DEBUSINE_SIGNING_PRIVATE_KEYS=[storage_private_key]),
@@ -704,5 +720,4 @@ class KeyTests(TestCase, DjangoTestCase):
                 signed_changes_path,
                 SigningMode.DEBSIGN,
                 123,
-                log_file,
             )
