@@ -13,7 +13,7 @@ from django.db import connection
 from django.db.migrations.executor import MigrationExecutor
 from django.test import TestCase
 
-from debusine.signing.db.models import _KeyPurpose
+from debusine.assets import KeyPurpose
 from debusine.signing.models import ProtectedKeyStorage
 
 
@@ -68,7 +68,7 @@ class MigrationTests(TestCase):
         old_apps = executor.loader.project_state(migrate_from).apps
         for old_fingerprint, old_private_key, old_public_key in old_keys:
             old_apps.get_model("signing", "Key").objects.create(
-                purpose=_KeyPurpose.UEFI,
+                purpose=KeyPurpose.UEFI,
                 fingerprint=old_fingerprint,
                 private_key=old_private_key,
                 public_key=old_public_key,
@@ -80,7 +80,7 @@ class MigrationTests(TestCase):
 
         for new_fingerprint, new_private_key, new_public_key in new_keys:
             key = new_apps.get_model("signing", "Key").objects.get(
-                purpose=_KeyPurpose.UEFI, fingerprint=new_fingerprint
+                purpose=KeyPurpose.UEFI, fingerprint=new_fingerprint
             )
             self.assertEqual(key.private_key, new_private_key)
             self.assertEqual(bytes(key.public_key), new_public_key)
@@ -90,7 +90,7 @@ class MigrationTests(TestCase):
 
         for old_fingerprint, old_private_key, old_public_key in old_keys:
             key = old_apps.get_model("signing", "Key").objects.get(
-                purpose=_KeyPurpose.UEFI, fingerprint=old_fingerprint
+                purpose=KeyPurpose.UEFI, fingerprint=old_fingerprint
             )
             self.assertEqual(key.private_key, old_private_key)
             self.assertEqual(bytes(key.public_key), old_public_key)
