@@ -17,6 +17,7 @@ from debusine.signing.db.models import Key
 from debusine.signing.tasks import BaseSigningTask
 from debusine.signing.tasks.models import GenerateKeyData
 from debusine.tasks.models import BaseDynamicTaskData
+from debusine.tasks.server import TaskDatabaseInterface
 
 
 class GenerateKey(BaseSigningTask[GenerateKeyData, BaseDynamicTaskData]):
@@ -65,3 +66,10 @@ class GenerateKey(BaseSigningTask[GenerateKeyData, BaseDynamicTaskData]):
     def get_label(self) -> str:
         """Return the task label."""
         return f"generate {self.data.purpose} key"
+
+    def build_dynamic_data(
+        self,
+        task_database: TaskDatabaseInterface,  # noqa: U100
+    ) -> BaseDynamicTaskData:
+        """Resolve artifact lookups for this task."""
+        return BaseDynamicTaskData()
