@@ -9,7 +9,7 @@
 
 """Models used by debusine signing tasks."""
 
-from debusine.artifacts.models import KeyPurpose
+from debusine.assets import KeyPurpose
 from debusine.tasks.models import (
     BaseDynamicTaskData,
     BaseTaskData,
@@ -36,25 +36,24 @@ class SignData(BaseTaskData):
 
     purpose: KeyPurpose
     unsigned: LookupMultiple
-    key: LookupSingle
+    key: str
 
 
 class SignDynamicData(BaseDynamicTaskData):
     """Dynamic data for the Sign task."""
 
     unsigned_ids: list[int]
-    key_id: int
+    unsigned_binary_package_names: list[str | None] = []
 
 
 class DebsignData(BaseTaskData):
     """In-memory task data for the Debsign task."""
 
     unsigned: LookupSingle
-    key: LookupSingle
+    key: str
 
 
 class DebsignDynamicData(BaseDynamicTaskData):
     """Dynamic data for the Debsign task."""
 
     unsigned_id: int
-    key_id: int
