@@ -19,8 +19,9 @@ from cryptography.x509 import load_pem_x509_certificate
 from django.test import TransactionTestCase, override_settings
 from nacl.public import PrivateKey, SealedBox
 
-from debusine.artifacts import SigningKeyArtifact, WorkRequestDebugLogs
-from debusine.artifacts.models import EmptyArtifactData, KeyPurpose
+from debusine.artifacts import WorkRequestDebugLogs
+from debusine.artifacts.models import EmptyArtifactData
+from debusine.assets import AssetCategory, KeyPurpose, SigningKeyData
 from debusine.client.debusine import Debusine
 from debusine.signing.db.models import Key
 from debusine.signing.gnupg import gpg_ephemeral_context
@@ -74,31 +75,17 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
             base64.b64decode(key.private_key["encrypted"].encode())
         )
 
-        # The task uploaded an artifact with the expected properties.
-        debusine_mock.upload_artifact.assert_has_calls(
-            [
-                mock.call(
-                    SigningKeyArtifact.create(
-                        KeyPurpose.UEFI, key.fingerprint, key.public_key
-                    ),
-                    workspace="System",
-                    work_request=1,
-                ),
-                mock.call(
-                    WorkRequestDebugLogs(
-                        category=WorkRequestDebugLogs._category,
-                        data=EmptyArtifactData(),
-                        files={
-                            "cmd-output.log": (
-                                Path(debug_log_files_directory.name)
-                                / "cmd-output.log"
-                            )
-                        },
-                    ),
-                    workspace="System",
-                    work_request=1,
-                ),
-            ]
+        # The task created an asset with the expected properties.
+        debusine_mock.asset_create.assert_called_once_with(
+            category=AssetCategory.SIGNING_KEY,
+            data=SigningKeyData(
+                purpose=KeyPurpose.UEFI,
+                fingerprint=key.fingerprint,
+                public_key=base64.b64encode(key.public_key).decode(),
+                description="A UEFI key",
+            ),
+            workspace="System",
+            work_request=1,
         )
 
     def test_openpgp(self) -> None:
@@ -140,31 +127,17 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
             base64.b64decode(key.private_key["encrypted"].encode())
         )
 
-        # The task uploaded an artifact with the expected properties.
-        debusine_mock.upload_artifact.assert_has_calls(
-            [
-                mock.call(
-                    SigningKeyArtifact.create(
-                        KeyPurpose.OPENPGP, key.fingerprint, key.public_key
-                    ),
-                    workspace="System",
-                    work_request=1,
-                ),
-                mock.call(
-                    WorkRequestDebugLogs(
-                        category=WorkRequestDebugLogs._category,
-                        data=EmptyArtifactData(),
-                        files={
-                            "cmd-output.log": (
-                                Path(debug_log_files_directory.name)
-                                / "cmd-output.log"
-                            )
-                        },
-                    ),
-                    workspace="System",
-                    work_request=1,
-                ),
-            ]
+        # The task created a signing asset with the expected properties.
+        debusine_mock.asset_create.assert_called_once_with(
+            category=AssetCategory.SIGNING_KEY,
+            data=SigningKeyData(
+                purpose=KeyPurpose.OPENPGP,
+                fingerprint=key.fingerprint,
+                public_key=base64.b64encode(key.public_key).decode(),
+                description="An OpenPGP key",
+            ),
+            workspace="System",
+            work_request=1,
         )
 
     def test_generate_failure(self) -> None:
@@ -190,12 +163,18 @@ class GenerateKeyTests(TestCase, TransactionTestCase):
         # No key was generated.
         self.assertQuerySetEqual(Key.objects.all(), [])
 
-        # Only a debug logs artifact was uploaded, not a signing key artifact.
+        # No asset was created
+        debusine_mock.asset_create.assert_not_called()
+
+        # Debug logs artifact was uploaded
         debusine_mock.upload_artifact.assert_called_once_with(
             WorkRequestDebugLogs(
                 category=WorkRequestDebugLogs._category,
                 data=EmptyArtifactData(),
                 files={
+                    "stages.log": (
+                        Path(debug_log_files_directory.name) / "stages.log"
+                    ),
                     "cmd-output.log": (
                         Path(debug_log_files_directory.name) / "cmd-output.log"
                     ),
