@@ -31,20 +31,21 @@ except ImportError:
 from debusine.artifacts import (
     LocalArtifact,
     SigningInputArtifact,
-    SigningKeyArtifact,
     SigningOutputArtifact,
     WorkRequestDebugLogs,
 )
 from debusine.artifacts.models import (
     ArtifactCategory,
+    DebusineSigningInput,
     DebusineSigningOutput,
     EmptyArtifactData,
-    KeyPurpose,
     SigningResult,
 )
+from debusine.assets import AssetCategory, KeyPurpose
 from debusine.client.debusine import Debusine
 from debusine.client.models import (
     ArtifactResponse,
+    AssetPermissionCheckResponse,
     FileResponse,
     FilesResponseType,
     RemoteArtifact,
@@ -56,6 +57,7 @@ from debusine.signing.models import SigningMode
 from debusine.signing.tasks import BaseSigningTask, Sign
 from debusine.signing.tasks.models import SignDynamicData
 from debusine.tasks.models import LookupMultiple, WorkerType
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import FakeTaskDatabase
 from debusine.test import TestCase
 from debusine.utils import calculate_hash, is_command_available
@@ -236,15 +238,16 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
 
     def test_compute_dynamic_data(self) -> None:
         """Dynamic data receives relevant artifact IDs."""
-        key_lookup = "bookworm@debian:suite-signing-keys/key:uefi"
         task_db = FakeTaskDatabase(
-            single_lookups={
-                # key
-                (key_lookup, None): 1,
-            },
             multiple_lookups={
                 # unsigned
-                (LookupMultiple.parse_obj([2]), None): [2]
+                (LookupMultiple.parse_obj([2]), None): [
+                    ArtifactInfo(
+                        id=2,
+                        category=ArtifactCategory.SIGNING_INPUT,
+                        data=DebusineSigningInput(),
+                    )
+                ]
             },
         )
 
@@ -252,14 +255,59 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             task_data={
                 "purpose": KeyPurpose.UEFI,
                 "unsigned": [2],
-                "key": key_lookup,
+                "key": "ABC123",
             }
         )
         self.assertEqual(
             task.compute_dynamic_data(task_db),
-            SignDynamicData(unsigned_ids=[2], key_id=1),
+            SignDynamicData(
+                unsigned_ids=[2], unsigned_binary_package_names=[None]
+            ),
         )
 
+    def test_compute_dynamic_data_copies_binary_package_name(self) -> None:
+        """Dynamic data copies `binary_package_name` from `unsigned`."""
+        task_db = FakeTaskDatabase(
+            multiple_lookups={
+                # unsigned
+                (LookupMultiple.parse_obj([2]), None): [
+                    ArtifactInfo(
+                        id=2,
+                        category=ArtifactCategory.SIGNING_INPUT,
+                        data=DebusineSigningInput(binary_package_name="hello"),
+                    )
+                ]
+            },
+        )
+
+        task = Sign(
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
+        )
+        self.assertEqual(
+            task.compute_dynamic_data(task_db),
+            SignDynamicData(
+                unsigned_ids=[2], unsigned_binary_package_names=["hello"]
+            ),
+        )
+
+    def test_get_source_artifacts_ids(self) -> None:
+        """Test get_source_artifacts_ids."""
+        task = Sign(
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": "ABC123",
+            }
+        )
+        self.assertEqual(task.get_source_artifacts_ids(), [])
+
+        task.dynamic_data = SignDynamicData(unsigned_ids=[2])
+        self.assertEqual(task.get_source_artifacts_ids(), [2])
+
     def test_fetch_input_missing_trusted_cert(self) -> None:
         """fetch_input requires trusted_certs to match configuration."""
         trusted_cert = "0" * 64
@@ -274,8 +322,10 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
 
         task = Sign(
             task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1},
-            dynamic_task_data={"unsigned_ids": [2], "key_id": 1},
+            dynamic_task_data={"unsigned_ids": [2]},
         )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
         debusine_mock = self.mock_debusine(task)
         debusine_mock.download_artifact.side_effect = download_artifact
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
@@ -302,85 +352,72 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             private_key={},
             public_key=b"",
         )
-        signing_key = SigningKeyArtifact.create(
-            KeyPurpose.UEFI, fingerprint=fingerprint, public_key=b""
-        )
-        signing_key_id = 1
         signing_input = SigningInputArtifact.create(
             [], self.create_temporary_directory(), trusted_certs=[trusted_cert]
         )
         signing_input_id = 2
-        responses = {
-            response.id: response
-            for response in (
-                self.make_artifact_response(signing_key, signing_key_id),
-                self.make_artifact_response(signing_input, signing_input_id),
-            )
-        }
-
-        def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
-        ) -> ArtifactResponse:
-            return responses[artifact_id]
+        signing_input_response = self.make_artifact_response(
+            signing_input, signing_input_id
+        )
 
         task = Sign(
             task_data={
                 "purpose": KeyPurpose.UEFI,
                 "unsigned": [signing_input_id],
-                "key": signing_key_id,
+                "key": fingerprint,
             },
             dynamic_task_data={
                 "unsigned_ids": [signing_input_id],
-                "key_id": signing_key_id,
             },
         )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
         debusine_mock = self.mock_debusine(task)
-        debusine_mock.download_artifact.side_effect = download_artifact
+        debusine_mock.download_artifact.return_value = signing_input_response
         destination = self.create_temporary_directory()
 
         with override_settings(DEBUSINE_SIGNING_TRUSTED_CERTS=[trusted_cert]):
             self.assertTrue(task.fetch_input(destination))
 
-        self.assertEqual(task._files, {signing_input_id: responses[2].files})
+        self.assertEqual(
+            task._files, {signing_input_id: signing_input_response.files}
+        )
         self.assertEqual(task._key, key)
+        debusine_mock.download_artifact.assert_called_once_with(
+            signing_input_id, destination, tarball=mock.ANY
+        )
 
     def test_fetch_input_mismatched_purpose(self) -> None:
         """fetch_input requires purpose to match key.purpose."""
         fingerprint = "0" * 64
-        signing_key = SigningKeyArtifact.create(
-            KeyPurpose.UEFI, fingerprint=fingerprint, public_key=b""
+        Key.objects.create(
+            purpose=Key.Purpose.UEFI,
+            fingerprint=fingerprint,
+            private_key={},
         )
-        signing_key_id = 1
+
         signing_input = SigningInputArtifact.create(
             [], self.create_temporary_directory()
         )
         signing_input_id = 2
-        responses = {
-            response.id: response
-            for response in (
-                self.make_artifact_response(signing_key, signing_key_id),
-                self.make_artifact_response(signing_input, signing_input_id),
-            )
-        }
-
-        def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
-        ) -> ArtifactResponse:
-            return responses[artifact_id]
+        signing_input_response = self.make_artifact_response(
+            signing_input, signing_input_id
+        )
 
         task = Sign(
             task_data={
                 "purpose": KeyPurpose.OPENPGP,
                 "unsigned": [signing_input_id],
-                "key": signing_key_id,
+                "key": fingerprint,
             },
             dynamic_task_data={
                 "unsigned_ids": [signing_input_id],
-                "key_id": signing_key_id,
             },
         )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
         debusine_mock = self.mock_debusine(task)
-        debusine_mock.download_artifact.side_effect = download_artifact
+        debusine_mock.download_artifact.return_value = signing_input_response
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
         task._debug_log_files_directory = debug_log_files_directory
@@ -390,46 +427,37 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
 
         self.assertEqual(
             Path(debug_log_files_directory.name, "fetch_input.log").read_text(),
-            "Signing key has purpose uefi; expected openpgp\n",
+            f"Signing key openpgp:{fingerprint} does not exist\n",
+        )
+        debusine_mock.download_artifact.assert_called_once_with(
+            signing_input_id, destination, tarball=mock.ANY
         )
 
     def test_fetch_input_missing_key(self) -> None:
         """fetch_input requires the signing key to exist in the database."""
         fingerprint = "0" * 64
-        signing_key = SigningKeyArtifact.create(
-            KeyPurpose.UEFI, fingerprint=fingerprint, public_key=b""
-        )
-        signing_key_id = 1
         signing_input = SigningInputArtifact.create(
             [], self.create_temporary_directory()
         )
         signing_input_id = 2
-        responses = {
-            response.id: response
-            for response in (
-                self.make_artifact_response(signing_key, signing_key_id),
-                self.make_artifact_response(signing_input, signing_input_id),
-            )
-        }
-
-        def download_artifact(
-            artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
-        ) -> ArtifactResponse:
-            return responses[artifact_id]
+        signing_input_response = self.make_artifact_response(
+            signing_input, signing_input_id
+        )
 
         task = Sign(
             task_data={
                 "purpose": KeyPurpose.UEFI,
                 "unsigned": [signing_input_id],
-                "key": signing_key_id,
+                "key": fingerprint,
             },
             dynamic_task_data={
                 "unsigned_ids": [signing_input_id],
-                "key_id": signing_key_id,
             },
         )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
         debusine_mock = self.mock_debusine(task)
-        debusine_mock.download_artifact.side_effect = download_artifact
+        debusine_mock.download_artifact.return_value = signing_input_response
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
         task._debug_log_files_directory = debug_log_files_directory
@@ -441,6 +469,9 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             Path(debug_log_files_directory.name, "fetch_input.log").read_text(),
             f"Signing key uefi:{fingerprint} does not exist\n",
         )
+        debusine_mock.download_artifact.assert_called_once_with(
+            signing_input_id, destination, tarball=mock.ANY
+        )
 
     def test_fetch_input_multiple_unsigned_ids(self) -> None:
         """fetch_input handles multiple unsigned artifacts."""
@@ -451,10 +482,6 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             private_key={},
             public_key=b"",
         )
-        signing_key = SigningKeyArtifact.create(
-            KeyPurpose.UEFI, fingerprint=fingerprint, public_key=b""
-        )
-        signing_key_id = 1
         signing_inputs = [
             SigningInputArtifact.create([], self.create_temporary_directory())
             for _ in range(2)
@@ -463,7 +490,6 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         responses = {
             artifact_id: self.make_artifact_response(artifact, artifact_id)
             for artifact, artifact_id in (
-                (signing_key, signing_key_id),
                 (signing_inputs[0], signing_input_ids[0]),
                 (signing_inputs[1], signing_input_ids[1]),
             )
@@ -478,13 +504,14 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             task_data={
                 "purpose": KeyPurpose.UEFI,
                 "unsigned": signing_input_ids,
-                "key": signing_key_id,
+                "key": fingerprint,
             },
             dynamic_task_data={
                 "unsigned_ids": signing_input_ids,
-                "key_id": signing_key_id,
             },
         )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
         debusine_mock = self.mock_debusine(task)
         debusine_mock.download_artifact.side_effect = download_artifact
         destination = self.create_temporary_directory()
@@ -501,6 +528,65 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         )
         self.assertEqual(task._key, key)
 
+    def test_fetch_input_permission_denied(self) -> None:
+        fingerprint = "0" * 64
+        Key.objects.create(
+            purpose=Key.Purpose.UEFI,
+            fingerprint=fingerprint,
+            private_key={},
+            public_key=b"",
+        )
+        signing_input = SigningInputArtifact.create(
+            [], self.create_temporary_directory()
+        )
+        signing_input_id = 2
+        signing_input_response = self.make_artifact_response(
+            signing_input, signing_input_id
+        )
+
+        task = Sign(
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [signing_input_id],
+                "key": fingerprint,
+            },
+            dynamic_task_data={
+                "unsigned_ids": [signing_input_id],
+            },
+        )
+        task.work_request_id = 1
+        task.workspace_name = "workspace"
+        debusine_mock = self.mock_debusine(task)
+        debusine_mock.download_artifact.return_value = signing_input_response
+        debusine_mock.asset_permission_check.return_value = (
+            AssetPermissionCheckResponse(
+                has_permission=False,
+                username="testuser",
+                user_id=123,
+                resource={},
+            )
+        )
+        debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
+        self.addCleanup(debug_log_files_directory.cleanup)
+        task._debug_log_files_directory = debug_log_files_directory
+        destination = self.create_temporary_directory()
+
+        self.assertFalse(task.fetch_input(destination))
+
+        self.assertEqual(
+            Path(debug_log_files_directory.name, "fetch_input.log").read_text(),
+            f"User testuser does not have permission to sign "
+            f"{signing_input_id} with uefi:{fingerprint}.\n",
+        )
+        debusine_mock.asset_permission_check.assert_called_once_with(
+            asset_category=AssetCategory.SIGNING_KEY,
+            asset_slug=f"uefi:{fingerprint}",
+            permission_name="sign_with",
+            artifact_id=signing_input_id,
+            work_request_id=task.work_request_id,
+            workspace=task.workspace_name,
+        )
+
     def test_run_error(self) -> None:
         """run() records an error result if signing fails."""
         key = Key.objects.create(
@@ -513,10 +599,22 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         execute_directory = self.create_temporary_directory()
         (image := download_directory / "image").touch()
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": key.fingerprint,
+            }
         )
         task.work_request_id = 1
         task._files = {2: self.make_files_response({"image": image})}
+        task._permission_checks = {
+            2: AssetPermissionCheckResponse(
+                has_permission=True,
+                username="testuser",
+                user_id=123,
+                resource={},
+            )
+        }
         task._key = key
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
@@ -543,10 +641,22 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         execute_directory = self.create_temporary_directory()
         (image := download_directory / "image").touch()
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": key.fingerprint,
+            }
         )
         task.work_request_id = 1
         task._files = {2: self.make_files_response({"image": image})}
+        task._permission_checks = {
+            2: AssetPermissionCheckResponse(
+                has_permission=True,
+                username="testuser",
+                user_id=123,
+                resource={},
+            )
+        }
         task._key = key
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
@@ -565,6 +675,9 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
             execute_directory / "output" / "image.sig",
             SigningMode.DETACHED,
             1,
+            username="testuser",
+            user_id=123,
+            resource={},
             log_file=mock.ANY,
         )
 
@@ -581,12 +694,24 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         (image := download_directory / "image").touch()
         (foo_ko := download_directory / "foo.ko").touch()
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": key.fingerprint,
+            }
         )
         task.work_request_id = 1
         task._files = {
             2: self.make_files_response({"image": image, "foo.ko": foo_ko})
         }
+        task._permission_checks = {
+            2: AssetPermissionCheckResponse(
+                has_permission=True,
+                username="testuser",
+                user_id=123,
+                resource={},
+            )
+        }
         task._key = key
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
         self.addCleanup(debug_log_files_directory.cleanup)
@@ -623,7 +748,11 @@ class SignTests(SignTestMixin, TestCase, DjangoTestCase):
         execute_directory = self.create_temporary_directory()
         (foo_ko := download_directory / "foo.ko").touch()
         task = Sign(
-            task_data={"purpose": KeyPurpose.UEFI, "unsigned": [2], "key": 1}
+            task_data={
+                "purpose": KeyPurpose.UEFI,
+                "unsigned": [2],
+                "key": key.fingerprint,
+            }
         )
         task.work_request_id = 1
         task._files = {2: self.make_files_response({"foo.ko": foo_ko})}
@@ -667,12 +796,6 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
             key = Key.objects.generate(
                 Key.Purpose.UEFI, "A UEFI key", 1, log_file
             )
-        signing_key = SigningKeyArtifact.create(
-            KeyPurpose.UEFI,
-            fingerprint=key.fingerprint,
-            public_key=key.public_key,
-        )
-        signing_key_id = 1
         temp_path = self.create_temporary_directory()
         (minimal_pecoff_path := temp_path / "image").write_bytes(
             minimal_pecoff_image
@@ -688,7 +811,6 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
         responses = {
             artifact_id: self.make_artifact_response(artifact, artifact_id)
             for artifact, artifact_id in (
-                (signing_key, signing_key_id),
                 (signing_inputs[0], signing_input_ids[0]),
                 (signing_inputs[1], signing_input_ids[1]),
             )
@@ -705,6 +827,8 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
                 shutil.copy(minimal_pecoff_path, destination)
             elif artifact_id == signing_input_ids[1]:
                 shutil.copy(minimal_pecoff_path2, destination)
+            else:  # pragma: no cover
+                pass
             return responses[artifact_id]
 
         def upload_artifact(
@@ -729,16 +853,23 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
             task_data={
                 "purpose": KeyPurpose.UEFI,
                 "unsigned": signing_input_ids,
-                "key": signing_key_id,
+                "key": key.fingerprint,
             },
             dynamic_task_data={
                 "unsigned_ids": signing_input_ids,
-                "key_id": signing_key_id,
             },
         )
         debusine_mock = self.mock_debusine(task)
         debusine_mock.download_artifact.side_effect = download_artifact
         debusine_mock.upload_artifact.side_effect = upload_artifact
+        debusine_mock.asset_permission_check.return_value = (
+            AssetPermissionCheckResponse(
+                has_permission=True,
+                username="testuser",
+                user_id=123,
+                resource={},
+            )
+        )
         task.work_request_id = 1
         task.workspace_name = "System"
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
@@ -809,11 +940,9 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
                 mock.call(
                     signing_output_ids[0], signing_input_ids[0], "relates-to"
                 ),
-                mock.call(signing_output_ids[0], signing_key_id, "relates-to"),
                 mock.call(
                     signing_output_ids[1], signing_input_ids[1], "relates-to"
                 ),
-                mock.call(signing_output_ids[1], signing_key_id, "relates-to"),
             ]
         )
 
@@ -848,25 +977,15 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
             key = Key.objects.generate(
                 Key.Purpose.OPENPGP, "An OpenPGP key", 1, log_file
             )
-        signing_key = SigningKeyArtifact.create(
-            KeyPurpose.OPENPGP,
-            fingerprint=key.fingerprint,
-            public_key=key.public_key,
-        )
-        signing_key_id = 1
         temp_path = self.create_temporary_directory()
         (changes_path := temp_path / "foo.changes").write_text(
             "A changes file\n"
         )
         signing_input = SigningInputArtifact.create([changes_path], temp_path)
         signing_input_id = 2
-        responses = {
-            response.id: response
-            for response in (
-                self.make_artifact_response(signing_key, signing_key_id),
-                self.make_artifact_response(signing_input, signing_input_id),
-            )
-        }
+        signing_input_response = self.make_artifact_response(
+            signing_input, signing_input_id
+        )
         uploaded_paths: list[Path] = []
         output_path = self.create_temporary_directory()
         signing_output_id = 3
@@ -874,9 +993,9 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
         def download_artifact(
             artifact_id: int, destination: Path, **kwargs: Any  # noqa: U100
         ) -> ArtifactResponse:
-            if artifact_id == signing_input_id:
-                shutil.copy(changes_path, destination)
-            return responses[artifact_id]
+            assert artifact_id == signing_input_id
+            shutil.copy(changes_path, destination)
+            return signing_input_response
 
         def upload_artifact(
             local_artifact: LocalArtifact[Any], **kwargs: Any
@@ -891,16 +1010,23 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
             task_data={
                 "purpose": KeyPurpose.OPENPGP,
                 "unsigned": [signing_input_id],
-                "key": signing_key_id,
+                "key": key.fingerprint,
             },
             dynamic_task_data={
                 "unsigned_ids": [signing_input_id],
-                "key_id": signing_key_id,
             },
         )
         debusine_mock = self.mock_debusine(task)
         debusine_mock.download_artifact.side_effect = download_artifact
         debusine_mock.upload_artifact.side_effect = upload_artifact
+        debusine_mock.asset_permission_check.return_value = (
+            AssetPermissionCheckResponse(
+                has_permission=True,
+                username="testuser",
+                user_id=123,
+                resource={},
+            )
+        )
         task.work_request_id = 1
         task.workspace_name = "System"
         debug_log_files_directory = TemporaryDirectory(prefix="debusine-tests-")
@@ -912,6 +1038,15 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
         ):
             self.assertTrue(task.execute())
 
+        debusine_mock.asset_permission_check.assert_called_once_with(
+            asset_category=AssetCategory.SIGNING_KEY,
+            asset_slug=f"openpgp:{key.fingerprint}",
+            permission_name="sign_with",
+            artifact_id=signing_input_id,
+            work_request_id=task.work_request_id,
+            workspace=task.workspace_name,
+        )
+
         # The task uploaded an artifact with the expected properties.
         self.assertEqual(len(uploaded_paths), 2)
         debusine_mock.upload_artifact.assert_has_calls(
@@ -950,11 +1085,10 @@ class SignIntegrationTests(SignTestMixin, TestCase, TransactionTestCase):
                 ),
             ]
         )
-        debusine_mock.relation_create.assert_has_calls(
-            [
-                mock.call(signing_output_id, signing_input_id, "relates-to"),
-                mock.call(signing_output_id, signing_key_id, "relates-to"),
-            ]
+        debusine_mock.relation_create.assert_any_call(
+            signing_output_id,
+            signing_input_id,
+            "relates-to",
         )
 
         # The file was properly signed.
