@@ -97,10 +97,16 @@ class Autopkgtest(
             and worker_metadata.get(available_key, False)
         )
 
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> AutopkgtestDynamicData:
-        """Resolve artifact lookups for this task."""
+        """
+        Resolve artifact lookups for this task.
+
+        :subject: source package name out of ``input:source_artifact``
+        :runtime_context: ``host_architecture:executor_backend``
+        :configuration_context: codename of ``environment``
+        """
         input_source_artifact = task_database.lookup_single_artifact(
             self.data.input.source_artifact
         )
