@@ -47,10 +47,14 @@ class Blhc(
         # list of build logs to be checked
         self._blhc_target: Path | None = None
 
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface
     ) -> BlhcDynamicData:
-        """Resolve artifact lookups for this task."""
+        """
+        Resolve artifact lookups for this task.
+
+        :subject: source package name out of the analyzed build log
+        """
         input_artifact = task_database.lookup_single_artifact(
             self.data.input.artifact
         )
