@@ -8,7 +8,7 @@
 # contained in the LICENSE file.
 
 """Task to use debdiff in debusine."""
-
+from itertools import chain
 from pathlib import Path
 from typing import Any
 
@@ -127,6 +127,16 @@ class DebDiff(
             subject=subject,
         )
 
+    def get_source_artifacts_ids(self) -> list[int]:
+        """Return the list of source artifact IDs used by this task."""
+        if self.dynamic_data is None:
+            return []
+
+        input_ids = self.dynamic_data.input_source_artifacts_ids or []
+        binary_ids = self.dynamic_data.input_binary_artifacts_ids or []
+
+        return list(chain(input_ids, *binary_ids))
+
     def _cmdline(self) -> list[str]:
         """
         Build the debdiff command line.
@@ -192,7 +202,8 @@ class DebDiff(
             ["apt-get", "update"], run_as_root=True, check=True
         )
         self.executor_instance.run(
-            ["apt-get", "--yes", "install", "devscripts"],
+            # diffstat needed for the flag --diffstat
+            ["apt-get", "--yes", "install", "devscripts", "diffstat"],
             run_as_root=True,
             check=True,
         )
@@ -245,6 +256,7 @@ class DebDiff(
         self, exec_directory: Path, *, execution_success: bool  # noqa: U100
     ) -> None:
         """Upload the DebDiffArtifact with the files and relationships."""
+        assert self.dynamic_data
         if not self.debusine:
             raise AssertionError("self.debusine not set")
 
@@ -267,6 +279,14 @@ class DebDiff(
                 uploaded.id, source_artifact_id, RelationType.RELATES_TO
             )
 
+        if (
+            binary_artifacts_ids := self.dynamic_data.input_binary_artifacts_ids
+        ) is not None:
+            for binary_artifact_id in chain.from_iterable(binary_artifacts_ids):
+                self.debusine.relation_create(
+                    uploaded.id, binary_artifact_id, RelationType.RELATES_TO
+                )
+
     def task_succeeded(
         self, returncode: int | None, execute_directory: Path  # noqa: U100
     ) -> bool:
