@@ -322,6 +322,14 @@ class IncusLXCImageCache(IncusImageCache):
             ),
             "/lib/systemd/system/systemd-networkd.service",
         )
+        self.add_tar_symlink(
+            dest,
+            (
+                "rootfs/etc/systemd/system/network-online.target.wants/"
+                "systemd-networkd-wait-online.service"
+            ),
+            "/lib/systemd/system/systemd-networkd-wait-online.service",
+        )
         # Install lxc.generator. Contains hacks for systemd < 257
         self.add_tar_file(
             dest,
