@@ -1,4 +1,4 @@
-# Copyright 2023 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -14,6 +14,7 @@ from typing import Any, AnyStr, Literal
 from unittest.mock import Mock, call, patch
 
 from debusine.tasks.executors.base import (
+    ExecutorStatistics,
     InstanceInterface,
     InstanceNotRunning,
     InstanceRunning,
@@ -66,6 +67,13 @@ class TestInstance(InstanceInterface):
         """Not implemented."""
         raise NotImplementedError()
 
+    @classmethod
+    def get_statistics(
+        cls, instance: InstanceInterface | None
+    ) -> ExecutorStatistics:
+        """Not implemented."""
+        raise NotImplementedError()
+
 
 class ExecutorTests(TestCase):
     """Unit tests for the Executor basic interface."""
