@@ -355,6 +355,10 @@ class IncusLXCImageCacheTests(IncusImageCacheCommonTests):
                         "rootfs/etc/systemd/system/multi-user.target.wants/"
                         "systemd-networkd.service"
                     ),
+                    (
+                        "rootfs/etc/systemd/system/network-online.target.wants/"
+                        "systemd-networkd-wait-online.service"
+                    ),
                     "rootfs/etc/systemd/system-generators/lxc",
                 ],
             )
