@@ -102,6 +102,38 @@ class ExtractForSigning(
         assert isinstance(template_artifact.data, DebianBinaryPackage)
         binary_package_name = get_binary_package_name(template_artifact.data)
 
+        binary_artifacts = task_database.lookup_multiple_artifacts(
+            self.data.input.binary_artifacts
+        )
+        individual_binary_artifact_ids = []
+        upload_artifact_ids = []
+        for i, binary_artifact in enumerate(binary_artifacts):
+            self.ensure_artifact_categories(
+                configuration_key=f"input.binary_artifacts[{i}]",
+                category=binary_artifact.category,
+                expected=(
+                    ArtifactCategory.BINARY_PACKAGE,
+                    ArtifactCategory.UPLOAD,
+                ),
+            )
+            match binary_artifact.category:
+                case ArtifactCategory.BINARY_PACKAGE:
+                    individual_binary_artifact_ids.append(binary_artifact.id)
+                case ArtifactCategory.UPLOAD:
+                    upload_artifact_ids.append(binary_artifact.id)
+                case _ as unreachable:
+                    raise AssertionError(
+                        f"Unexpected artifact category: {unreachable}"
+                    )
+        if upload_artifact_ids:
+            individual_binary_artifact_ids.extend(
+                task_database.find_related_artifacts(
+                    upload_artifact_ids,
+                    ArtifactCategory.BINARY_PACKAGE,
+                    relation_type=RelationType.EXTENDS,
+                ).get_ids()
+            )
+
         return ExtractForSigningDynamicData(
             environment_id=self.get_environment(
                 task_database,
@@ -109,9 +141,7 @@ class ExtractForSigning(
                 default_category=CollectionCategory.ENVIRONMENTS,
             ).id,
             input_template_artifact_id=template_artifact.id,
-            input_binary_artifacts_ids=task_database.lookup_multiple_artifacts(
-                self.data.input.binary_artifacts
-            ).get_ids(),
+            input_binary_artifacts_ids=individual_binary_artifact_ids,
             subject=binary_package_name,
         )
 
@@ -119,8 +149,9 @@ class ExtractForSigning(
         """Download the required artifacts."""
         assert self.dynamic_data is not None
 
+        (template_destination := destination / "template").mkdir()
         artifact = self.fetch_artifact(
-            self.dynamic_data.input_template_artifact_id, destination
+            self.dynamic_data.input_template_artifact_id, template_destination
         )
         if artifact.category != ArtifactCategory.BINARY_PACKAGE:
             self.append_to_log_file(
@@ -145,28 +176,23 @@ class ExtractForSigning(
                 ],
             )
             return False
-        self._template_path = destination / template_file
+        self._template_path = template_destination / template_file
 
+        (binary_destination := destination / "binary").mkdir()
         self._binary_artifacts = {}
         self._binary_paths = {}
         for artifact_id in self.dynamic_data.input_binary_artifacts_ids:
-            artifact = self.fetch_artifact(artifact_id, destination)
-            if artifact.category != ArtifactCategory.BINARY_PACKAGE:
-                self.append_to_log_file(
-                    "fetch_input.log",
-                    [
-                        f"Expected each of binary_artifacts to be of category "
-                        f"{ArtifactCategory.BINARY_PACKAGE}; got "
-                        f"{artifact.category}"
-                    ],
-                )
-                return False
+            artifact = self.fetch_artifact(artifact_id, binary_destination)
+            # Checked by build_dynamic_data.
+            assert artifact.category == ArtifactCategory.BINARY_PACKAGE
             binary_deb_name = artifact.data["deb_fields"]["Package"]
             self._binary_artifacts[binary_deb_name] = artifact_id
             # Checked by BinaryPackage validator.
             assert len(artifact.files) == 1
             binary_file = list(artifact.files)[0]
-            self._binary_paths[binary_deb_name] = destination / binary_file
+            self._binary_paths[binary_deb_name] = (
+                binary_destination / binary_file
+            )
 
         return True
 
