@@ -37,10 +37,6 @@ from debusine.tasks.models import (
 from debusine.tasks.server import TaskDatabaseInterface
 
 
-class LintianVersionParseError(Exception):
-    """The output of "lintian --version" cannot be parsed."""
-
-
 class Lintian(
     RunCommandTask[LintianData, LintianDynamicData],
     BaseTaskWithExecutor[LintianData, LintianDynamicData],
@@ -678,18 +674,14 @@ class Lintian(
             )
 
         out = self.executor_instance.run(
-            ["lintian", "--version"],
+            ["lintian", "--print-version"],
             encoding="utf-8",
             errors="ignore",
             text=True,
             check=True,
         ).stdout
 
-        m = re.match("Lintian v(.*)", out)
-        if m is None:
-            raise LintianVersionParseError(out)
-
-        return m.group(1)
+        return out.strip()
 
     def get_label(self) -> str:
         """Return the task label."""
