@@ -1257,22 +1257,22 @@ class MergeUploadsInput(BaseTaskDataModel):
     uploads: LookupMultiple
 
 
-class MergeUploadsData(BaseTaskDataWithExecutor):
+class MergeUploadsData(BaseTaskData):
     """In memory task data for the MergeUploads task."""
 
-    # BaseTaskDataWithExecutor declares this as optional, but it's required
-    # here.
-    environment: LookupSingle
+    # No longer used.
+    backend: BackendType = BackendType.AUTO
+    # No longer used.
+    environment: LookupSingle | None = None
 
     input: MergeUploadsInput
 
 
-class MergeUploadsDynamicData(BaseDynamicTaskDataWithExecutor):
+class MergeUploadsDynamicData(BaseDynamicTaskData):
     """Expanded dynamic data for the MergeUploads task."""
 
-    # BaseDynamicTaskDataWithExecutor declares this as optional, but it's
-    # required here.
-    environment_id: int
+    # No longer used.
+    environment_id: int | None = None
 
     input_uploads_ids: list[int]
 
