@@ -118,16 +118,6 @@ class SimpleSystemImageBuild(SystemBootstrap[SystemImageBuildData]):
                 cmd.append(f"--include={package}")
         # End deal with the keyrings
 
-        # Used by `upload_artifacts`
-        cmd.extend(
-            [
-                "--customize-hook=download /etc/os-release "
-                f"{shlex.quote(self._OS_RELEASE_FILE)}",
-                "--customize-hook=tar-out /var/lib/dpkg "
-                f"{shlex.quote(self._VAR_LIB_DPKG)}.tar",
-            ]
-        )
-
         # customization_script
         if script := self._customization_script:
             # special case autopkgtest script
@@ -147,25 +137,62 @@ class SimpleSystemImageBuild(SystemBootstrap[SystemImageBuildData]):
                     ]
                 )
 
-        cmd.append(
-            "--customize-hook=copy-in /usr/lib/python3/dist-packages/"
-            "debusine/tasks/data/overlays/incus-agent/ /"
+        # Used by `upload_artifacts`
+        cmd.extend(
+            [
+                (
+                    "--customize-hook=download /etc/os-release "
+                    f"{shlex.quote(self._OS_RELEASE_FILE)}"
+                ),
+                (
+                    "--customize-hook=tar-out /var/lib/dpkg "
+                    f"{shlex.quote(self._VAR_LIB_DPKG)}.tar"
+                ),
+            ]
         )
-        cmd.append(
-            "--customize-hook=copy-in /usr/lib/python3/dist-packages/"
-            "debusine/tasks/data/overlays/systemd-boot/ /"
+
+        overlay_dir = (
+            "/usr/lib/python3/dist-packages/debusine/tasks/data/overlays"
         )
+        for overlay, path, mode in (
+            ("incus-agent", "lib/systemd/system/incus-agent.service", 0o644),
+            ("incus-agent", "lib/systemd/incus-agent-setup", 0o755),
+            ("incus-agent", "lib/udev/rules.d/99-incus-agent.rules", 0o644),
+            (
+                "systemd-boot",
+                "etc/kernel/postinst.d/zz-update-systemd-boot",
+                0o755,
+            ),
+            (
+                "systemd-boot",
+                "etc/kernel/postrm.d/zz-update-systemd-boot",
+                0o755,
+            ),
+        ):
+            cmd.extend(
+                [
+                    (
+                        f"--customize-hook=upload "
+                        f"{overlay_dir}/{overlay}/{path} /{path}"
+                    ),
+                    f'--customize-hook=chmod {mode:o} "$1/{path}"',
+                ]
+            )
 
         if variant := self.data.bootstrap_options.variant:
             cmd.append(f"--variant={variant}")
 
         extra_packages = self.data.bootstrap_options.extra_packages
 
-        if kernel := self.data.disk_image.kernel_package:
-            extra_packages.append(kernel)
+        kernel = self.data.disk_image.kernel_package
+        if kernel is None:
+            # Only introduced in bullseye
+            # We require the generic kernel for 9p support (#1027174)
+            kernel = "linux-image-generic"
+        extra_packages.append(kernel)
 
-        if extra_packages:
-            cmd.append("--include=" + ",".join(extra_packages))
+        assert extra_packages
+        cmd.append("--include=" + ",".join(extra_packages))
 
         cmd.append(str(self._host_sources_file))
 
