@@ -22,7 +22,7 @@ class SystemImageBuild(
 
     TASK_VERSION = 1
 
-    def compute_dynamic_data(
+    def build_dynamic_data(
         self, task_database: TaskDatabaseInterface  # noqa: U100
     ) -> BaseDynamicTaskData:
         """Compute dynamic data."""
