@@ -12,7 +12,8 @@ import datetime
 import hashlib
 import shlex
 import tarfile
-from collections.abc import Mapping
+from collections.abc import Collection as AbcCollection
+from collections.abc import Mapping, Sequence
 from io import BytesIO
 from pathlib import Path
 from shutil import rmtree
@@ -32,6 +33,7 @@ from debusine.client.models import (
     ArtifactResponse,
     FileResponse,
     FilesResponseType,
+    RelationType,
     StrMaxLength255,
 )
 from debusine.tasks import BaseExternalTask, BaseTask, BaseTaskWithExecutor
@@ -85,11 +87,19 @@ class FakeTaskDatabaseBase(TaskDatabaseInterface):
             ]
             | None
         ) = None,
+        relations: (
+            Mapping[
+                tuple[Sequence[int], ArtifactCategory, RelationType],
+                list[ArtifactInfo],
+            ]
+            | None
+        ) = None,
         settings: dict[str, str] | None = None,
     ) -> None:
         """Construct a :py:class:`FakeTaskDatabase`."""
         self.single_lookups = single_lookups or {}
         self.multiple_lookups = multiple_lookups or {}
+        self.relations = relations or {}
         self.settings = settings or {}
 
     @overload
@@ -130,6 +140,19 @@ class FakeTaskDatabaseBase(TaskDatabaseInterface):
             self.multiple_lookups[(lookup, default_category)]
         )
 
+    def find_related_artifacts(
+        self,
+        artifact_ids: AbcCollection[int],
+        target_category: ArtifactCategory,
+        relation_type: RelationType = RelationType.RELATES_TO,
+    ) -> MultipleArtifactInfo:
+        """Pretend to find artifacts via relations."""
+        return MultipleArtifactInfo(
+            self.relations[
+                (tuple(artifact_ids), target_category, relation_type)
+            ]
+        )
+
     @overload
     def lookup_single_collection(
         self,
@@ -513,13 +536,19 @@ class ExternalTaskHelperMixin(
             files_to_upload=[],
         )
 
-    def fake_debian_upload_artifact(self) -> ArtifactResponse:
+    def fake_debian_upload_artifact(
+        self,
+        source_package_name: str = "foo",
+        architecture: str = "source",
+        names: list[str] | None = None,
+    ) -> ArtifactResponse:
         """Create a fake ArtifactResponse for a debian:upload."""
-        names = [
-            "foo_1.0.orig.tar.xz",
-            "foo_1.0-1.debian.tar.xz",
-            "foo_1.0-1.dsc",
-        ]
+        if names is None:
+            names = [
+                "{source_package_name}_1.0.orig.tar.xz",
+                "{source_package_name}_1.0-1.debian.tar.xz",
+                "{source_package_name}_1.0-1.dsc",
+            ]
         return ArtifactResponse(
             id=8,
             workspace="Testing",
@@ -530,7 +559,7 @@ class ExternalTaskHelperMixin(
             data={
                 "type": "dpkg",
                 "changes_fields": {
-                    "Architecture": "source",
+                    "Architecture": architecture,
                     "Files": [{"name": name} for name in names],
                 },
             },
@@ -552,7 +581,10 @@ class ExternalTaskHelperMixin(
                             pydantic.AnyUrl, f"https://example.com/{name}"
                         ),
                     )
-                    for name in ["foo_1.0-1_source.changes", *names]
+                    for name in [
+                        f"{source_package_name}_1.0-1_{architecture}.changes",
+                        *names,
+                    ]
                 }
             ),
             files_to_upload=[],
