@@ -13,6 +13,11 @@ from pathlib import Path
 from unittest.mock import call
 
 from debusine.artifacts import BlhcArtifact
+from debusine.artifacts.models import (
+    ArtifactCategory,
+    DebianPackageBuildLog,
+    EmptyArtifactData,
+)
 from debusine.client.models import (
     LookupResultType,
     LookupSingleResponse,
@@ -20,6 +25,7 @@ from debusine.client.models import (
 )
 from debusine.tasks import Blhc, TaskConfigError
 from debusine.tasks.models import BlhcDynamicData
+from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
@@ -27,8 +33,8 @@ from debusine.tasks.tests.helper_mixin import (
 from debusine.test import TestCase
 
 
-class BlhcTaskTests(ExternalTaskHelperMixin[Blhc], TestCase):
-    """Test the Blhc Task class."""
+class BlhcTests(ExternalTaskHelperMixin[Blhc], TestCase):
+    """Test the Blhc class."""
 
     SAMPLE_TASK_DATA = {
         "input": {"artifact": 421},
@@ -91,15 +97,46 @@ class BlhcTaskTests(ExternalTaskHelperMixin[Blhc], TestCase):
         task_db = FakeTaskDatabase(
             single_lookups={
                 # input.artifact
-                (421, None): 421
+                (421, None): ArtifactInfo(
+                    id=421,
+                    category=ArtifactCategory.PACKAGE_BUILD_LOG,
+                    data=DebianPackageBuildLog(
+                        source="hello",
+                        version="1.0",
+                        filename="hello_1.0_amd64.buildlog",
+                    ),
+                )
             }
         )
 
         self.assertEqual(
             self.task.compute_dynamic_data(task_db),
-            BlhcDynamicData(input_artifact_id=421),
+            BlhcDynamicData(
+                input_artifact_id=421, subject="hello", runtime_context=None
+            ),
         )
 
+    def test_compute_dynamic_data_raise_task_config_error(self) -> None:
+        """Artifact input.artifact has unexpected category."""
+        task_db = FakeTaskDatabase(
+            single_lookups={
+                # input.artifact
+                (421, None): ArtifactInfo(
+                    id=421,
+                    category=ArtifactCategory.SOURCE_PACKAGE,
+                    data=EmptyArtifactData(),
+                )
+            }
+        )
+
+        with self.assertRaisesRegex(
+            TaskConfigError,
+            r"^input.artifact: unexpected artifact category: "
+            r"'debian:source-package'. "
+            r"Valid categories: \['debian:package-build-log'\]$",
+        ):
+            self.task.compute_dynamic_data(task_db),
+
     def test_configure_for_execution_from_artifact_error_no_build_logs(
         self,
     ) -> None:
