@@ -484,7 +484,7 @@ class ExtractForSigningTests(
         self.assertEqual(self.task._template_deb_name, "hello-signed-template")
         self.assertEqual(
             self.task._template_path,
-            destination / "hello-signed-template_1.0+1_amd64.deb",
+            destination / "template" / "hello-signed-template_1.0+1_amd64.deb",
         )
         self.assertEqual(
             self.task._binary_artifacts, {"hello": 3, "libhello1": 4}
@@ -492,8 +492,10 @@ class ExtractForSigningTests(
         self.assertEqual(
             self.task._binary_paths,
             {
-                "hello": destination / "hello_1.0-1_amd64.deb",
-                "libhello1": destination / "libhello1_1.0-1_amd64.deb",
+                "hello": destination / "binary" / "hello_1.0-1_amd64.deb",
+                "libhello1": (
+                    destination / "binary" / "libhello1_1.0-1_amd64.deb"
+                ),
             },
         )
 
@@ -608,12 +610,14 @@ class ExtractForSigningTests(
         self.task._local_execute_directory = self.create_temporary_directory()
         self.task._template_deb_name = "hello-signed-template"
         self.task._template_path = (
-            self.task._remote_execute_directory
+            download_directory
+            / "template"
             / "hello-signed-template_1.0+1_amd64.deb"
         )
         manifest_path = PurePath(
             "usr/share/code-signing", self.task._template_deb_name, "files.json"
         )
+        self.task._template_path.parent.mkdir()
         ArtifactPlayground.write_deb_file(
             self.task._template_path,
             data_files={manifest_path: json.dumps({"packages": {}}).encode()},
@@ -642,7 +646,8 @@ class ExtractForSigningTests(
         self.assertTrue(self.task.configure_for_execution(download_directory))
         assert isinstance(self.task.executor_instance, mock.MagicMock)
         self.task.executor_instance.run.side_effect = _mock_run
-        hello_path = download_directory / "hello_1.0-1_amd64.deb"
+        hello_path = download_directory / "binary" / "hello_1.0-1_amd64.deb"
+        hello_path.parent.mkdir()
         ArtifactPlayground.write_deb_file(hello_path)
         self.task._binary_paths = {"hello": hello_path}
         self.task._remote_execute_directory = self.create_temporary_directory()
@@ -674,7 +679,8 @@ class ExtractForSigningTests(
         self.assertTrue(self.task.configure_for_execution(download_directory))
         assert isinstance(self.task.executor_instance, mock.MagicMock)
         self.task.executor_instance.run.side_effect = _mock_run
-        hello_path = download_directory / "hello_1.0-1_amd64.deb"
+        hello_path = download_directory / "binary" / "hello_1.0-1_amd64.deb"
+        hello_path.parent.mkdir()
         ArtifactPlayground.write_deb_file(hello_path)
         self.task._binary_paths = {"hello": hello_path}
         self.task._remote_execute_directory = self.create_temporary_directory()
@@ -700,7 +706,8 @@ class ExtractForSigningTests(
         self.assertTrue(self.task.configure_for_execution(download_directory))
         assert isinstance(self.task.executor_instance, mock.MagicMock)
         self.task.executor_instance.run.side_effect = _mock_run
-        hello_path = download_directory / "hello_1.0-1_amd64.deb"
+        hello_path = download_directory / "binary" / "hello_1.0-1_amd64.deb"
+        hello_path.parent.mkdir()
         ArtifactPlayground.write_deb_file(
             hello_path, data_symlinks={PurePath("evil-symlink"): PurePath("..")}
         )
@@ -736,7 +743,8 @@ class ExtractForSigningTests(
         self.assertTrue(self.task.configure_for_execution(download_directory))
         assert isinstance(self.task.executor_instance, mock.MagicMock)
         self.task.executor_instance.run.side_effect = _mock_run
-        hello_path = download_directory / "hello_1.0-1_amd64.deb"
+        hello_path = download_directory / "binary" / "hello_1.0-1_amd64.deb"
+        hello_path.parent.mkdir()
         ArtifactPlayground.write_deb_file(
             hello_path, data_files={PurePath("boot/vmlinuz"): b"vmlinuz"}
         )
@@ -768,7 +776,8 @@ class ExtractForSigningTests(
         self.assertTrue(self.task.configure_for_execution(download_directory))
         assert isinstance(self.task.executor_instance, mock.MagicMock)
         self.task.executor_instance.run.side_effect = _mock_run
-        hello_path = download_directory / "hello_1.0-1_amd64.deb"
+        hello_path = download_directory / "binary" / "hello_1.0-1_amd64.deb"
+        hello_path.parent.mkdir()
         ArtifactPlayground.write_deb_file(
             hello_path, data_files={PurePath("boot/vmlinuz"): b"vmlinuz"}
         )
@@ -936,6 +945,7 @@ class ExtractForSigningTests(
             for file_name in artifacts[artifact_id].files:
                 target = destination / file_name
                 target.parent.mkdir(parents=True, exist_ok=True)
+                assert not target.exists()
                 shutil.copy2(artifact_files[(artifact_id, file_name)], target)
             return artifacts[artifact_id]
 
