@@ -31,7 +31,7 @@ from debusine.tasks.tests.helper_mixin import (
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
-from debusine.test.utils import (
+from debusine.test.test_utils import (
     create_artifact_response,
     create_system_tarball_data,
 )
