@@ -18,11 +18,15 @@ from unittest.mock import call
 from debusine.artifacts.local_artifact import DebianSystemTarballArtifact
 from debusine.tasks import MmDebstrap
 from debusine.tasks.models import (
+    BaseDynamicTaskData,
     MmDebstrapBootstrapOptions,
     MmDebstrapData,
     WorkerType,
 )
-from debusine.tasks.tests.helper_mixin import ExternalTaskHelperMixin
+from debusine.tasks.tests.helper_mixin import (
+    ExternalTaskHelperMixin,
+    FakeTaskDatabase,
+)
 from debusine.test import TestCase
 
 
@@ -32,7 +36,7 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
     SAMPLE_TASK_DATA = {
         "bootstrap_options": {
             "architecture": "amd64",
-            "extra_packages": ["hello"],
+            "extra_packages": ["hello", "python3"],
             "use_signed_by": True,
         },
         "bootstrap_repositories": [
@@ -57,6 +61,19 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
         """Initialize test."""
         self.configure_task()
 
+    def test_compute_dynamic_data(self) -> None:
+        """Test compute_dynamic_data."""
+        task_database = FakeTaskDatabase()
+
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_database),
+            BaseDynamicTaskData(
+                subject="bookworm",
+                runtime_context="amd64:None:hello,python3",
+                configuration_context="amd64",
+            ),
+        )
+
     def test_configure_task(self) -> None:
         """self.configure_task() does not raise any exception."""
         self.configure_task(self.SAMPLE_TASK_DATA)
@@ -202,7 +219,7 @@ class MmDebstrapTests(ExternalTaskHelperMixin[MmDebstrap], TestCase):
             'echo $? > "$1/test_sbin_init"',
             f'--customize-hook=download test_sbin_init {test_sbin_init}',
             '--customize-hook=rm "$1/test_sbin_init"',
-            "--include=hello",
+            "--include=hello,python3",
             "",
             "system.tar.xz",
             "",
