@@ -480,8 +480,23 @@ class SystemBootstrapDataTests(TestCase):
             }
         )
 
-    def test_keyring_file_url_not_under_usr_share_keyrings(self) -> None:
-        """Keyring file:// URLs not under /usr/share/keyrings/ are rejected."""
+    def test_keyring_file_url_under_usr_local_share_keyrings(self) -> None:
+        """Keyring file:// URLs under /usr/local/share/keyrings/ are allowed."""
+        task_models.SystemBootstrapRepository.parse_obj(
+            {
+                "mirror": "https://deb.debian.org/deb",
+                "suite": "bookworm",
+                "check_signature_with": "external",
+                "keyring": {
+                    "url": (
+                        "file:///usr/local/share/keyrings/local-keyring.gpg"
+                    )
+                },
+            }
+        )
+
+    def test_keyring_file_url_not_under_allowed_directory(self) -> None:
+        """Keyring file:// URLs not under an allowed directory are rejected."""
         common_kwargs = {
             "mirror": "https://deb.debian.org/deb",
             "suite": "bookworm",
@@ -489,7 +504,8 @@ class SystemBootstrapDataTests(TestCase):
         }
 
         error_msg = (
-            "file:// URLs for keyrings must be under /usr/share/keyrings/"
+            "file:// URLs for keyrings must be under /usr/share/keyrings/ or "
+            "/usr/local/share/keyrings/"
         )
         with self.assertRaisesRegex(ValueError, error_msg):
             task_models.SystemBootstrapRepository.parse_obj(
