@@ -46,7 +46,7 @@ from debusine.tasks.tests.helper_mixin import (
     FakeTaskDatabase,
 )
 from debusine.test import TestCase
-from debusine.test.utils import create_system_tarball_data
+from debusine.test.test_utils import create_system_tarball_data
 
 
 class PiupartsTests(ExternalTaskHelperMixin[Piuparts], TestCase):
