@@ -29,6 +29,50 @@ from debusine.tasks.tests.helper_mixin import (
 )
 from debusine.test import TestCase
 
+OVERLAYS = [
+    (
+        "--customize-hook=upload "
+        "/usr/lib/python3/dist-packages/debusine/tasks/data/overlays/"
+        "incus-agent/lib/systemd/system/incus-agent.service "
+        "/lib/systemd/system/incus-agent.service"
+    ),
+    '--customize-hook=chmod 644 "$1/lib/systemd/system/incus-agent.service"',
+    (
+        "--customize-hook=upload "
+        "/usr/lib/python3/dist-packages/debusine/tasks/data/overlays/"
+        "incus-agent/lib/systemd/incus-agent-setup "
+        "/lib/systemd/incus-agent-setup"
+    ),
+    '--customize-hook=chmod 755 "$1/lib/systemd/incus-agent-setup"',
+    (
+        "--customize-hook=upload "
+        "/usr/lib/python3/dist-packages/debusine/tasks/data/overlays/"
+        "incus-agent/lib/udev/rules.d/99-incus-agent.rules "
+        "/lib/udev/rules.d/99-incus-agent.rules"
+    ),
+    '--customize-hook=chmod 644 "$1/lib/udev/rules.d/99-incus-agent.rules"',
+    (
+        "--customize-hook=upload "
+        "/usr/lib/python3/dist-packages/debusine/tasks/data/overlays/"
+        "systemd-boot/etc/kernel/postinst.d/zz-update-systemd-boot "
+        "/etc/kernel/postinst.d/zz-update-systemd-boot"
+    ),
+    (
+        "--customize-hook=chmod 755 "
+        '"$1/etc/kernel/postinst.d/zz-update-systemd-boot"'
+    ),
+    (
+        "--customize-hook=upload "
+        "/usr/lib/python3/dist-packages/debusine/tasks/data/overlays/"
+        "systemd-boot/etc/kernel/postrm.d/zz-update-systemd-boot "
+        "/etc/kernel/postrm.d/zz-update-systemd-boot"
+    ),
+    (
+        "--customize-hook=chmod 755 "
+        '"$1/etc/kernel/postrm.d/zz-update-systemd-boot"'
+    ),
+]
+
 
 class SimpleSystemImageBuildTests(
     ExternalTaskHelperMixin[SimpleSystemImageBuild], TestCase
@@ -259,17 +303,18 @@ class SimpleSystemImageBuildTests(
             "--",
             "--verbose",
             "--hook-dir=/usr/share/mmdebstrap/hooks/maybe-jessie-or-older",
-            '--customize-hook=cd "$1" && '
-            "find etc/apt/sources.list.d -type f -delete",
+            (
+                '--customize-hook=cd "$1" && '
+                "find etc/apt/sources.list.d -type f -delete"
+            ),
             "--customize-hook=upload None /etc/apt/sources.list.d/file.sources",
             '--customize-hook=mkdir "$1/etc/apt/keyrings-debusine"',
             "--customize-hook=download /etc/os-release os-release",
             "--customize-hook=tar-out /var/lib/dpkg var_lib_dpkg.tar",
-            "--customize-hook=copy-in /usr/lib/python3/dist-packages/"
-            "debusine/tasks/data/overlays/incus-agent/ /",
-            "--customize-hook=copy-in /usr/lib/python3/dist-packages/"
-            "debusine/tasks/data/overlays/systemd-boot/ /",
-            "--include=hello",
+        ]
+        expected += OVERLAYS
+        expected += [
+            "--include=hello,linux-image-generic",
             "None",
         ]
 
@@ -303,11 +348,10 @@ class SimpleSystemImageBuildTests(
             '--customize-hook=mkdir "$1/etc/apt/keyrings-debusine"',
             f"--customize-hook=download /etc/os-release {os_release_file}",
             f"--customize-hook=tar-out /var/lib/dpkg {var_lib_dpkg}.tar",
-            "--customize-hook=copy-in /usr/lib/python3/dist-packages/"
-            "debusine/tasks/data/overlays/incus-agent/ /",
-            "--customize-hook=copy-in /usr/lib/python3/dist-packages/"
-            "debusine/tasks/data/overlays/systemd-boot/ /",
-            '--include=hello',
+        ]
+        expected += OVERLAYS
+        expected += [
+            '--include=hello,linux-image-generic',
             str(self.task._host_sources_file),
         ]
 
@@ -337,8 +381,7 @@ class SimpleSystemImageBuildTests(
         self.configure_task(override={"bootstrap_options": bootstrap_options})
         self.task._host_sources_file = Path("some-file.sources")
         cmdline = self.task._cmdline()
-        for arg in cmdline:
-            self.assertFalse(arg.startswith("--include="))
+        self.assertIn("--include=linux-image-generic", cmdline)
 
     def test_cmdline_extra_packages(self) -> None:
         """_cmdline() include --include=extra_packages."""
@@ -352,7 +395,7 @@ class SimpleSystemImageBuildTests(
         self.configure_task(override={"bootstrap_options": bootstrap_options})
         self.task._host_sources_file = Path("some-file.sources")
         cmdline = self.task._cmdline()
-        self.assertIn("--include=hello,bye", cmdline)
+        self.assertIn("--include=hello,bye,linux-image-generic", cmdline)
 
     def test_cmdline_kernel_package(self) -> None:
         """_cmdline() include --include=kernel_package."""
