@@ -40,7 +40,7 @@ from debusine.client.models import ArtifactResponse
 from debusine.tasks import (
     BaseExternalTask,
     BaseTask,
-    BaseTaskWithExecutor,
+    DefaultDynamicData,
     ExtraRepositoryMixin,
     RunCommandTask,
     Sbuild,
@@ -66,17 +66,19 @@ from debusine.tasks.server import ArtifactInfo
 from debusine.tasks.tests.helper_mixin import (
     ExternalTaskHelperMixin,
     FakeTaskDatabase,
-    TestTaskMixin,
+    TestBaseExternalTask,
+    TestBaseTask,
+    TestBaseTaskWithExecutor,
 )
 from debusine.test import TestCase
-from debusine.test.utils import (
+from debusine.test.test_utils import (
     create_system_tarball_data,
     preserve_task_registry,
 )
 
 
-class TestBaseTask(
-    TestTaskMixin, BaseExternalTask[BaseTaskData, BaseDynamicTaskData]
+class TestBaseTask1(
+    TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
 ):
     """Sample class to test BaseExternalTask class."""
 
@@ -87,9 +89,8 @@ class TestBaseTask(
         raise NotImplementedError()
 
 
-class TestBaseTaskWithExecutor(
-    TestTaskMixin,
-    BaseTaskWithExecutor[
+class TestBaseTaskWithExecutor1(
+    TestBaseTaskWithExecutor[
         BaseTaskDataWithExecutor, BaseDynamicTaskDataWithExecutor
     ],
 ):
@@ -113,7 +114,7 @@ class TestBaseTask2Data(BaseTaskData):
 
 
 class TestBaseTask2(
-    TestTaskMixin, BaseExternalTask[TestBaseTask2Data, BaseDynamicTaskData]
+    TestBaseExternalTask[TestBaseTask2Data, BaseDynamicTaskData],
 ):
     """Test BaseExternalTask class with data validation."""
 
@@ -126,9 +127,7 @@ class TestBaseTask2(
         raise NotImplementedError()
 
 
-class TestBaseTaskDataWithExecutorAndArchitecture(
-    TestTaskMixin, BaseTaskDataWithExecutor
-):
+class TestBaseTaskDataWithExecutorAndArchitecture(BaseTaskDataWithExecutor):
     """BaseTaskDataWithExecutor with a host architecture."""
 
     __test__ = False
@@ -137,8 +136,7 @@ class TestBaseTaskDataWithExecutorAndArchitecture(
 
 
 class TestBaseTaskWithExecutorAndArchitecture(
-    TestTaskMixin,
-    BaseTaskWithExecutor[
+    TestBaseTaskWithExecutor[
         TestBaseTaskDataWithExecutorAndArchitecture,
         BaseDynamicTaskDataWithExecutor,
     ],
@@ -155,7 +153,7 @@ class TestBaseTaskWithExecutorAndArchitecture(
 
 
 class TestExtraRepositoryMixin(
-    TestTaskMixin,
+    TestBaseTask[BaseTaskDataWithExtraRepositories, BaseDynamicTaskData],
     ExtraRepositoryMixin[
         BaseTaskDataWithExtraRepositories, BaseDynamicTaskData
     ],
@@ -167,6 +165,25 @@ class TestExtraRepositoryMixin(
         raise NotImplementedError()
 
 
+class TestDefaultDynamicData(
+    DefaultDynamicData[BaseTaskData],
+    TestBaseTask[BaseTaskData, BaseDynamicTaskData],
+):
+    """Test DefaultDynamicData."""
+
+    TASK_TYPE = TaskTypes.WORKER
+
+    def _execute(self) -> bool:
+        raise NotImplementedError()
+
+    def _upload_work_request_debug_logs(self) -> None:
+        raise NotImplementedError()
+
+    def run(self, execute_directory: Path) -> bool:
+        """Unused abstract method from BaseExternalTask."""
+        raise NotImplementedError()
+
+
 class BaseTaskTests(TestCase):
     """Unit tests for BaseTask class."""
 
@@ -174,7 +191,7 @@ class BaseTaskTests(TestCase):
 
     def setUp(self) -> None:
         """Create the shared attributes."""
-        self.task = TestBaseTask({})
+        self.task = TestBaseTask1({})
         self.task2 = TestBaseTask2({"foo": "bar"})
 
     def tearDown(self) -> None:
@@ -189,7 +206,7 @@ class BaseTaskTests(TestCase):
 
     def test_name_is_set(self) -> None:
         """task.name is built from class name."""
-        self.assertEqual(self.task.name, "testbasetask")
+        self.assertEqual(self.task.name, "testbasetask1")
 
     def test_logger_is_configured(self) -> None:
         """task.logger is available."""
@@ -329,7 +346,7 @@ class BaseTaskTests(TestCase):
     def test_analyze_worker_without_task_version(self) -> None:
         """analyze_worker() reports an unknown task version."""
         metadata = self.task.analyze_worker()
-        self.assertIsNone(metadata["testbasetask:version"])
+        self.assertIsNone(metadata["testbasetask1:version"])
 
     def test_analyze_worker_with_task_version(self) -> None:
         """analyze_worker() reports a task version."""
@@ -342,7 +359,10 @@ class BaseTaskTests(TestCase):
             BaseTask,
             "_sub_tasks",
             new={
-                TaskTypes.WORKER: {"task": TestBaseTask, "task2": TestBaseTask2}
+                TaskTypes.WORKER: {
+                    "task": TestBaseTask1,
+                    "task2": TestBaseTask2,
+                }
             },
         )
         patcher.start()
@@ -412,15 +432,13 @@ class BaseTaskTests(TestCase):
         with self.assertRaisesRegex(AssertionError, msg):
 
             class Somesubclassname(
-                TestTaskMixin,
-                BaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+                TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
             ):
                 def run(self, execute_directory: Path) -> bool:  # noqa: U100
                     raise NotImplementedError()
 
             class SomeSubclassName(
-                TestTaskMixin,
-                BaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+                TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
             ):
                 def run(self, execute_directory: Path) -> bool:  # noqa: U100
                     raise NotImplementedError()
@@ -432,7 +450,7 @@ class BaseTaskTests(TestCase):
         with self.assertRaisesRegex(AssertionError, msg):
 
             class Somesubclassname(
-                TestTaskMixin, BaseTask[BaseTaskData, BaseDynamicTaskData]
+                TestBaseTask[BaseTaskData, BaseDynamicTaskData]
             ):
                 TASK_TYPE = TaskTypes.SERVER
 
@@ -443,8 +461,7 @@ class BaseTaskTests(TestCase):
                     raise NotImplementedError()
 
             class SomeSubclassName(
-                TestTaskMixin,
-                BaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+                TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
             ):
                 def run(self, execute_directory: Path) -> bool:  # noqa: U100
                     raise NotImplementedError()
@@ -453,14 +470,13 @@ class BaseTaskTests(TestCase):
         with self.assertRaisesRegex(AssertionError, msg):
 
             class SomeSubclassName1(
-                TestTaskMixin,
-                BaseExternalTask[BaseTaskData, BaseDynamicTaskData],
+                TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData],
             ):
                 def run(self, execute_directory: Path) -> bool:  # noqa: U100
                     raise NotImplementedError()
 
             class Somesubclassname1(
-                TestTaskMixin, BaseTask[BaseTaskData, BaseDynamicTaskData]
+                TestBaseTask[BaseTaskData, BaseDynamicTaskData]
             ):
                 TASK_TYPE = TaskTypes.SERVER
 
@@ -657,13 +673,13 @@ class BaseTaskTests(TestCase):
     def test_task_names_worker(self) -> None:
         """BaseTask.task_names() returns all available worker tasks."""
         task_names = self.task.task_names(TaskTypes.WORKER)
-        self.assertTrue(len(task_names) > 0)
+        self.assertGreater(len(task_names), 0)
         self.assertIn("noop", task_names)
 
     def test_worker_task_names(self) -> None:
         """BaseTask.worker_task_names() returns all available worker tasks."""
         task_names = self.task.worker_task_names()
-        self.assertTrue(len(task_names) > 0)
+        self.assertGreater(len(task_names), 0)
         self.assertIn("noop", task_names)
 
     def test_is_worker_task(self) -> None:
@@ -725,9 +741,16 @@ class BaseTaskTests(TestCase):
         self.assertEqual(task.data.foo, "bar")
         self.assertIsNone(task.data.bar)
 
+    def test_compute_dynamic_data(self) -> None:
+        task_db = FakeTaskDatabase()
 
-class TestBaseExternalTask(
-    TestTaskMixin, BaseExternalTask[BaseTaskData, BaseDynamicTaskData]
+        self.assertEqual(
+            self.task.compute_dynamic_data(task_db), BaseDynamicTaskData()
+        )
+
+
+class TestBaseExternalTask1(
+    TestBaseExternalTask[BaseTaskData, BaseDynamicTaskData]
 ):
     """Sample class to test BaseExternalTask class."""
 
@@ -739,7 +762,7 @@ class TestBaseExternalTask(
 
 
 class TestBaseExternalTask2(
-    TestTaskMixin, BaseExternalTask[TestBaseTask2Data, BaseDynamicTaskData]
+    TestBaseExternalTask[TestBaseTask2Data, BaseDynamicTaskData]
 ):
     """Test BaseExternalTask class with data validation."""
 
@@ -757,7 +780,7 @@ class BaseExternalTaskTests(TestCase):
 
     def setUp(self) -> None:
         """Create the shared attributes."""
-        self.task = TestBaseExternalTask({})
+        self.task = TestBaseExternalTask1({})
         self.task2 = TestBaseExternalTask2({"foo": "bar"})
         self.worker_metadata = {"system:worker_type": WorkerType.EXTERNAL}
 
@@ -776,7 +799,7 @@ class BaseExternalTaskTests(TestCase):
         """Ensure can_run_on returns False if versions differ."""
         self.assertIsNone(self.task.TASK_VERSION)
         metadata = {**self.worker_metadata, **self.task.analyze_worker()}
-        metadata["testbaseexternaltask:version"] = 1
+        metadata["testbaseexternaltask1:version"] = 1
         self.assertFalse(self.task.can_run_on(metadata))
 
     def test_can_run_on_type_mismatch(self) -> None:
@@ -871,6 +894,26 @@ class BaseExternalTaskTests(TestCase):
             self.create_temporary_directory(), execution_success=True
         )
 
+    def test_execute_fetch_input_logs(self) -> None:
+        """_execute(): if fetch_input() raises an exception: log it."""
+        msg = "There is some exception"
+        exc = Exception(msg)
+
+        with mock.patch.object(
+            self.task, "fetch_input", autospec=True, side_effect=exc
+        ):
+            self.assertFalse(self.task._execute())
+
+        assert self.task._debug_log_files_directory
+        log = (
+            Path(self.task._debug_log_files_directory.name) / "fetch_input.log"
+        ).read_text()
+
+        self.assertRegex(log, f"^Exception type: Exception\nMessage: {msg}")
+
+        traceback_str = "".join(traceback.format_exception(exc))
+        self.assertIn(traceback_str, log)
+
     def test_execute_configure_for_execution_logs(self) -> None:
         """_execute(): if configure_for_execution() raises an exc: log it."""
         msg = "There is some exception"
@@ -1259,7 +1302,8 @@ class TestsTaskConfigError(TestCase):
 
 
 class RunCommandTaskForTesting(
-    TestTaskMixin, RunCommandTask[BaseTaskData, BaseDynamicTaskData]
+    TestBaseTask[BaseTaskData, BaseDynamicTaskData],
+    RunCommandTask[BaseTaskData, BaseDynamicTaskData],
 ):
     """Used in TaskTests."""
 
@@ -1951,7 +1995,7 @@ class BaseTaskWithExecutorTests(TestCase):
 
     def setUp(self) -> None:
         """Create the shared attributes."""
-        self.task = TestBaseTaskWithExecutor({})
+        self.task = TestBaseTaskWithExecutor1({})
 
     def tearDown(self) -> None:
         """Delete directory to avoid ResourceWarning."""
@@ -2238,10 +2282,21 @@ class TestTestTaskMixin(TestCase):
 
     def test_label(self) -> None:
         """Test get_label."""
-        task = TestTaskMixin()
+        task = TestBaseTask1({})
         self.assertEqual(task.get_label(), "test")
 
 
+class DefaultDynamicDataTest(TestCase):
+    """Test DefaultDynamicData."""
+
+    def test_dynamic_data(self) -> None:
+        """Test build_dynamic_data."""
+        task = TestDefaultDynamicData({})
+        self.assertEqual(
+            task.compute_dynamic_data(FakeTaskDatabase()), BaseDynamicTaskData()
+        )
+
+
 class ExtraRepositoryMixinTests(TestCase):
     """Unit tests for BaseTaskWithExecutor class."""
 
