@@ -11,20 +11,28 @@
 
 import contextlib
 import hashlib
+import io
 import os
+import re
 import shutil
 import tempfile
 import textwrap
 import unittest
+from collections import defaultdict
 from collections.abc import Generator
 from configparser import ConfigParser
+from datetime import datetime, timezone
 from pathlib import Path
-from typing import Any, TYPE_CHECKING
+from typing import Any, TYPE_CHECKING, cast
 from unittest import mock
 from unittest.util import safe_repr
 
 import debian.deb822 as deb822
+import lxml
+import lxml.etree
+import lxml.objectify
 import responses
+import yaml
 
 from debusine.artifacts.local_artifact import deb822dict_to_dict
 from debusine.artifacts.models import DebianSourcePackage
@@ -410,3 +418,247 @@ class TestCase(unittest.TestCase):
                 'Version': version,
             },
         )
+
+    # LXML does not seem to know about HTML5 structural tags
+    re_lxml_false_positive_tags = re.compile(
+        r"Tag (?:nav|footer|header|article) invalid"
+    )
+
+    def _filter_parser_error_log(
+        self, error_log: lxml.etree._ListErrorLog
+    ) -> list[str]:
+        """Filter lxml parser error log for known false positives."""
+        # _LogEntry documentation:
+        # https://lxml.de/apidoc/lxml.etree.html#lxml.etree._LogEntry
+        Domains = lxml.etree.ErrorDomains
+
+        errors: list[str] = []
+        for error in error_log:
+            match (error.domain, error.type_name):
+                case Domains.HTML, "HTML_UNKNOWN_TAG":
+                    if self.re_lxml_false_positive_tags.match(error.message):
+                        continue
+                    # Without this pass, python coverage is currently unable to
+                    # detect that code does flow through here
+                    pass
+
+            errors.append(f"{error.line}:{error.type_name}:{error.message}")
+        return errors
+
+    def dump_html(
+        self,
+        content: str,
+        *,
+        title: str | None = None,
+        parser: lxml.etree.HTMLParser | None = None,
+    ) -> None:
+        """
+        Dump HTML on standard output.
+
+        :param content: HTML string to dump
+        :param title: optional title text to write before the dump
+        :param parser: optional parser instance that parsed the HTML, to use to
+                       annotate code with parser errors
+        """
+        from rich.console import Console
+        from rich.syntax import Syntax
+
+        console = Console()
+        if title is not None:
+            console.print("")
+            console.print(
+                f"    {title}",
+                markup=False,
+                highlight=False,
+                style="bold bright_red",
+            )
+            console.print("")
+
+        errors_by_line: dict[int, list[str]] | None = None
+        if parser:
+            errors_by_line = defaultdict(list)
+            for e in parser.error_log:
+                errors_by_line[e.line].append(f"{e.type_name}:{e.message}")
+        else:
+            errors_by_line = {}
+
+        syntax = Syntax(content, "html", line_numbers=True)
+        text = syntax.highlight(content)
+        for lineno, line in enumerate(text.split(), start=1):
+            console.print(
+                f"{lineno:03d}", end=" ", style="bright_black", highlight=False
+            )
+            console.print(line)
+            if line_errors := errors_by_line.get(lineno):
+                for error in line_errors:
+                    console.print(
+                        f"  → {error}",
+                        style="red",
+                        markup=False,
+                        highlight=False,
+                    )
+
+    def dump_element(
+        self,
+        element: lxml.objectify.ObjectifiedElement,
+        *,
+        title: str | None = None,
+    ) -> None:
+        """
+        Dump a LXML element on standard output.
+
+        :param element: element to dump
+        :param title: optional title text to write before the dump
+        """
+        self.dump_html(lxml.etree.tostring(element).decode(), title=title)
+
+    def assertHTMLValid(
+        self,
+        content: str | bytes,
+        dump_on_error: bool = False,
+    ) -> lxml.objectify.ObjectifiedElement:
+        """
+        Parse the response contents as HTML and checks contents.
+
+        Assert:
+        - HTML parses without errors
+        - No "debusine-widget-errors" in the contents
+
+        Returns the parsed tree.
+        """
+        if isinstance(content, str):
+            content = content.encode()
+            parser = lxml.etree.HTMLParser(
+                remove_blank_text=True, encoding="utf8"
+            )
+        else:
+            parser = lxml.etree.HTMLParser(remove_blank_text=True)
+        parser.set_element_class_lookup(
+            lxml.objectify.ObjectifyElementClassLookup()
+        )
+        with io.BytesIO(content) as fd:
+            tree = lxml.etree.parse(fd, parser)
+        errors = self._filter_parser_error_log(parser.error_log)
+
+        # Dump source with errors if requested
+        if errors and dump_on_error:
+            self.dump_html(
+                content.decode(), parser=parser, title="HTML parser errors"
+            )
+
+        self.assertEqual(errors, [])
+
+        # Assert that no "debusine-widget-error" is displayed. It appears
+        # if a widget has an error
+        self.assertEqual(
+            tree.xpath('//*[@data-role="debusine-widget-error"]'), []
+        )
+
+        return cast(lxml.objectify.ObjectifiedElement, tree.getroot())
+
+    def assertHasElement(
+        self,
+        tree: lxml.objectify.ObjectifiedElement,
+        xpath: str,
+        dump_on_error: bool = False,
+    ) -> lxml.objectify.ObjectifiedElement:
+        """
+        Ensure that the tree has strictly one matching element.
+
+        :param tree: tree to search
+        :param xpath: XPath query to match
+        :returns: resulting element
+        """
+        el = tree.xpath(xpath)
+        if not el:
+            message = f"{xpath!r} not found in tree"
+            if dump_on_error:
+                self.dump_element(tree, title=message)
+            self.fail(message)
+        if len(el) > 1:
+            message = f"{xpath!r} matched {len(el)} elements instead of one"
+            if dump_on_error:
+                self.dump_element(tree, title=message)
+            self.fail(message)
+        assert isinstance(el[0], lxml.objectify.ObjectifiedElement)
+        return el[0]
+
+    def assertElementHasClass(
+        self, element: lxml.objectify.ObjectifiedElement, name: str
+    ) -> None:
+        """Ensure the element has the given class in the DOM."""
+        dom_class = element.get("class")
+        if dom_class is None:
+            self.fail("Element does not have a class attribute")
+        self.assertIn(name, dom_class.split())
+
+    def assertElementHasNoClass(
+        self, element: lxml.objectify.ObjectifiedElement, name: str
+    ) -> None:
+        """Ensure the element does not have the given class in the DOM."""
+        if (dom_class := element.get("class")) is None:
+            return
+        self.assertNotIn(name, dom_class.split())
+
+    def get_node_text_normalized(
+        self, node: lxml.objectify.ObjectifiedElement
+    ) -> str:
+        """Get the node text contents, with spaces normalized."""
+        # itertext will iterate on whitespace-only blocks, so it needs two
+        # passes: one to reconstruct the text, and one to normalise whitespace
+        sample = "".join(node.itertext())
+        return " ".join(sample.strip().split())
+
+    def assertTextContentEqual(
+        self,
+        node: lxml.objectify.ObjectifiedElement,
+        text: str,
+    ) -> None:
+        """
+        Ensure that node.text matches the given text.
+
+        Both expected and actual text are normalised so that consecutive
+        whitespace and newlines become a single space, to simplify dealing with
+        the way HTML collapses whitespace.
+        """
+        sample = self.get_node_text_normalized(node)
+        text = " ".join(text.strip().split())
+        self.assertEqual(sample, text)
+
+    def assertDatetimeContentEqual(
+        self,
+        node: lxml.objectify.ObjectifiedElement,
+        dt: datetime,
+        anywhere: bool = False,
+    ) -> None:
+        """
+        Ensure the row contains a datetime matching the given one.
+
+        Text surrounding the datetime value is ignored.
+        """
+        text = self.get_node_text_normalized(node)
+        regex = r"\d+-\d+-\d+ \d+:\d+"
+        if not anywhere:
+            regex = f"^{regex}$"
+
+        if not (m := re.search(regex, text)):
+            self.fail(
+                f"text content {text!r} does not contain"
+                " a YYYY-MM-DD HH:MM datetime"
+            )
+        actual = datetime.strptime(m.group(), "%Y-%m-%d %H:%M").replace(
+            tzinfo=timezone.utc
+        )
+        expected = dt.replace(second=0, microsecond=0)
+        self.assertEqual(actual, expected)
+
+    def assertYAMLContentEqual(
+        self, node: lxml.objectify.ObjectifiedElement, data: Any
+    ) -> None:
+        """Ensure that the node contents match yaml-formatted data."""
+        encoded = "".join(node.itertext())
+        try:
+            actual = yaml.safe_load(encoded)
+        except yaml.parser.ParserError as e:
+            self.fail(f"node text {encoded!r} does not parse as YAML: {e}")
+        self.assertEqual(actual, data)
