@@ -34,6 +34,7 @@ from unittest import mock
 from unittest.util import safe_repr
 
 import django.test
+import lxml
 import requests
 from asgiref.sync import async_to_sync
 from channels.layers import get_channel_layer
@@ -41,6 +42,7 @@ from django.conf import settings
 from django.db import transaction
 from django.db.models import Model, QuerySet
 from django.http.response import HttpResponseBase
+from rest_framework import status
 
 from debusine.db.context import ContextConsistencyError, context
 from debusine.db.models import Scope, Token, User, WorkRequest, Workspace
@@ -263,6 +265,24 @@ class BaseDjangoTestCase(django.test.SimpleTestCase, BaseTestCase):
         yield sid
         transaction.savepoint_rollback(sid)
 
+    def assertResponseHTML(
+        self,
+        response: TestResponseType,
+        status_code: int = status.HTTP_200_OK,
+        content_type: str = "text/html; charset=utf-8",
+    ) -> lxml.objectify.ObjectifiedElement:
+        """
+        Check that the response is valid HTML.
+
+        :param response: the Django response to check
+        :param status_code: expected status code
+        :param content_type: expected content type
+        :returns: the parsed HTML contents
+        """
+        self.assertEqual(response.status_code, status_code)
+        self.assertEqual(response.headers["Content-Type"], content_type)
+        return self.assertHTMLValid(response.content)
+
     def assertResponse400(
         self, response: "_MonkeyPatchedWSGIResponse", error: str
     ) -> None:
@@ -553,19 +573,19 @@ class PlaygroundTestCase(BaseDjangoTestCase):
     def _build_permission_when_role_testlist(
         self,
         predicate: Callable[[PermissionUser], bool],
-        roles: Roles | str | Collection[Roles | str] = (),
+        roles: str | Collection[str] = (),
         *,
-        scope_roles: Roles | str | Collection[Roles | str] = (),
-        workspace_roles: Roles | str | Collection[Roles | str] = (),
-    ) -> list[tuple[Model, Roles]]:
+        scope_roles: str | Collection[str] = (),
+        workspace_roles: str | Collection[str] = (),
+    ) -> list[tuple[Model, str]]:
         """Build a TODO list for testing the predicate with the given roles."""
         assert isinstance(predicate, PermissionCheckPredicate)
         resource = predicate.__self__
-        roles = resolve_roles_list(getattr(resource, "Roles", None), roles)
-        scope_roles = resolve_roles_list(Scope.Roles, scope_roles)
-        workspace_roles = resolve_roles_list(Workspace.Roles, workspace_roles)
+        roles = resolve_roles_list(resource, roles)
+        scope_roles = resolve_roles_list(Scope, scope_roles)
+        workspace_roles = resolve_roles_list(Workspace, workspace_roles)
 
-        tests: list[tuple[Model, Roles]] = []
+        tests: list[tuple[Model, str]] = []
         if scope_roles:
             scope = get_resource_scope(resource)
             assert scope is not None
@@ -582,10 +602,10 @@ class PlaygroundTestCase(BaseDjangoTestCase):
         self,
         predicate: Callable[[PermissionUser], bool],
         user: User,
-        roles: Roles | str | Collection[Roles | str] = (),
+        roles: str | Collection[str] = (),
         *,
-        scope_roles: Roles | str | Collection[Roles | str] = (),
-        workspace_roles: Roles | str | Collection[Roles | str] = (),
+        scope_roles: str | Collection[str] = (),
+        workspace_roles: str | Collection[str] = (),
     ) -> None:
         """
         Check that assigning roles does not make the predicate come true.
@@ -651,9 +671,9 @@ class PlaygroundTestCase(BaseDjangoTestCase):
         self,
         predicate: Callable[[PermissionUser], bool],
         user: User,
-        roles: Roles | str | Collection[Roles | str] = (),
-        scope_roles: Roles | str | Collection[Roles | str] = (),
-        workspace_roles: Roles | str | Collection[Roles | str] = (),
+        roles: str | Collection[str] = (),
+        scope_roles: str | Collection[str] = (),
+        workspace_roles: str | Collection[str] = (),
     ) -> None:
         """
         Check that assigning roles makes the predicate come true.
