@@ -1,4 +1,4 @@
-# Copyright 2024 The Debusine Developers
+# Copyright © The Debusine Developers
 # See the AUTHORS file at the top-level directory of this distribution
 #
 # This file is part of Debusine. It is subject to the license terms
@@ -15,7 +15,11 @@ import debusine.db.playground
 from debusine.db.context import context
 from debusine.db.models import Artifact, FileInArtifact, FileStore, Workspace
 from debusine.db.playground import scenarios
-from debusine.test.django import PlaygroundMixin, TestCase, TransactionTestCase
+from debusine.test.django import (
+    PlaygroundTestCase,
+    TestCase,
+    TransactionTestCase,
+)
 from debusine.test.playground import Playground
 
 
@@ -30,7 +34,7 @@ class IntScenario(scenarios.Scenario):
         self.value = 0
 
 
-class ScenarioTestMixins(PlaygroundMixin):
+class ScenarioTestCase(PlaygroundTestCase):
     """Common scenario-related tests."""
 
     scenario = scenarios.DefaultContext()
@@ -56,7 +60,7 @@ class ScenarioTestMixins(PlaygroundMixin):
         self.int_scenario.value = 1
 
 
-class MemoryPlaygroundTestCaseTest(ScenarioTestMixins, TestCase):
+class MemoryPlaygroundTestCaseTest(ScenarioTestCase, TestCase):
     """Test instantiation of playground in normal test case."""
 
     def test_instantiated(self) -> None:
@@ -67,7 +71,7 @@ class MemoryPlaygroundTestCaseTest(ScenarioTestMixins, TestCase):
         )
 
 
-class LocalPlaygroundTestCaseTest(ScenarioTestMixins, TestCase):
+class LocalPlaygroundTestCaseTest(ScenarioTestCase, TestCase):
     """Test instantiation of playground in normal test case."""
 
     playground_memory_file_store = False
@@ -98,7 +102,7 @@ class NoPlaygroundTestCaseTest(TestCase):
 
 
 class MemoryPlaygroundTransactionTestCaseTest(
-    ScenarioTestMixins, TransactionTestCase
+    ScenarioTestCase, TransactionTestCase
 ):
     """Test instantiation of playground in normal test case."""
 
@@ -113,7 +117,7 @@ class MemoryPlaygroundTransactionTestCaseTest(
 
 
 class LocalPlaygroundTransactionTestCaseTest(
-    ScenarioTestMixins, TransactionTestCase
+    ScenarioTestCase, TransactionTestCase
 ):
     """Test instantiation of playground in normal test case."""
 
@@ -232,7 +236,7 @@ class PlaygroundTransactionalTestMixin(TestCase):
         self.assertEqual(len(files1), 1)
         self.assertEqual(len(files2), 2)
         self.assertEqual(Artifact.objects.count(), 2)
-        backend = self.workspace.default_file_store.get_backend_object()
+        backend = self.workspace.scope.download_file_backend(files1[0])
         with backend.get_stream(files1[0]) as fd:
             self.assertEqual(fd.read(), b"testdeb")
         with backend.get_stream(files2[0]) as fd:
@@ -282,5 +286,5 @@ class PlaygroundTransactionalTestFileStore(PlaygroundTransactionalTestMixin):
 
 
 # Avoid running tests from the common base
-del ScenarioTestMixins
+del ScenarioTestCase
 del PlaygroundTransactionalTestMixin
