@@ -101,7 +101,7 @@ class ParsedContentRange(TypedDict):
 
 
 def parse_content_range_header(
-    headers: Mapping[str, str]
+    headers: Mapping[str, str],
 ) -> ParsedContentRange | None:
     """Parse headers["Content-Range"]. Return dictionary with information."""
     header_name = "Content-Range"
