@@ -59,7 +59,8 @@ def widget(context: Context, widget: Widget) -> str:
         # report to locate the stack trace in the logs
         logger.warning("widget %r failed to render", widget, exc_info=True)
         return format_html(
-            "<span class='bg-danger text-white'>{ts} UTC: {widget}"
+            "<span data-role='debusine-widget-error'"
+            " class='bg-danger text-white'>{ts} UTC: {widget}"
             " failed to render</span>",
             widget=widget.__class__.__name__,
             ts=datetime.utcnow().isoformat(),
@@ -135,3 +136,9 @@ def format_yaml(value: Any) -> str:
 def icon(name: str) -> str:
     """Lookup an icon by name."""
     return "bi-" + getattr(Icons, name.upper(), "square-fill")
+
+
+@register.filter(name="sorted")
+def sorted_(value: Any) -> Any:
+    """Sort a sequence of values."""
+    return sorted(value)
