@@ -51,9 +51,11 @@ def workflow_runtime_status_small(workflow: WorkRequest) -> str:
         WorkRequest.RuntimeStatuses.BLOCKED: ("dark", "B"),
     }
 
-    if workflow.status == WorkRequest.Statuses.RUNNING:
+    if (
+        workflow.status == WorkRequest.Statuses.RUNNING
+        and workflow.workflow_runtime_status is not None
+    ):
         # Return badge with the workflow.workflow_runtime_status only
-        assert workflow.workflow_runtime_status
         workflow_runtime_status = WorkRequest.RuntimeStatuses(
             workflow.workflow_runtime_status
         )
