@@ -16,7 +16,6 @@ from typing import Any, ClassVar
 from django import forms
 from django.contrib.auth import get_user_model
 from django.core.files.uploadedfile import SimpleUploadedFile
-from django.http import QueryDict
 from django.utils.datastructures import MultiValueDict
 
 try:
@@ -29,14 +28,7 @@ import yaml
 from debusine.artifacts import PackageBuildLog
 from debusine.artifacts.models import ArtifactCategory, DebianPackageBuildLog
 from debusine.db.context import context
-from debusine.db.models import (
-    Artifact,
-    Group,
-    Token,
-    User,
-    WorkRequest,
-    Workspace,
-)
+from debusine.db.models import Artifact, Group, Token, User, Workspace
 from debusine.db.playground import scenarios
 from debusine.tasks import BaseTask
 from debusine.test.django import TestCase
@@ -77,6 +69,9 @@ class BootstrapMixinForm(BootstrapMixin, forms.Form):
     date_field = forms.DateField(
         widget=forms.DateInput(attrs={"class": existing_class})
     )
+    boolean_field = forms.BooleanField(
+        widget=forms.CheckboxInput(attrs={"class": existing_class})
+    )
 
 
 class BootstrapMixinTests(TestCase):
@@ -154,6 +149,16 @@ class BootstrapMixinTests(TestCase):
             date_widget_class, f"{BootstrapMixinForm.existing_class}"
         )
 
+    def test_booleanfield_bootstrap_class(self) -> None:
+        """Test BooleanField has form-check-input."""
+        boolean_widget_class = self.form.fields[
+            "boolean_field"
+        ].widget.attrs.get("class")
+        self.assertEqual(
+            boolean_widget_class,
+            f"{BootstrapMixinForm.existing_class} form-check-input",
+        )
+
     def test_asterisk_set_if_required(self) -> None:
         """Test asterisk is set if field is required."""
         char_field_required = self.form.fields["char_field_required"]
@@ -792,18 +797,28 @@ class WorkflowFilterFormTests(TestCase):
 
     def test_statuses_field(self) -> None:
         """Test statuses field."""
-        choices = sorted(WorkRequest.Statuses.choices)
-        choices.remove(("running", "Running"))
-
-        choices.extend(
+        self.assertEqual(
+            self.form.fields["statuses"].widget.choices,
             [
-                (f"running__{value}", f"running__{label}")
-                for value, label in WorkRequest.RuntimeStatuses.choices
-            ]
+                ('pending', 'Pending'),
+                (
+                    'Running',
+                    [
+                        ('running__needs_input', 'Needs Input'),
+                        ('running__running', 'Running'),
+                        ('running__waiting', 'Waiting'),
+                        ('running__pending', 'Pending'),
+                        ('running__aborted', 'Aborted'),
+                        ('running__completed', 'Completed'),
+                        ('running__blocked', 'Blocked'),
+                        ('running__any', 'Any'),
+                    ],
+                ),
+                ('completed', 'Completed'),
+                ('aborted', 'Aborted'),
+                ('blocked', 'Blocked'),
+            ],
         )
-        choices.extend([("running__any", "running__Any")])
-
-        self.assertEqual(self.form.fields["statuses"].widget.choices, choices)
 
     def test_runtime_statuses_field(self) -> None:
         """Test runtime_statuses field."""
@@ -866,33 +881,6 @@ class WorkflowFilterFormTests(TestCase):
         """Test with_failed_work_requests field."""
         self.assertIn("with_failed_work_requests", self.form.fields)
 
-    def test_statuses_runtime_statuses(self) -> None:
-        """Test statuses_runtime_statuses() generated HTML."""
-        self.form.data = QueryDict(
-            "statuses=pending&statuses=running__needs_input"
-        )
-        self.assertHTMLEqual(
-            self.form.statuses_runtime_statuses(),
-            """
-            <select name="statuses" id="id_statuses" multiple>
-                <option value="pending" selected>Pending</option>
-                <optgroup label="Running"><option value="running__needs_input"
-                        selected>Needs Input</option>
-                    <option value="running__running">Running</option>
-                    <option value="running__waiting">Waiting</option>
-                    <option value="running__pending">Pending</option>
-                    <option value="running__aborted">Aborted</option>
-                    <option value="running__completed">Completed</option>
-                    <option value="running__blocked">Blocked</option>
-                    <option value="running__any">Any</option>
-                </optgroup>
-            <option value="completed">Completed</option>
-            <option value="aborted">Aborted</option>
-            <option value="blocked">Blocked</option>
-            </select>
-            """,
-        )
-
 
 class DaysFieldTest(TestCase):
     """Tests for DaysField."""
