@@ -7,11 +7,19 @@
 # modified, propagated, or distributed except according to the terms
 # contained in the LICENSE file.
 """Django pages URL's for Debusine."""
-from django.urls import include, path
+from django.urls import URLPattern, URLResolver, include, path
 from django.views.generic import RedirectView
 
+service_urlpatterns: list[URLPattern | URLResolver] = [
+    path(
+        "groups/",
+        include("debusine.web.urls.groups", namespace="groups"),
+    ),
+]
+
 # Scoped URLs
 urlpatterns = [
+    path("-/", include(service_urlpatterns)),
     # TODO Preserved for compatibility redirects only
     path(
         "accounts/oidc_callback/<name>/",
