@@ -50,7 +50,7 @@ workspace_urlpatterns: list[URLPattern | URLResolver] = [
     path(
         "artifact/",
         include(
-            "debusine.web.urls.workspace_artifacts",
+            "debusine.web.urls.artifacts",
             namespace="artifacts",
         ),
     ),
