@@ -151,9 +151,21 @@ class ArtifactDetailView(
             )
         return file_list
 
+    def _current_view_is_specialized(self) -> bool:
+        """
+        Specialized (based on a plugin) view will be served.
+
+        User did not force the default view, a plugin exists, and the work
+        request passes validation.
+        """
+        return ArtifactPlugin.plugin_for(self.artifact.category) is not None
+
     def get_context_data(self, **kwargs: Any) -> dict[str, Any]:
         """Return context for this view."""
         ctx = super().get_context_data(**kwargs)
+
+        plugin_view = ArtifactPlugin.plugin_for(self.artifact.category)
+
         for file_in_artifact in self.file_list:
             self.add_object_ui_shortcuts(
                 file_in_artifact,
@@ -165,6 +177,7 @@ class ArtifactDetailView(
         ctx["artifact_label"] = self.object.get_label()
         ctx["file_list"] = self.file_list
         ctx["artifact_data"] = format_yaml(self.object.data)
+
         ctx["forward"] = (
             Artifact.objects.filter(targeted_by__artifact=self.object)
             .annotate(relation_type=F("targeted_by__type"))
@@ -176,12 +189,35 @@ class ArtifactDetailView(
             .filter(relation_type=ArtifactRelation.Relations.EXTENDS)
             .annotate_complete()
         )
+        ctx["relation_count"] = (
+            ctx["forward"].count() + ctx["reverse_extends"].count()
+        )
 
         if len(self.file_list) == 1:
-            ctx["file"] = FileWidget.create(self.file_list[0])
+            # If there is only one file: display it inline. Otherwise the
+            # template will list the files
+            ctx["file"] = FileWidget.create(self.file_list[0], file_tag="files")
+
+        ctx["file_count"] = len(self.file_list)
+
+        if plugin_view is not None:
+            # Add specific specialized_plugin information
+            ctx.update(
+                **plugin_view(self.artifact).get_context_data(),
+                **{"specialized_view": True},
+            )
 
         return ctx
 
+    def get_template_names(self) -> list[str]:
+        """Return the plugin's template_name or the default one."""
+        if self._current_view_is_specialized():
+            plugin_class = ArtifactPlugin.plugin_for(self.artifact.category)
+            assert plugin_class is not None
+            return [plugin_class.template_name]
+
+        return [self.template_name]
+
 
 class FileView(
     PathMixin,
@@ -442,3 +478,40 @@ class CreateArtifactView(
             return super().form_invalid(form)
         finally:
             form.cleanup()
+
+
+class ArtifactPlugin(abc.ABC):
+    """
+    Artifacts with specific outputs must subclass it.
+
+    When subclassing, the subclass:
+    - Is automatically used by the /artifact/ID/ endpoint
+    - Must define "artifact_category" and "template_name"
+    - Must implement "get_context_data()"
+    """
+
+    model = Artifact
+    artifact_category: ArtifactCategory
+    template_name: str
+    name: str
+
+    _artifact_plugins: dict[str, type["ArtifactPlugin"]] = {}
+
+    def __init_subclass__(cls, **kwargs: Any) -> None:  # noqa: U100
+        """Register the plugin."""
+        cls._artifact_plugins[cls.artifact_category] = cls
+
+    def __init__(self, artifact: Artifact) -> None:
+        """Store the artifact to display."""
+        self.artifact = artifact
+
+    @classmethod
+    def plugin_for(
+        cls, artifact_category: str
+    ) -> type["ArtifactPlugin"] | None:
+        """Return ArtifactPlugin for artifact_category or None."""
+        return cls._artifact_plugins.get(artifact_category)
+
+    @abc.abstractmethod
+    def get_context_data(self) -> dict[str, Any]:
+        """Return context data for rendering the artifact view."""
