@@ -17,7 +17,6 @@ from django.contrib.auth import views as auth_views
 from django.db.models import QuerySet
 from django.http import HttpRequest, HttpResponse, HttpResponseRedirect
 from django.shortcuts import get_object_or_404
-from django.template.response import TemplateResponse
 from django.urls import reverse
 
 from debusine.db.context import context
@@ -43,17 +42,19 @@ from debusine.web.views.base import (
     SingleObjectMixinBase,
     UpdateViewBase,
 )
-from debusine.web.views.pagination import Ordering, Pagination
+from debusine.web.views.table import TableMixin
+from debusine.web.views.tables import (
+    GroupAuditLogTable,
+    GroupMembershipAdminTable,
+    GroupMembershipTable,
+)
 
 
 class LoginView(BaseUIView, auth_views.LoginView):
     """Class for the login view."""
 
     template_name = "account/login.html"
-
-    def get_title(self) -> str:
-        """Return the page title."""
-        return "Log in"
+    title = "Log in"
 
     def get(
         self, request: HttpRequest, *args: Any, **kwargs: Any
@@ -77,10 +78,7 @@ class LogoutView(
     """Class for the logout view."""
 
     template_name = "account/logged_out.html"
-
-    def get_title(self) -> str:
-        """Return the page title."""
-        return "Logged out"
+    title = "Logged out"
 
     def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
         """Tell the base template we are a logout view."""
@@ -98,17 +96,19 @@ class UserDetailView(BaseUIView, DetailViewBase[User]):
     # current user
     context_object_name = "person"
 
+    def get_title(self) -> str:
+        """Return the page title."""
+        return self.object.get_full_name()
+
     def get_queryset(self) -> QuerySet[User]:
         """All workspaces for authenticated users or only public ones."""
         queryset = cast(UserQuerySet[Any], super().get_queryset())
         return queryset.can_display(context.user)
 
-    def get_object(
-        self, queryset: QuerySet[User] | None = None  # noqa: U100
-    ) -> User:
+    def get_object(self, queryset: QuerySet[User] | None = None) -> User:
         """Return the collection object to show."""
-        if queryset is None:
-            queryset = self.get_queryset()  # pragma: no cover
+        assert queryset is None
+        queryset = self.get_queryset()
         return get_object_or_404(queryset, username=self.kwargs["username"])
 
     def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
@@ -176,6 +176,7 @@ class UserTokenListView(TokenView, ListViewBase[Token]):
     template_name = "web/user_token-list.html"
     context_object_name = "token_list"
     ordering = "name"
+    title = "API tokens"
 
     def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
         """Add bits needed by the template."""
@@ -216,17 +217,51 @@ class UserTokenCreateView(TokenFormView, CreateViewBase[Token, TokenForm]):
 
     template_name = "web/user_token-form.html"
     action = "Create"
+    context_object_name = "token"
+
+    def get_form_kwargs(self) -> dict[str, Any]:
+        """Extend the default kwarg arguments: add "user"."""
+        kwargs = super().get_form_kwargs()
+        kwargs["user"] = self.user
+        return kwargs
+
+    def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
+        """Add the user object from URL kwargs."""
+        ctx = super().get_context_data(**kwargs)
+        ctx["person"] = self.user
+        return ctx
+
+    def get_title(self) -> str:
+        """Switch title if we have a newly created token."""
+        if self.object:
+            return "Token created"
+        else:
+            return super().get_title()
+
+    def get_template_names(self) -> list[str]:
+        """Switch template if we have a newly created token."""
+        if self.object:
+            return ["web/user_token-created.html"]
+        else:
+            return [self.template_name]
 
     def form_valid(self, form: TokenForm) -> HttpResponse:
         """Validate form and return token created page."""
-        form.instance.user = self.user
-        token = form.save()
-
-        return TemplateResponse(
-            self.request,
-            "web/user_token-created.html",
-            {"token": token, "person": self.user},
-        )
+        # FIXME: a successful form submission should not render a page and
+        # should redirect to another page instead, to avoid unintended
+        # behaviour if the user triggers a reload. See:
+        # https://developer.mozilla.org/en-US/docs/Web/HTTP/Guides/Redirections#temporary_responses_to_unsafe_requests # noqa: E501
+        #
+        # However, here we lose the token key after we send a response, so to
+        # avoid rendering a page to show it here, we need to store it somewhere
+        # the target of the redirect can find it.
+        #
+        # That would introduce further issues in managing the security and the
+        # lifetime of that storage, and the worst that can happen with a reload
+        # here is the generation of another token, which seems like the lesser
+        # risk for now.
+        self.object = form.save()
+        return self.render_to_response(self.get_context_data(**self.kwargs))
 
 
 class UserTokenUpdateView(TokenFormView, UpdateViewBase[Token, TokenForm]):
@@ -311,12 +346,11 @@ class GroupDetailView(
         return f"Group {self.object}"
 
     def get_object(
-        self,
-        queryset: QuerySet[Group, Group] | None = None,
+        self, queryset: QuerySet[Group, Group] | None = None
     ) -> Group:
         """Get the Group to display."""
-        if queryset is None:
-            queryset = self.get_queryset()  # pragma: no cover
+        assert queryset is None
+        queryset = self.get_queryset()
         group = get_object_or_404(
             cast(GroupQuerySet[Any], queryset).can_display(
                 context.require_user()
@@ -335,35 +369,26 @@ class GroupDetailView(
     def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
         """Add bits needed by the template."""
         assert context.user and context.user.is_authenticated
-        pagination = Pagination(
-            self.request,
-            GroupMembership.objects.filter(group=self.object).select_related(
-                "user"
-            ),
-            default_ordering=("role", "user__username"),
-        )
-        pagination.add_column(
-            "name",
-            "Name",
-            Ordering(
-                asc=["user__username", "role"], desc=["-user__username", "role"]
-            ),
-        )
-        pagination.add_column(
-            "role",
-            "Role",
-            Ordering(
-                asc=["role", "user__username"], desc=["-role", "user__username"]
-            ),
-        )
-
         ctx = super().get_context_data(
-            pagination=pagination,
             asc=self.request.GET.get("asc", "1"),
             order=self.request.GET.get("order"),
             **kwargs,
         )
-        ctx["can_manage"] = self.object.can_manage(context.require_user())
+
+        ctx["can_manage"] = can_manage = self.object.can_manage(
+            context.require_user()
+        )
+
+        table_class = (
+            GroupMembershipAdminTable if can_manage else GroupMembershipTable
+        )
+        ctx["members"] = table_class(
+            request=self.request,
+            object_list=GroupMembership.objects.filter(
+                group=self.object
+            ).select_related("user"),
+        ).get_paginator(per_page=10)
+
         audit_log = self.object.audit_log.order_by(
             "-created_at"
         ).select_related("actor")[:6]
@@ -375,6 +400,7 @@ class GroupDetailView(
             ).role
         except GroupMembership.DoesNotExist:
             pass
+
         return ctx
 
     # Adapted from BaseDeleteView, since ProcessFormView does not set
@@ -407,12 +433,11 @@ class MembershipView(SingleObjectMixinBase[GroupMembership], BaseUIView):
     """Base view for group management for a user."""
 
     def get_object(
-        self,
-        queryset: QuerySet[GroupMembership, GroupMembership] | None = None,
+        self, queryset: QuerySet[GroupMembership, GroupMembership] | None = None
     ) -> GroupMembership:
         """Get the GroupMembership to display."""
-        if queryset is None:
-            queryset = self.get_queryset()  # pragma: no cover
+        assert queryset is None
+        queryset = self.get_queryset()
         membership = get_object_or_404(
             queryset.filter(
                 group__in=Group.objects.can_manage(context.require_user()),
@@ -486,12 +511,16 @@ class MembershipDeleteView(
         return self.object.group.get_absolute_url()
 
 
-class GroupAuditLogView(ListViewBase[GroupAuditLog], BaseUIView):
+class GroupAuditLogView(
+    TableMixin[GroupAuditLog], ListViewBase[GroupAuditLog], BaseUIView
+):
     """View for showing the full group audit log."""
 
     model = GroupAuditLog
     template_name = "web/group_audit_log-list.html"
     context_object_name = "audit_log"
+    table_class = GroupAuditLogTable
+    paginate_by = 10
 
     @cached_property
     def group(self) -> Group:
@@ -516,25 +545,3 @@ class GroupAuditLogView(ListViewBase[GroupAuditLog], BaseUIView):
     def get_title(self) -> str:
         """Return the page title."""
         return f"Audit log for {self.group}"
-
-    def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
-        """Add bits needed by the template."""
-        ctx = super().get_context_data(**kwargs)
-        pagination = Pagination(
-            self.request, self.get_queryset(), default_ordering="-created_at"
-        )
-        pagination.add_column(
-            "date",
-            "Date",
-            Ordering("created_at"),
-        )
-        pagination.add_column(
-            "actor",
-            "User",
-            Ordering(
-                asc=["actor__username", "-created_at"],
-                desc=["-actor__username", "-created_at"],
-            ),
-        )
-        ctx["pagination"] = pagination
-        return ctx
