@@ -26,8 +26,9 @@ from debusine.web.views.base import (
     WorkspaceView,
 )
 from debusine.web.views.base_rightbar import RightbarUIView
-from debusine.web.views.view_utils import PaginationMixin, format_yaml
-from debusine.web.web_utils import ui_prototype
+from debusine.web.views.table import TableMixin
+from debusine.web.views.tables import CollectionItemTable
+from debusine.web.views.view_utils import format_yaml
 
 
 class CollectionListView(WorkspaceView, ListViewBase[Collection]):
@@ -38,6 +39,10 @@ class CollectionListView(WorkspaceView, ListViewBase[Collection]):
     context_object_name = "collection_list"
     ordering = ["category", "name"]
 
+    def get_title(self) -> str:
+        """Return the page title."""
+        return f"{context.require_workspace().name} collections"
+
     def get_queryset(self) -> QuerySet[Collection]:
         """Filter collection by accessible workspace."""
         queryset = super().get_queryset()
@@ -46,7 +51,6 @@ class CollectionListView(WorkspaceView, ListViewBase[Collection]):
         )
 
 
-@ui_prototype
 class CollectionCategoryListView(WorkspaceView, ListViewBase[Collection]):
     """List collections with a given category."""
 
@@ -55,6 +59,13 @@ class CollectionCategoryListView(WorkspaceView, ListViewBase[Collection]):
     context_object_name = "collection_list"
     ordering = ["name"]
 
+    def get_title(self) -> str:
+        """Return the page title."""
+        return (
+            f"{context.require_workspace().name}"
+            f" {self.kwargs['ccat']} collections"
+        )
+
     def get_queryset(self) -> QuerySet[Collection]:
         """Filter collection by accessible workspace."""
         queryset = super().get_queryset()
@@ -117,7 +128,6 @@ class CollectionDetailView(CollectionViewMixin, DetailViewBase[Collection]):
         """Return the collection object to show."""
         return self.collection
 
-    @ui_prototype
     def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
         """Return context_data with work_request_list and workspace_list."""
         context = super().get_context_data(**kwargs)
@@ -152,9 +162,9 @@ class CollectionDetailView(CollectionViewMixin, DetailViewBase[Collection]):
 
 
 class CollectionSearchView(
+    TableMixin[CollectionItem],
     CollectionViewMixin,
     FormMixinBase[CollectionSearchForm],
-    PaginationMixin,
     ListViewBase[CollectionItem],
 ):
     """Search a collection contents."""
@@ -162,12 +172,12 @@ class CollectionSearchView(
     model = CollectionItem
     template_name = "web/collection-search.html"
     context_object_name = "item_list"
-    paginate_by = 50
     form_class = CollectionSearchForm
+    table_class = CollectionItemTable
+    paginate_by = 50
 
-    @ui_prototype
     def get_queryset(self) -> QuerySet[CollectionItem]:
-        """All items belonging to the current collection."""
+        """Filter items based on the search form."""
         queryset = (
             super().get_queryset().filter(parent_collection=self.collection)
         )
@@ -176,6 +186,7 @@ class CollectionSearchView(
             "parent_collection__workspace",
             "collection",
             "artifact",
+            "artifact__workspace",
         )
         # FIXME: this builds the form twice: if it becomes a problem we can
         # override get_form to cache its result
@@ -189,9 +200,10 @@ class CollectionSearchView(
                 queryset = queryset.filter(removed_at__isnull=False)
             else:
                 queryset = queryset.filter(removed_at__isnull=True)
+        else:
+            queryset = queryset.none()
         return queryset
 
-    @ui_prototype
     def get_form_kwargs(self) -> dict[str, Any]:
         """Get arguments used to instantiate the form."""
         kwargs = super().get_form_kwargs()
@@ -199,27 +211,17 @@ class CollectionSearchView(
         kwargs["data"] = self.request.GET
         return kwargs
 
-    @ui_prototype
-    def get_ordering(self) -> str:
-        """Return field used for sorting."""
-        order = self.request.GET.get("order")
-        if order in ("name", "category", "created_at"):
-            if self.request.GET["asc"] == "0":
-                return "-" + order
-            else:
-                return order
-        return "category"
-
-    @ui_prototype
     def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
         """Return context_data with work_request_list and workspace_list."""
         context = super().get_context_data(**kwargs)
         context["collection"] = self.collection
-        context["order"] = self.get_ordering().removeprefix("-")
-        context["asc"] = self.request.GET.get("asc", "0")
-        items = list(context["object_list"])
-        context["items"] = items
-        for item in items:
+        # TODO: we need to match model objects exactly in order to be able to
+        # lookup UI shortcuts, and this is a hack to do that, and also avoid to
+        # iterate queryset twice.
+        # See !1540 for a possible way to avoid this hack
+        page_obj = context["paginator"].page_obj
+        page_obj.object_list = list(page_obj.object_list)
+        for item in page_obj.object_list:
             self.add_object_ui_shortcuts(
                 item, ui_shortcuts.create_collection_item(item)
             )
@@ -229,12 +231,9 @@ class CollectionSearchView(
                     ui_shortcuts.create_artifact_view(item.artifact),
                     ui_shortcuts.create_artifact_download(item.artifact),
                 )
-            else:
-                pass
         return context
 
 
-@ui_prototype
 class CollectionItemDetailView(
     CollectionViewMixin, DetailViewBase[CollectionItem]
 ):
@@ -265,8 +264,8 @@ class CollectionItemDetailView(
         self, queryset: QuerySet[CollectionItem] | None = None
     ) -> CollectionItem:
         """Return the collection object to show."""
-        if queryset is None:
-            queryset = self.get_queryset()  # pragma: no cover
+        assert queryset is None
+        queryset = self.get_queryset()
         try:
             return queryset.filter(removed_at__isnull=True).get(
                 pk=self.kwargs["iid"], parent_collection=self.collection
@@ -278,7 +277,6 @@ class CollectionItemDetailView(
                 f"{self.kwargs['cname']}@{self.kwargs['ccat']}"
             )
 
-    @ui_prototype
     def get_context_data(self, *args: Any, **kwargs: Any) -> dict[str, Any]:
         """Return context_data."""
         context = super().get_context_data(**kwargs)
