@@ -42,7 +42,7 @@ class LintianAnalysisTag(BaseArtifactDataModel):
 class LintianView(WorkRequestPlugin):
     """View for Lintian work request."""
 
-    template_name = "web/lintian-detail.html"
+    template_name = "web/lintian-work_request-detail.html"
     task_type = TaskTypes.WORKER
     task_name = "lintian"
 
