@@ -48,6 +48,7 @@ from debusine.test.django import (
     override_permission,
 )
 from debusine.web.views import ui_shortcuts
+from debusine.web.views.artifacts import ArtifactDetailView, FileView
 from debusine.web.views.tests.utils import ViewTestMixin, date_format
 
 
@@ -209,23 +210,35 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
             self.assertEqual(sidebar[5].value, "never")
 
     def assertFile(
-        self, tree: lxml.objectify.ObjectifiedElement, artifact: Artifact
+        self,
+        tree: lxml.objectify.ObjectifiedElement,
+        artifact: Artifact,
+        *,
+        suffix: str | None = None,
     ) -> None:
         """Ensure that the single file in the artifact is shown."""
         file = artifact.fileinartifact_set.select_related("file").get()
-        file_contents = self.assertHasElement(
-            tree, "//div[@id='file-contents']"
+        div_id = (
+            "file-contents" if suffix is None else f"file-contents-{suffix}"
         )
+        file_contents = self.assertHasElement(tree, f"//div[@id='{div_id}']")
+
+        self.assertTextContentEqual(file_contents.div[0].div[0], file.path)
+        gadgets = file_contents.div[0].div[1]
 
         if file.complete:
-            self.assertTextContentEqual(file_contents.div[0], file.path)
+            self.assertEqual(gadgets.get("class"), "btn-group btn-group-sm")
+            self.assertEqual(
+                gadgets.a[0].get("href"), file.get_absolute_url_raw()
+            )
+            self.assertEqual(
+                gadgets.a[1].get("href"), file.get_absolute_url_download()
+            )
             card_body_class = file_contents.div[1].get("class")
             assert card_body_class is not None
             self.assertIn("card-body", card_body_class.split())
         else:
-            self.assertTextContentEqual(
-                file_contents.div[0], f"{file.path} (incomplete)"
-            )
+            self.assertTextContentEqual(gadgets, "incomplete")
             self.assertEqual(len(file_contents.div), 1)
 
     def assertFileList(
@@ -262,10 +275,33 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         )
         num_relations = len(relations) + len(targeted_by)
         tables = tree.xpath("//table[@id='relation-list']")
+
+        relation_button = self.assertHasElement(
+            tree, "//button[@id='nav-relations-tab']"
+        )
+
+        if num_relations == 1:
+            self.assertTextContentEqual(relation_button, "Relation (1)")
+        else:
+            self.assertTextContentEqual(
+                relation_button, f"Relations ({num_relations})"
+            )
+
+        button_classes = relation_button.get("class")
+        assert button_classes is not None
+
         if num_relations == 0:
+            self.assertIn("disabled", button_classes.split())
+
+            # No "relation-list" tables
             self.assertFalse(tables)
+
+            div = tree.xpath("//div[@id='nav-relations']")
+            self.assertTextContentEqual(div[0], "No relations.")
             return
 
+        self.assertNotIn("disabled", button_classes.split())
+
         table = tables[0]
         self.assertEqual(len(table.tbody.tr), num_relations)
         for tr, relation in zip(table.tbody.tr, relations):
@@ -310,6 +346,12 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
             "get_context_data",
         )
 
+    def test_get_title(self) -> None:
+        """Test get_title method."""
+        view = ArtifactDetailView()
+        view.object = self.artifact
+        self.assertEqual(view.get_title(), "Artifact debusine:test")
+
     def test_invalid_artifact_id(self) -> None:
         """Test viewing an artifact ID that does not exist."""
         artifact_id = Artifact.objects.aggregate(Max("id"))['id__max'] + 1
@@ -320,30 +362,33 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
             "No artifact found matching the query",
         )
 
-    def test_get_success_html_list(self) -> None:
+    def test_get_success_html_files_list(self) -> None:
         """View shows a list of files."""
         with context.disable_permission_checks():
             work_request = self.playground.create_work_request(task_name="noop")
             self.artifact.created_by_work_request = work_request
             self.artifact.save()
         response = self._get()
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertResponseHTML(response)
         self.assertArtifactSidebarMatches(response, self.artifact)
-
-        tree = self.assertHTMLValid(response)
         self.assertFileList(tree, self.artifact)
         self.assertRelations(tree, self.artifact)
         self.assertFalse(tree.xpath("//div[@id='file-contents']"))
 
-    def test_get_success_html_list_incomplete(self) -> None:
+        file_button = self.assertHasElement(
+            tree, "//button[@id='nav-files-tab']"
+        )
+        assert file_button is not None
+        self.assertTextContentEqual(file_button, "Files (2)")
+
+    def test_get_success_html_files_list_incomplete(self) -> None:
         """View shows incomplete files in a list differently."""
         self.artifact.fileinartifact_set.filter(path="AUTHORS").update(
             complete=False
         )
         response = self._get()
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertResponseHTML(response)
         self.assertArtifactSidebarMatches(response, self.artifact)
-        tree = self.assertHTMLValid(response)
         self.assertFileList(tree, self.artifact)
         self.assertRelations(tree, self.artifact)
         self.assertFalse(tree.xpath("//div[@id='file-contents']"))
@@ -353,35 +398,49 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         with context.disable_permission_checks():
             self.artifact.fileinartifact_set.filter(path="AUTHORS").delete()
         response = self._get()
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertResponseHTML(response)
         self.assertArtifactSidebarMatches(response, self.artifact)
-        tree = self.assertHTMLValid(response)
+        file_button = self.assertHasElement(
+            tree, "//button[@id='nav-files-tab']"
+        )
+        assert file_button is not None
+        self.assertTextContentEqual(file_button, "File (1)")
+
+        file_classes = file_button.get("class")
+        assert file_classes is not None
+        self.assertNotIn("disabled", file_classes.split())
         self.assertFalse(tree.xpath("//table[@id='file-list']"))
-        self.assertFile(tree, self.artifact)
+        self.assertFile(tree, self.artifact, suffix="files")
 
     def test_get_success_html_singlefile_incomplete(self) -> None:
         """View shows a single incomplete file in the artifact differently."""
         self.artifact.fileinartifact_set.filter(path="AUTHORS").delete()
         self.artifact.fileinartifact_set.update(complete=False)
         response = self._get()
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertResponseHTML(response)
         self.assertArtifactSidebarMatches(response, self.artifact)
-        tree = self.assertHTMLValid(response)
         self.assertFalse(tree.xpath("//table[@id='file-list']"))
-        self.assertFile(tree, self.artifact)
+        self.assertFile(tree, self.artifact, suffix="files")
 
     def test_get_success_html_empty_artifact(self) -> None:
         """Test HTML output if there are no files in the artifact."""
         with context.disable_permission_checks():
             artifact, _ = self.playground.create_artifact([])
         response = self._get(artifact.id)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertFalse(tree.xpath("//table[@id='file-list']"))
         self.assertFalse(tree.xpath("//div[@id='file-contents']"))
-        no_files = self.assertHasElement(tree, "//p[@id='no-files']")
-        self.assertTextContentEqual(
-            no_files, "The artifact does not have any files."
+
+        files_button = self.assertHasElement(
+            tree, "//button[@id='nav-files-tab']"
         )
+        self.assertTextContentEqual(files_button, "Files (0)")
+        files_classes = files_button.get("class")
+        assert files_classes is not None
+        self.assertIn("disabled", files_classes.split())
+
+        div = tree.xpath("//div[@id='nav-files']")
+        self.assertTextContentEqual(div[0], "No files.")
 
     def test_get_success_html_user_and_no_expiration(self) -> None:
         """Test HTML output with no user and expiration."""
@@ -401,7 +460,7 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
                 self.artifact, other_artifact
             )
         response = self._get()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertRelations(tree, self.artifact)
 
     def test_get_success_html_fwd_relations_incomplete(self) -> None:
@@ -411,7 +470,7 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         )
         self.playground.create_artifact_relation(self.artifact, other_artifact)
         response = self._get()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertRelations(tree, self.artifact)
 
     def test_get_success_html_rev_relations(self) -> None:
@@ -427,7 +486,7 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
                 relation_type=ArtifactRelation.Relations.EXTENDS,
             )
         response = self._get()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertRelations(tree, self.artifact)
 
     def test_get_success_html_rev_relations_incomplete(self) -> None:
@@ -442,7 +501,7 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
             relation_type=ArtifactRelation.Relations.EXTENDS,
         )
         response = self._get()
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertRelations(tree, self.artifact)
 
     def test_ui_shortcuts_source(self) -> None:
@@ -545,8 +604,42 @@ class ArtifactDetailViewTests(ViewTestMixin, PermissionTests):
         )
 
 
-class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
-    """Test ArtifactDetailFileView."""
+class FileViewTests(TestCase):
+    """Test :py:class:`FileView`."""
+
+    contents: ClassVar[dict[str, bytes]]
+    artifact: ClassVar[Artifact]
+    file: ClassVar[FileInArtifact]
+
+    @classmethod
+    def setUpTestData(cls) -> None:
+        """Set up the common test fixture."""
+        super().setUpTestData()
+        cls.contents = {
+            "file.md": b"Line 1\nLine 2\n",
+        }
+        cls.artifact, _ = cls.playground.create_artifact(
+            paths=cls.contents,
+            create_files=True,
+        )
+        cls.file = cls.artifact.fileinartifact_set.get(path="file.md")
+
+    def test_get_title(self) -> None:
+        """Test get_title method."""
+        view = FileView()
+        view.object = self.file
+        self.assertEqual(view.get_title(), self.file.path)
+
+    def test_get_title_incomplete(self) -> None:
+        """Test get_title method with an incomplete file."""
+        self.file.complete = False
+        view = FileView()
+        view.object = self.file
+        self.assertEqual(view.get_title(), self.file.path + " (incomplete)")
+
+
+class FileDetailViewTests(ViewTestMixin, PermissionTests):
+    """Test FileDetailView."""
 
     user: ClassVar[User]
     token: ClassVar[Token]
@@ -555,7 +648,6 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
     file: ClassVar[FileInArtifact]
 
     @classmethod
-    @context.disable_permission_checks()
     def setUpTestData(cls) -> None:
         """Set up the common test fixture."""
         super().setUpTestData()
@@ -593,16 +685,7 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
 
     def test_permissions(self) -> None:
         """Test basic permission enforcement."""
-        url = reverse(
-            "workspaces:artifacts:detail-file",
-            kwargs={
-                "wname": self.artifact.workspace.name,
-                "artifact_id": self.artifact.id,
-                "file_in_artifact_id": self.file.id,
-                "path": self.file.path,
-            },
-        )
-
+        url = self.file.get_absolute_url()
         self.assertSetsCurrentWorkspace(self.artifact.workspace, url)
         self.assertEnforcesPermission(
             self.artifact.can_display,
@@ -615,11 +698,10 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
         artifact_id = Artifact.objects.aggregate(Max("id"))['id__max'] + 1
         response = self.client.get(
             reverse(
-                "workspaces:artifacts:detail-file",
+                "workspaces:artifacts:file-detail",
                 kwargs={
                     "wname": self.playground.get_default_workspace().name,
                     "artifact_id": artifact_id,
-                    "file_in_artifact_id": self.file.id,
                     "path": self.file.path,
                 },
             ),
@@ -628,41 +710,17 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
         self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
         self.assertEqual(
             response.context["exception"],
-            "No file in artifact found matching the query",
-        )
-
-    def test_invalid_file_in_artifact_id(self) -> None:
-        """Test viewing a file_in_artifact ID that does not exist."""
-        file_in_artifact_id = (
-            FileInArtifact.objects.aggregate(Max("id"))['id__max'] + 1
-        )
-        response = self.client.get(
-            reverse(
-                "workspaces:artifacts:detail-file",
-                kwargs={
-                    "wname": self.artifact.workspace.name,
-                    "artifact_id": self.artifact.id,
-                    "file_in_artifact_id": file_in_artifact_id,
-                    "path": self.file.path,
-                },
-            ),
-            headers={"token": self.token.key},
-        )
-        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
-        self.assertEqual(
-            response.context["exception"],
-            "No file in artifact found matching the query",
+            "No FileInArtifact matches the given query.",
         )
 
     def test_invalid_file_in_artifact_path(self) -> None:
         """Test viewing a file_path that does not exist."""
         response = self.client.get(
             reverse(
-                "workspaces:artifacts:detail-file",
+                "workspaces:artifacts:file-detail",
                 kwargs={
                     "wname": self.artifact.workspace.name,
                     "artifact_id": self.artifact.id,
-                    "file_in_artifact_id": self.file.id,
                     "path": "invalid-path",
                 },
             ),
@@ -671,13 +729,13 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
         self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
         self.assertEqual(
             response.context["exception"],
-            "No file in artifact found matching the query",
+            "No FileInArtifact matches the given query.",
         )
 
     def test_get(self) -> None:
         """Test a simple get."""
         response = self.permission_tests_get(include_token=True)
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertResponseHTML(response)
         self.assertEqual(
             response.context["main_ui_shortcuts"],
             [
@@ -687,7 +745,6 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
             ],
         )
         self.assertIn("file", response.context)
-        tree = self.assertHTMLValid(response)
         title = self.assertHasElement(tree, "head/title")
         self.assertTextContentEqual(title, f"Debusine - {self.file.path}")
 
@@ -696,7 +753,7 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
         self.file.complete = False
         self.file.save()
         response = self.permission_tests_get(include_token=True)
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
+        tree = self.assertResponseHTML(response)
         self.assertEqual(
             response.context["main_ui_shortcuts"],
             [
@@ -706,15 +763,14 @@ class ArtifactDetailFileViewTests(ViewTestMixin, PermissionTests):
             ],
         )
         self.assertIn("file", response.context)
-        tree = self.assertHTMLValid(response)
         title = self.assertHasElement(tree, "head/title")
         self.assertTextContentEqual(
             title, f"Debusine - {self.file.path} (incomplete)"
         )
 
 
-class ArtifactDetailFileViewRawTests(ViewTestMixin, PermissionTests):
-    """Test ArtifactDetailFileRawView."""
+class FileDetailViewRawTests(ViewTestMixin, PermissionTests):
+    """Test FileDetailRawView."""
 
     playground_memory_file_store = False
 
@@ -771,59 +827,14 @@ class ArtifactDetailFileViewRawTests(ViewTestMixin, PermissionTests):
             "get",
         )
 
-    def test_invalid_artifact_id(self) -> None:
-        """Test viewing an artifact ID that does not exist."""
-        artifact_id = Artifact.objects.aggregate(Max("id"))['id__max'] + 1
-        response = self.client.get(
-            reverse(
-                "workspaces:artifacts:detail-file-raw",
-                kwargs={
-                    "wname": self.playground.get_default_workspace().name,
-                    "artifact_id": artifact_id,
-                    "file_in_artifact_id": self.file.id,
-                    "path": self.file.path,
-                },
-            ),
-            headers={"token": self.token.key},
-        )
-        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
-        self.assertEqual(
-            response.context["exception"],
-            "No FileInArtifact matches the given query.",
-        )
-
-    def test_invalid_file_in_artifact_id(self) -> None:
-        """Test viewing a file_in_artifact ID that does not exist."""
-        file_in_artifact_id = (
-            FileInArtifact.objects.aggregate(Max("id"))['id__max'] + 1
-        )
-        response = self.client.get(
-            reverse(
-                "workspaces:artifacts:detail-file-raw",
-                kwargs={
-                    "wname": self.artifact.workspace.name,
-                    "artifact_id": self.artifact.id,
-                    "file_in_artifact_id": file_in_artifact_id,
-                    "path": self.file.path,
-                },
-            ),
-            headers={"token": self.token.key},
-        )
-        self.assertEqual(response.status_code, status.HTTP_404_NOT_FOUND)
-        self.assertEqual(
-            response.context["exception"],
-            "No FileInArtifact matches the given query.",
-        )
-
     def test_invalid_file_in_artifact_path(self) -> None:
         """Test viewing a file_path that does not exist."""
         response = self.client.get(
             reverse(
-                "workspaces:artifacts:detail-file-raw",
+                "workspaces:artifacts:file-raw",
                 kwargs={
                     "wname": self.artifact.workspace.name,
                     "artifact_id": self.artifact.id,
-                    "file_in_artifact_id": self.file.id,
                     "path": "invalid-path",
                 },
             ),
@@ -904,7 +915,7 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
             headers["HTTP_TOKEN"] = self.token.key
         return self.client.get(
             reverse(
-                "workspaces:artifacts:download-path",
+                "workspaces:artifacts:file-download",
                 kwargs={
                     "wname": self.artifact.workspace.name,
                     "artifact_id": self.artifact.id,
@@ -934,7 +945,7 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
 
         return self.client.get(
             reverse(
-                "workspaces:artifacts:download-path",
+                "workspaces:artifacts:file-download",
                 kwargs={
                     "wname": self.artifact.workspace.name,
                     "artifact_id": artifact_id,
@@ -958,7 +969,7 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
         }
         viewname = "workspaces:artifacts:download"
         if subdirectory is not None:
-            viewname = "workspaces:artifacts:download-path"
+            viewname = "workspaces:artifacts:file-download"
             reverse_kwargs["path"] = subdirectory
 
         if archive is not None:
@@ -1029,7 +1040,7 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
     def test_permissions(self) -> None:
         """Test basic permission enforcement."""
         url = reverse(
-            "workspaces:artifacts:download-path",
+            "workspaces:artifacts:file-download",
             kwargs={
                 "wname": self.artifact.workspace.name,
                 "artifact_id": self.artifact.id,
@@ -1051,7 +1062,7 @@ class DownloadPathViewTests(ViewTestMixin, PermissionTests):
         instead of using Content-Disposition filename.
         """
         url = reverse(
-            "workspaces:artifacts:download-path",
+            "workspaces:artifacts:file-download",
             kwargs={"wname": "test", "artifact_id": 10, "path": "package.deb"},
         )
         self.assertFalse(url.endswith("/"))
@@ -1285,7 +1296,7 @@ class DownloadPathViewAuthTests(TestCase):
         }
 
         if subpath:
-            viewname = "workspaces:artifacts:download-path"
+            viewname = "workspaces:artifacts:file-download"
             reverse_kwargs["path"] = "README"
         else:
             viewname = "workspaces:artifacts:download"
