@@ -699,3 +699,44 @@ class AutopkgtestViewTests(TestCase):
                     AutopkgtestView._get_fail_on_scenarios(fail_on),
                     expected_scenarios,
                 )
+
+    def test_show_configured_task_data(self) -> None:
+        """The view shows configured task data if present."""
+        with context.disable_permission_checks():
+            environment_artifact, _ = self.playground.create_artifact(
+                paths=["system.tar.xz"],
+                category=ArtifactCategory.SYSTEM_TARBALL,
+                data={
+                    "vendor": "debian",
+                    "codename": "bookworm",
+                    "architecture": "amd64",
+                    "variant": "",
+                },
+                create_files=True,
+            )
+            source_artifact, _ = self.playground.create_artifact(
+                paths=["hello.dsc"],
+                category=ArtifactCategory.SOURCE_PACKAGE,
+                data={"name": "hello", "version": "1.0", "type": "dpkg"},
+                create_files=True,
+            )
+            work_request: WorkRequest = self.create_work_request_for_artifacts(
+                environment_artifact, source_artifact, []
+            )
+
+        # Try unconfigured
+        autopkgtest_view = AutopkgtestView(work_request)
+        context_data = autopkgtest_view.get_context_data()
+        self.assertEqual(
+            context_data["request_data"]["host_architecture"], "amd64"
+        )
+
+        # Try configured
+        work_request.configured_task_data = work_request.task_data.copy()
+        work_request.configured_task_data["host_architecture"] = "arm64"
+        work_request.save()
+        autopkgtest_view = AutopkgtestView(work_request)
+        context_data = autopkgtest_view.get_context_data()
+        self.assertEqual(
+            context_data["request_data"]["host_architecture"], "arm64"
+        )
