@@ -15,6 +15,7 @@ from django.utils.safestring import SafeString
 from debusine.db.models import Artifact, User, WorkRequest
 from debusine.db.playground import scenarios
 from debusine.test.django import TestCase
+from debusine.web.icons import Icons
 from debusine.web.views import sidebar
 from debusine.web.views.base import Widget
 from debusine.web.views.sidebar import SidebarAction, SidebarElement
@@ -198,3 +199,25 @@ class TestSidebar(TestCase):
         self.assertIsNone(el.value)
         el = sidebar.create_work_request_supersedes(wr_new)
         self.assertEqual(el.value, "1")
+
+
+class TestCreateWorker(TestCase):
+    """Test :py:func:`create_worker`."""
+
+    def test_none(self) -> None:
+        """Test with no worker."""
+        item = sidebar.create_worker(None)
+        self.assertEqual(item.label, "Worker")
+        self.assertEqual(item.icon, Icons.WORKER)
+        self.assertIsNone(item.value)
+        self.assertFalse(hasattr(item, "url"))
+
+    def test_worker(self) -> None:
+        """Test with a worker."""
+        worker = self.playground.create_worker()
+        item = sidebar.create_worker(worker)
+        assert isinstance(item, SidebarAction)
+        self.assertEqual(item.label, "Worker")
+        self.assertEqual(item.icon, Icons.WORKER)
+        self.assertEqual(item.value, worker.name)
+        self.assertEqual(item.url, worker.get_absolute_url())
