@@ -18,6 +18,7 @@ from debusine.db.playground import scenarios
 from debusine.server.status import TaskQueueSummary, WorkerSummary
 from debusine.tasks.models import WorkerType
 from debusine.test.django import TestCase
+from debusine.web.views.task_status import TaskStatusView
 from debusine.web.views.tests.utils import ViewTestMixin
 
 
@@ -31,6 +32,11 @@ class TaskStatusViewTests(ViewTestMixin, TestCase):
         super().setUp()
         self.factory = RequestFactory()
 
+    def test_get_title(self) -> None:
+        """Test get_title method."""
+        view = TaskStatusView()
+        self.assertEqual(view.get_title(), "Task queue")
+
     def test_template_no_work_requests(self) -> None:
         """Test template display the "No work requests..." message."""
         response = self.client.get(reverse("task-status"))
@@ -49,7 +55,7 @@ class TaskStatusViewTests(ViewTestMixin, TestCase):
             },
         ).mark_connected()
         response = self.client.get(reverse("task-status"))
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         arch_table = self.assertHasElement(
             tree, "//table[@id='task-arch-queue']"
         )
@@ -85,8 +91,7 @@ class TaskStatusViewTests(ViewTestMixin, TestCase):
         self.playground.create_work_request(task_name="sbuild")
 
         response = self.client.get(reverse("task-status"))
-
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
 
         type_table = self.assertHasElement(tree, "//table[@id='task-queue']")
         row = self.assertHasElement(
@@ -168,6 +173,7 @@ class TaskStatusViewTests(ViewTestMixin, TestCase):
                 "arch_status": arch_status,
                 "worker_status": worker_status,
                 "workspace": context.workspace,
+                "base_template": "web/_base.html",
             }
         )
 
