@@ -268,19 +268,17 @@ class WorkersListViewOrderingTests(ViewTestMixin, TestCase):
             actual.append(self.get_node_text_normalized(tr.td[1]))
         self.assertEqual(actual, expected)
 
-    def test_get_ordering_ascending(self) -> None:
-        """WorkersListView.get_ordering return correct ordering: "name"."""
-        response = self.client.get(
-            reverse("workers:list"), {"order": "name", "asc": "1"}
-        )
+    def test_order_ascending(self) -> None:
+        """Test ordering alphabetically."""
+        response = self.client.get(reverse("workers:list"), {"order": "name"})
         self.assertTableEntries(
             response, ["a-external", "b-celery", "c-signing"]
         )
 
-    def test_get_ordering_descending(self) -> None:
-        """WorkersListView.get_ordering return correct ordering: "-name"."""
+    def test_order_descending(self) -> None:
+        """Test ordering reverse-alphabetically."""
         response = self.client.get(
-            reverse("workers:list"), {"order": "name", "asc": "0"}
+            reverse("workers:list"), {"order": "name.desc"}
         )
         self.assertTableEntries(
             response, ["c-signing", "b-celery", "a-external"]
@@ -304,7 +302,9 @@ class WorkerDetailViewTests(ViewTestMixin, TestCase):
         el = self.assertHasElement(tree, "//tr[@id='row-worker_type']")
         self.assertTextContentEqual(el.td[0], worker.worker_type)
         el = self.assertHasElement(tree, "//tr[@id='row-registered_at']")
-        self.assertDatetimeContentEqual(el.td[0], worker.registered_at)
+        self.assertDatetimeContentEqual(
+            el.td[0], worker.registered_at, anywhere=True
+        )
         el = self.assertHasElement(tree, "//tr[@id='row-connected_at']")
         self.assertTextContentEqual(el.td[0], "never")
         el = self.assertHasElement(tree, "//tr[@id='row-instance_created_at']")
@@ -322,7 +322,9 @@ class WorkerDetailViewTests(ViewTestMixin, TestCase):
         )
         assert worker.dynamic_metadata_updated_at is not None
         self.assertDatetimeContentEqual(
-            footer.xpath("//em")[0], worker.dynamic_metadata_updated_at
+            footer.xpath("//em")[0],
+            worker.dynamic_metadata_updated_at,
+            anywhere=True,
         )
 
         self.assertFalse(tree.xpath("//tr[@id='row-worker_pool']"))
@@ -345,20 +347,28 @@ class WorkerDetailViewTests(ViewTestMixin, TestCase):
         tree = self.assertResponseHTML(response)
 
         el = self.assertHasElement(tree, "//tr[@id='row-registered_at']")
-        self.assertDatetimeContentEqual(el.td[0], worker.registered_at)
+        self.assertDatetimeContentEqual(
+            el.td[0], worker.registered_at, anywhere=True
+        )
 
         el = self.assertHasElement(tree, "//tr[@id='row-connected_at']")
-        self.assertDatetimeContentEqual(el.td[0], worker.connected_at)
+        self.assertDatetimeContentEqual(
+            el.td[0], worker.connected_at, anywhere=True
+        )
 
         el = self.assertHasElement(tree, "//tr[@id='row-instance_created_at']")
-        self.assertDatetimeContentEqual(el.td[0], worker.instance_created_at)
+        self.assertDatetimeContentEqual(
+            el.td[0], worker.instance_created_at, anywhere=True
+        )
 
         el = self.assertHasElement(tree, "//tr[@id='row-metadata']")
         footer = self.assertHasElement(
             el, ".//div[contains(@class, 'card-footer')]"
         )
         self.assertDatetimeContentEqual(
-            footer.xpath("//em")[0], worker.dynamic_metadata_updated_at
+            footer.xpath("//em")[0],
+            worker.dynamic_metadata_updated_at,
+            anywhere=True,
         )
 
     def test_no_dynamic_metadata_updated_at(self) -> None:
@@ -426,8 +436,7 @@ class WorkerDetailViewTests(ViewTestMixin, TestCase):
         self.client.force_login(self.scenario.user)
         with override_permission(WorkRequest, "can_display", AllowAll):
             response = self.client.get(worker.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         table = self.assertHasElement(tree, "//table[@id='work_request-list']")
         self.assertEqual(len(table.tbody.tr), 1)
 
@@ -442,6 +451,5 @@ class WorkerDetailViewTests(ViewTestMixin, TestCase):
         self.client.force_login(self.scenario.user)
         with override_permission(WorkRequest, "can_display", DenyAll):
             response = self.client.get(worker.get_absolute_url())
-        self.assertEqual(response.status_code, status.HTTP_200_OK)
-        tree = self.assertHTMLValid(response)
+        tree = self.assertResponseHTML(response)
         self.assertFalse(tree.xpath("//table[@id='work_request-list']"))
