@@ -9,18 +9,18 @@
 
 """Utility code for testing views."""
 
-import io
 import re
 from collections.abc import Callable
 from datetime import datetime
-from typing import Any, TypeVar, cast
+from typing import Any, TypeVar
 from unittest import mock
 
 import lxml.etree
 import lxml.objectify
 from django.contrib.auth.models import AnonymousUser
 from django.http import HttpRequest
-from django.test import RequestFactory
+from django.template import engines
+from django.test import RequestFactory, override_settings
 from django.urls import resolve, reverse
 from django.utils.formats import date_format as django_date_format
 from django.views.generic.base import View
@@ -33,10 +33,9 @@ from debusine.test.django import (
     AllowAll,
     BaseDjangoTestCase,
     DenyAll,
-    TestResponseType,
     override_permission,
 )
-from debusine.web.utils import ui_prototype
+from debusine.web.views.base import Widget
 
 ViewClass = TypeVar("ViewClass", bound=View)
 
@@ -44,11 +43,6 @@ ViewClass = TypeVar("ViewClass", bound=View)
 class ViewTestMixin(BaseDjangoTestCase):
     """TestCase functions used to test Debusine views."""
 
-    # LXML does not seem to know about HTML5 structural tags
-    re_lxml_false_positive_tags = re.compile(
-        r"Tag (?:nav|footer|header|article) invalid"
-    )
-
     def make_request(self, url: str) -> HttpRequest:
         """Create a request that can be manipulated before invoking a view."""
         factory = RequestFactory()
@@ -77,63 +71,6 @@ class ViewTestMixin(BaseDjangoTestCase):
         view.setup(request, **kwargs)
         return view
 
-    def _filter_parser_error_log(
-        self, error_log: lxml.etree._ListErrorLog
-    ) -> list[str]:
-        """Filter lxml parser error log for known false positives."""
-        # _LogEntry documentation:
-        # https://lxml.de/apidoc/lxml.etree.html#lxml.etree._LogEntry
-        Domains = lxml.etree.ErrorDomains
-
-        errors: list[str] = []
-        for error in error_log:
-            match (error.domain, error.type_name):
-                case Domains.HTML, "HTML_UNKNOWN_TAG":
-                    if self.re_lxml_false_positive_tags.match(error.message):
-                        continue
-                    # Without this pass, python coverage is currently unable to
-                    # detect that code does flow through here
-                    pass
-
-            errors.append(f"{error.line}:{error.type_name}:{error.message}")
-        return errors
-
-    def assertHTMLValid(
-        self, response: TestResponseType
-    ) -> lxml.objectify.ObjectifiedElement:
-        """
-        Parse the response contents as HTML and ensure it is valid.
-
-        Returns the parsed tree.
-        """
-        parser = lxml.etree.HTMLParser(remove_blank_text=True)
-        parser.set_element_class_lookup(
-            lxml.objectify.ObjectifyElementClassLookup()
-        )
-        with io.BytesIO(response.content) as fd:
-            root = lxml.etree.parse(fd, parser)
-        errors = self._filter_parser_error_log(parser.error_log)
-        self.assertEqual(errors, [])
-        return cast(lxml.objectify.ObjectifiedElement, root)
-
-    def assertHasElement(
-        self, tree: lxml.objectify.ObjectifiedElement, xpath: str
-    ) -> lxml.objectify.ObjectifiedElement:
-        """
-        Ensure that the tree has strictly one matching element.
-
-        :param tree: tree to search
-        :param xpath: XPath query to match
-        :returns: resulting element
-        """
-        el = tree.xpath(xpath)
-        if not el:
-            self.fail(f"{xpath!r} not found in tree")
-        if len(el) > 1:
-            self.fail(f"{xpath!r} matched {len(el)} elements instead of one")
-        assert isinstance(el[0], lxml.objectify.ObjectifiedElement)
-        return el[0]
-
     @staticmethod
     def _normalize_node(node: lxml.objectify.ObjectifiedElement) -> str:
         """Normalize the HTML to ignore spaces and new lines."""
@@ -166,31 +103,6 @@ class ViewTestMixin(BaseDjangoTestCase):
 
         self.assertEqual(normalized_html_node, normalized_html_expected)
 
-    def get_node_text_normalized(
-        self, node: lxml.objectify.ObjectifiedElement
-    ) -> str:
-        """Get the node text contents, with spaces normalized."""
-        # itertext will iterate on whitespace-only blocks, so it needs two
-        # passes: one to reconstruct the text, and one to normalise whitespace
-        sample = "".join(node.itertext())
-        return " ".join(sample.strip().split())
-
-    def assertTextContentEqual(
-        self,
-        node: lxml.objectify.ObjectifiedElement,
-        text: str,
-    ) -> None:
-        """
-        Ensure that node.text matches the given text.
-
-        Both expected and actual text are normalised so that consecutive
-        whitespace and newlines become a single space, to simplify dealing with
-        the way HTML collapses whitespace.
-        """
-        sample = self.get_node_text_normalized(node)
-        text = " ".join(text.strip().split())
-        self.assertEqual(sample, text)
-
     def assertWorkRequestRow(
         self, tr: lxml.objectify.ObjectifiedElement, work_request: WorkRequest
     ) -> None:
@@ -216,16 +128,6 @@ class ViewTestMixin(BaseDjangoTestCase):
             self.fail("page has no workspace list table")
         return list(table[0].tbody.tr)
 
-    @ui_prototype
-    def collection_list_table_rows(
-        self, tree: lxml.etree._Element
-    ) -> list[lxml.objectify.ObjectifiedElement]:
-        """Find the collection list table in the page and return it."""
-        table = tree.xpath("//table[@id='collection-list-table']")
-        if not table:
-            self.fail("page has no collection list table")
-        return list(table[0].tbody.tr)
-
     def assertSetsCurrentWorkspace(
         self,
         workspace: Workspace,
@@ -273,6 +175,7 @@ class ViewTestMixin(BaseDjangoTestCase):
         url: str,
         target_name: str,
         method: str = "get",
+        error_code: int = status.HTTP_403_FORBIDDEN,
         **kwargs: Any,
     ) -> None:
         """
@@ -320,7 +223,7 @@ class ViewTestMixin(BaseDjangoTestCase):
             ),
         ):
             response = getattr(self.client, method)(url, **kwargs)
-            self.assertEqual(response.status_code, status.HTTP_403_FORBIDDEN)
+            self.assertEqual(response.status_code, error_code)
         target.assert_not_called()
 
     def assertNavCommonElements(
@@ -454,6 +357,30 @@ class ViewTestMixin(BaseDjangoTestCase):
         )
         self.assertEqual(ul.li[-1].form.get("action"), reverse("logout"))
 
+    def assertNavHasPlumbing(
+        self, tree: lxml.objectify.ObjectifiedElement
+    ) -> None:
+        """Check that the page navbar has "Plumbing" drop down."""
+        a = self.assertHasElement(tree, "//a[@id='navbar-plumbing-actions']")
+        self.assertTextContentEqual(a, "Plumbing")
+
+    def assertNavNoPlumbing(
+        self, tree: lxml.objectify.ObjectifiedElement
+    ) -> None:
+        """Check that the page navbar has no "Plumbing" drop down."""
+        self.assertFalse(tree.xpath("//a[@id='navbar-plumbing-actions']"))
+
+    def render_widget(self, widget: Widget) -> str:
+        """Render a widget."""
+        template = engines["django"].from_string(
+            "{% load debusine %}{% widget widget %}"
+        )
+        setattr(template, "engine", engines["django"])
+        # Set DEBUG=True to have render raise exceptions instead of logging
+        # them
+        with override_settings(DEBUG=True):
+            return template.render({"widget": widget})
+
 
 def html_check_icon(value: bool) -> str:
     """Return HTML for check icon."""
