@@ -15,7 +15,12 @@ from django.db.models import Count, QuerySet
 
 from debusine.db.context import context
 from debusine.db.models import Collection, WorkflowTemplate, Workspace
-from debusine.web.views.base import DetailViewBase, WorkspaceView
+from debusine.web.forms import WorkspaceForm
+from debusine.web.views.base import (
+    DetailViewBase,
+    UpdateViewBase,
+    WorkspaceView,
+)
 
 
 class WorkspaceDetailView(WorkspaceView, DetailViewBase[Workspace]):
@@ -59,3 +64,31 @@ class WorkspaceDetailView(WorkspaceView, DetailViewBase[Workspace]):
         ctx["collection_count"] = sum(s["count"] for s in stats)
 
         return ctx
+
+
+class WorkspaceUpdateView(
+    WorkspaceView, UpdateViewBase[Workspace, WorkspaceForm]
+):
+    """Configure a workspace."""
+
+    form_class = WorkspaceForm
+    template_name = "web/workspace-update.html"
+
+    def init_view(self) -> None:
+        """Set the current workspace."""
+        super().init_view()
+        # Enforced by WorkspaceView.init()
+        assert context.workspace is not None
+        self.enforce(context.workspace.can_configure)
+
+    def get_object(
+        self, queryset: QuerySet[Workspace] | None = None  # noqa: U100
+    ) -> Workspace:
+        """Return the workspace object to show."""
+        # Enforced by WorkspaceView.init()
+        assert context.workspace is not None
+        return context.workspace
+
+    def get_title(self) -> str:
+        """Return the page title."""
+        return f"Configure workspace {self.object.name}"
