@@ -272,7 +272,7 @@ class Worker:
 
         try:
             response = await self._async_http_client.post(
-                register_path, json=data
+                register_path, json=data, token=self._config.activation_token
             )
         except aiohttp.client_exceptions.ClientResponseError as err:
             self._log_server_error(err.status, 'Not available')
@@ -443,7 +443,7 @@ class Worker:
             logging.info("Connected to %s", self._config.api_url)
 
         async def work_request_available(
-            msg_content: dict[Any, Any]  # noqa: U100
+            msg_content: dict[Any, Any],  # noqa: U100
         ) -> None:
             await self._request_and_execute_work_request()
 
