@@ -165,10 +165,10 @@ class ConfigHandlerTests(TestCase):
     @classmethod
     @contextlib.contextmanager
     def _temporary_directory_with_token_file(
-        cls, token: str
+        cls, token: str, token_filename: str = "token"
     ) -> Generator[str, None, None]:
         with cls._temporary_config_directory() as temp_directory:
-            token_path = os.path.join(temp_directory, 'token')
+            token_path = os.path.join(temp_directory, token_filename)
             fd = os.open(token_path, os.O_WRONLY | os.O_CREAT, mode=0o600)
             os.write(fd, token.encode('utf-8'))
             os.close(fd)
@@ -179,10 +179,19 @@ class ConfigHandlerTests(TestCase):
         """ConfigHandler.token returns the token."""
         token = 'a84b92e3c3379733bf0d2b25700c4c'
 
-        with self._temporary_directory_with_token_file(token) as temp_directory:
-            config = ConfigHandler(directories=[temp_directory])
+        for prop, token_filename in (
+            ("token", "token"),
+            ("activation_token", "activation-token"),
+        ):
+            with (
+                self.subTest(prop=prop),
+                self._temporary_directory_with_token_file(
+                    token, token_filename=token_filename
+                ) as temp_directory,
+            ):
+                config = ConfigHandler(directories=[temp_directory])
 
-            self.assertEqual(config.token, token)
+                self.assertEqual(getattr(config, prop), token)
 
     def test_token_strip_new_line(self) -> None:
         """
@@ -190,61 +199,90 @@ class ConfigHandlerTests(TestCase):
 
         If the token file had a newline after the token: it's stripped.
         """
-        with self._temporary_config_directory() as temp_directory:
-            token = 'a84b92e3c3379733bf0d2b25700c4c'
-            with open(os.path.join(temp_directory, 'token'), 'w') as token_file:
-                token_file.write(token + '\n')
+        for prop, token_filename in (
+            ("token", "token"),
+            ("activation_token", "activation-token"),
+        ):
+            with (
+                self.subTest(prop=prop),
+                self._temporary_config_directory() as temp_directory,
+            ):
+                token = 'a84b92e3c3379733bf0d2b25700c4c'
+                with open(
+                    os.path.join(temp_directory, token_filename), 'w'
+                ) as token_file:
+                    token_file.write(token + '\n')
 
-                Path(token_file.name).chmod(0o600)
+                    Path(token_file.name).chmod(0o600)
 
-                config = ConfigHandler(directories=[temp_directory])
+                    config = ConfigHandler(directories=[temp_directory])
 
-            self.assertEqual(config.token, token)
+                self.assertEqual(getattr(config, prop), token)
 
     def test_token_file_does_not_exist(self) -> None:
         """ConfigHandler.token returns None for non-existing file."""
-        with self._temporary_config_directory() as temp_directory:
-            config = ConfigHandler(directories=[temp_directory])
-            # config.token access the token file
-            self.assertIsNone(config.token)
+        for prop in ("token", "activation_token"):
+            with (
+                self.subTest(prop=prop),
+                self._temporary_config_directory() as temp_directory,
+            ):
+                config = ConfigHandler(directories=[temp_directory])
+                # The property accesses the token file.
+                self.assertIsNone(getattr(config, prop))
 
     def test_token_file_cannot_be_read(self) -> None:
         """Token file cannot be read (not enough permissions)."""
-        with self._temporary_config_directory() as temp_directory:
-            token_file = os.path.join(temp_directory, 'token')
-            os.mkdir(token_file)
-            Path(token_file).chmod(0o000)
-
-            config = ConfigHandler(directories=[temp_directory])
-
+        for prop, token_filename in (
+            ("token", "token"),
+            ("activation_token", "activation-token"),
+        ):
             with (
-                self.assertLogsContains('Cannot read'),
-                self.assertRaisesSystemExit(3),
+                self.subTest(prop=prop),
+                self._temporary_config_directory() as temp_directory,
             ):
-                config.token
+                token_file = os.path.join(temp_directory, token_filename)
+                os.mkdir(token_file)
+                Path(token_file).chmod(0o000)
+
+                config = ConfigHandler(directories=[temp_directory])
+
+                with (
+                    self.assertLogsContains('Cannot read'),
+                    self.assertRaisesSystemExit(3),
+                ):
+                    getattr(config, prop)
 
     def test_token_too_open_fails(self) -> None:
         """ConfigHandler.token aborts: too open permissions."""
-        with self._temporary_config_directory() as temp_directory:
-            token = 'a84b92e3c3379733bf0d2b25700c4c'
-            with open(os.path.join(temp_directory, 'token'), 'w') as token_file:
-                token_file.write(token)
-
-                Path(token_file.name).chmod(0o640)
-
-            config = ConfigHandler(directories=[temp_directory])
-
-            log_message = (
-                f'Permission too open for {token_file.name}. '
-                'Make sure that the file is not accessible '
-                'by group or others'
-            )
-
+        for prop, token_filename in (
+            ("token", "token"),
+            ("activation_token", "activation-token"),
+        ):
             with (
-                self.assertLogsContains(log_message),
-                self.assertRaisesSystemExit(3),
+                self.subTest(prop=prop),
+                self._temporary_config_directory() as temp_directory,
             ):
-                config.token
+                token = 'a84b92e3c3379733bf0d2b25700c4c'
+                with open(
+                    os.path.join(temp_directory, token_filename), 'w'
+                ) as token_file:
+                    token_file.write(token)
+
+                    Path(token_file.name).chmod(0o640)
+
+                config = ConfigHandler(directories=[temp_directory])
+
+                log_message = (
+                    f'Permission too open for {token_file.name}. '
+                    'Make sure that the file is not accessible '
+                    'by group or others'
+                )
+
+                with (
+                    self.assertLogsContains(log_message),
+                    self.assertRaisesSystemExit(3),
+                ):
+                    getattr(config, prop)
 
     def test_token_is_cached(self) -> None:
         """ConfigHandler.token caches the result."""
@@ -258,9 +296,9 @@ class ConfigHandlerTests(TestCase):
                 token_file.write('new-token\n')
                 Path(token_file.name).chmod(0o600)
 
-            # config.token return the first token because it is cached
+            # config.token returns the first token because it is cached
             # (and it was not written via ConfigHandler.write_token which
-            # clears the cache)
+            # clears the cache).
             self.assertEqual(config.token, token)
 
     def test_write_token_clean_cache(self) -> None:
@@ -339,7 +377,7 @@ class ConfigHandlerTests(TestCase):
 
     def test_validate_config_or_fail(self) -> None:
         """
-        validate_config_or_fail() access Config.api_url and .token.
+        validate_config_or_fail() accesses Config properties.
 
         There are separate tests for the behaviour accessing the properties
         (to call Config._fail). Not re-implemented in this test to avoid
@@ -349,11 +387,15 @@ class ConfigHandlerTests(TestCase):
             config = ConfigHandler(directories=[temp_directory])
 
             mocked_token = self._mock_config_handle_property('token')
+            mocked_activation_token = self._mock_config_handle_property(
+                'activation_token'
+            )
             debusine_url = self._mock_config_handle_property('api_url')
 
             config.validate_config_or_fail()
 
             mocked_token.assert_called_once_with()
+            mocked_activation_token.assert_called_once_with()
             debusine_url.assert_called_once_with()
 
     def test_etc_config_lower_priority(self) -> None:
