@@ -13,6 +13,7 @@ import asyncio
 import concurrent.futures
 import functools
 import logging
+import secrets
 import signal
 import socket
 import threading
@@ -43,7 +44,6 @@ from debusine.client.models import (
     model_to_json_serializable_dict,
 )
 from debusine.tasks import (
-    BaseExternalTask,
     BaseTask,
     BaseTaskWithExecutor,
     Noop,
@@ -60,9 +60,9 @@ from debusine.tasks.models import (
     TaskTypes,
     WorkerType,
 )
-from debusine.tasks.tests.helper_mixin import TestTaskMixin
+from debusine.tasks.tests.helper_mixin import TestBaseExternalTask, TestBaseTask
 from debusine.test import TestCase
-from debusine.test.utils import create_artifact_response
+from debusine.test.test_utils import create_artifact_response
 from debusine.worker import Worker
 from debusine.worker.config import ConfigHandler
 from debusine.worker.system_information import (
@@ -264,6 +264,45 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
                 'worker_type': WorkerType.EXTERNAL,
             },
         )
+        assert registration_request.headers is not None
+        self.assertNotIn('token', registration_request.headers)
+
+    def test_registration_success_activation_token(self) -> None:
+        """Worker client registers successfully with an activation token."""
+        activation_token = secrets.token_hex(32)
+        activation_token_path = Path(self.config._activation_token_file)
+        activation_token_path.write_bytes(activation_token.encode())
+        activation_token_path.chmod(0o600)
+        self.assertEqual(self.config.activation_token, activation_token)
+
+        # No token in the configuration
+        self.assertIsNone(self.config.token)
+
+        # Worker connects: it will create a token, send and save
+        self.setup_worker(connect=True)
+
+        # A token has been retrieved
+        self.assertIsNotNone(self.config.token)
+
+        registration_request = self.server_requests[-2]
+        connect_request = self.server_requests[-1]
+
+        self.assertEqual(registration_request.path, '/api/1.0/worker/register/')
+        self.assertEqual(connect_request.path, '/api/ws/1.0/worker/connect/')
+
+        # Server side received expected registration data
+        self.assertEqual(
+            registration_request.json_content,
+            {
+                'token': self.config.token,
+                'fqdn': socket.getfqdn(),
+                'worker_type': WorkerType.EXTERNAL,
+            },
+        )
+        assert registration_request.headers is not None
+        self.assertEqual(
+            registration_request.headers['token'], activation_token
+        )
 
     def test_registration_failure(self) -> None:
         """
@@ -1064,8 +1103,7 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             pass
 
         class ServerTest(
-            TestTaskMixin,
-            BaseTask[ServerTestData, task_models.BaseDynamicTaskData],
+            TestBaseTask[ServerTestData, task_models.BaseDynamicTaskData],
         ):
             TASK_VERSION = 1
             TASK_TYPE = TaskTypes.SERVER
@@ -1113,8 +1151,9 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             pass
 
         class SigningTest(
-            TestTaskMixin,
-            BaseExternalTask[SigningTestData, task_models.BaseDynamicTaskData],
+            TestBaseExternalTask[
+                SigningTestData, task_models.BaseDynamicTaskData
+            ],
         ):
             TASK_VERSION = 1
             TASK_TYPE = TaskTypes.SIGNING
@@ -1190,8 +1229,9 @@ class WorkerTests(TestCase, server.DebusineAioHTTPTestCase):
             pass
 
         class SigningTest(
-            TestTaskMixin,
-            BaseExternalTask[SigningTestData, task_models.BaseDynamicTaskData],
+            TestBaseExternalTask[
+                SigningTestData, task_models.BaseDynamicTaskData
+            ],
         ):
             TASK_VERSION = 1
             TASK_TYPE = TaskTypes.SIGNING
